/*
 Linear Technology DC2343A Demonstration Board.
 Reference Application File for Controlling the LTC3335 Nanopower Buck-Boost DC/DC with Integrated Coulomb Counter on the DC2343A PCB.

 @verbatim
 The PIC16 FW identification and HW definition are contained in these files.
 @endverbatim

 http://www.linear.com/product/LTC3335#demoboards

 REVISION HISTORY
 $Revision: 847 $
 $Date: 2014-11-06 15:21:01 -0500 (Thu, 06 Nov 2014) $

 Copyright (c) 2014, Linear Technology Corp.(LTC)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 The views and conclusions contained in the software and documentation are those
 of the authors and should not be interpreted as representing official policies,
 either expressed or implied, of Linear Technology Corp.

 */

#ifndef __DC2343_H__
#define __DC2343_H__

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Includes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include <xc.h>

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Definitions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Definitions specific to the DC2343A board
#define DC2343_FW_REV_MAJOR            0x01
#define DC2343_FW_REV_MINOR            0x00
#define DC2343_FW_REV_SUB_MINOR        0x02
#define DC2343_FW_REV_BETA             0x00

typedef struct
{
    char rev_major;
    char rev_minor;
    char rev_sub_minor;
    char rev_beta;
} DC2343_FW_REV_TYPE;

// IDSTRING that the board sends back to the host
#define DC2343_IDSTRING            "DC2343,LTC3335 demonstration board"
#define DC2343_IDSTRING_SIZE       (sizeof(DC2343_IDSTRING) - 1)
#define DC2343_COMPANY_STRING      "Linear Technology Inc."
#define DC2343_COMPANY_STRING_SIZE (sizeof(DC2343_COMPANY_STRING) - 1)

#define DC2343_MODEL_NUM_DEFAULT   "DC2343"
#define DC2343_MODEL_NUM_SIZE      (sizeof(DC2343_MODEL_NUM_DEFAULT) - 1)  // The size of the model number string (non-unicode), no null terminator
#define DC2343_SERIAL_NUM_DEFAULT  "None             "
#define DC2343_SERIAL_NUM_SIZE     (sizeof(DC2343_SERIAL_NUM_DEFAULT) - 1) // The size of the serial number string (non-unicode), no null terminator
#define DC2343_NONSTANDARD_INDUCTOR_DEFAULT  ""
#define DC2343_NONSTANDARD_INDUCTOR_SIZE     16                            // The size of the nonstandard inductor string (non-unicode), no null terminator

// DC2343A HW definition
#define DC2343_OSCILLATOR_FREQUENCY    48000000                            // Oscillator frequency for DC2343A board
#define DC2343_INSTRUCTION_CLOCK       (DC2343_OSCILLATOR_FREQUENCY / 4)   // Instruction frequency derived from oscillator
// Configure clock to operate at DC2343_OSCILLATOR_FREQUENCY and sync to USB
// HFINTOSC @ 16MHz, 3X PLL, PLL enabled
// Active clock tuning enabled for USB
// Wait until internal oscillator and PLL are ready (~2ms from table 5-3 of datasheet).
#define DC2343_CONFIGURE_CLOCK      {OSCCON = 0xFC; ACTCON = 0x90; while((OSCSTATbits.HFIOFR == 0) || (OSCSTATbits.PLLRDY == 0));}

// Global Interrupt Control
#define DC2343_ENABLE_INTERRUPTS    {INTCONbits.GIE = 1;}
#define DC2343_DISABLE_INTERRUPTS   {INTCONbits.GIE = 0;}

// DC2343A I/O definition
#define LED_GREEN_OUT_PIN           LATCbits.LATC4
#define LED_GREEN_OUT_PIN_SETUP     {LED_GREEN_OUT_PIN = 0; TRISCbits.TRISC4 = 0;}

#define LED_AMBER_OUT_PIN           LATCbits.LATC3
#define LED_AMBER_OUT_PIN_SETUP     {LED_AMBER_OUT_PIN = 0; TRISCbits.TRISC3 = 0;}

#define I2C_SCL_OUT_PIN             LATBbits.LATB6
#define I2C_SCL_OUT_PIN_SETUP       {I2C_SCL_OUT_PIN; TRISBbits.TRISB6 = 0;}

#define I2C_SDA_OUT_PIN             LATBbits.LATB4
#define I2C_SDA_OUT_PIN_SETUP       {I2C_SDA_OUT_PIN = 1; TRISBbits.TRISB4 = 0;}

#define I2C_SDA_IN_PIN              PORTBbits.RB4
#define I2C_SDA_IN_PIN_SETUP        {TRISBbits.TRISB4 = 1;}

#define I2C_PIC_HW_SETUP            {TRISBbits.TRISB4 = 1; TRISBbits.TRISB6 = 1; ANSELB = 0;}

#define IRQ_IN_PIN                  PORTCbits.RC5
#define IRQ_IN_PIN_SETUP            {TRISCbits.TRISC5 = 1;}

#define DVCC_OUT_PIN                PORTBbits.RB7
#define DVCC_OUT_PIN_SETUP          {DVCC_OUT_PIN = 1; TRISBbits.TRISB7 = 0;}

#define PGOOD_IN_PIN                PORTCbits.RC6
#define PGOOD_IN_PIN_SETUP          {TRISCbits.TRISC6 = 1; ANSELCbits.ANSC6 = 0;}

#define ENABLE_ADC_OUT_PIN          LATCbits.LATC7
#define ENABLE_ADC_OUT_PIN_SETUP    {ENABLE_ADC_OUT_PIN = 0; TRISCbits.TRISC7 = 0;}

#define VBAT_ADC_PIN                PORTCbits.RB5
#define VBAT_ADC_PIN_SETUP          {TRISBbits.TRISB5 = 1; ANSELBbits.ANSB5 = 1;}
#define VBAT_ADC_CHANNEL            (11)

#define VOUT_ADC_PIN                PORTCbits.RC2
#define VOUT_ADC_PIN_SETUP          {TRISCbits.TRISC2 = 1; ANSELCbits.ANSC2 = 1;}
#define VOUT_ADC_CHANNEL            (6)

typedef enum {
    DC2343_EN_CONFIGURATION_ON,
    DC2343_EN_CONFIGURATION_EXT,
    DC2343_EN_CONFIGURATION_OFF,
    DC2343_NUM_EN_CONFIGURATIONS,
} DC2343_EN_CONFIGURATION_TYPE;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Prototypes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

#endif
