﻿Option Strict On
Option Explicit On

Imports System.Drawing
Imports System.Windows.Forms
Imports Microsoft.VisualBasic
Imports Microsoft.VisualBasic.PowerPacks

Public Class Battery_Coulomb_Counter_Graphic

    Private Const MAX_LINES As Integer = 256    ' Maximum number of lines in the battery control
    Private Const BORDER_LINE_WIDTH As Integer = 4
    Private Const CONTROL_LINE_WIDTH As Integer = 3
    Private Const EDGE_MARGIN As Integer = 8
    Private Const MID_MARGIN As Integer = 24
    Private Const LABEL_OFFSET As Integer = 10
    Private Const TOOLTIP_OFFSET As Integer = -10
    Private Const POPUP_TIMEOUT As Integer = 1000
    Private Const TOOLTIP_TIME_ON As Integer = 1000
    Private Const TOOLTIP_TIME_OFF As Integer = 3000

    Private Const SCALE_TICKS As Integer = 8          ' Divide scale into SCALE_TICKS ticks + 1 tick at 0
    Private Const SCALE_MAJOR_TICKS As Integer = 4    ' Divide scale into major SCALE_MAJOR_TICKS ticks + 1 tick at 0
    Private Const SCALE_TICK_WIDTH As Integer = 1
    Private Const SCALE_TICK_LENGTH As Integer = 2
    Private Const SCALE_MAJOR_TICK_WIDTH As Integer = 3
    Private Const SCALE_MAJOR_TICK_LENGTH As Integer = (2 * SCALE_TICK_LENGTH)

    Private Const PERCENT_DISCHARGED_FORMAT As String = "#0.0"
    Private Const COULOMBS_FORMAT As String = "0"
    Private OK_COLOR As Color = SystemColors.WindowText
    Private ERROR_COLOR As Color = Color.Red

    Private _resolution As Single         ' Ah per line
    Private _battery_capacity_ah As Double
    Private _battery_coulombs_discharged_ah As Double
    Private _alarm_ah As Double
    Private _use_software_correction As Boolean
    Private _software_correction_factor As Single
    Private maintain_percent As Boolean     ' This indicates if the red and green lines should scale as the black line is dragged to maintain their original %
    Private debug_textbox As TextBox

    Private battery_capacity_dragging As Boolean
    Private battery_coulombs_discharged_dragging As Boolean
    Private alarm_dragging As Boolean

    ' Create properties so that Forms can read values but not write them.
    Public Property resolution() As Single
        Get
            If (Me.use_software_correction = False) Then
                Return _resolution
            Else
                Return _resolution / (1 + Me.software_correction_factor / 100)
            End If
        End Get
        Private Set(ByVal value As Single)
            _resolution = value
        End Set
    End Property
    Public ReadOnly Property range() As Single
        Get
            Return (MAX_LINES - 1) * resolution
        End Get
    End Property
    Public Property battery_capacity_ah() As Double
        Get
            Return _battery_capacity_ah
        End Get
        Private Set(ByVal value As Double)
            _battery_capacity_ah = value
        End Set
    End Property
    Public Property battery_coulombs_discharged_ah() As Double
        Get
            Return _battery_coulombs_discharged_ah
        End Get
        Private Set(ByVal value As Double)
            _battery_coulombs_discharged_ah = value
        End Set
    End Property
    Public Property alarm_ah() As Double
        Get
            Return _alarm_ah
        End Get
        Private Set(ByVal value As Double)
            _alarm_ah = value
        End Set
    End Property
    Public Property use_software_correction() As Boolean
        Get
            Return _use_software_correction
        End Get
        Private Set(ByVal value As Boolean)
            _use_software_correction = value
        End Set
    End Property
    Public Property software_correction_factor() As Single
        Get
            Return _software_correction_factor
        End Get
        Private Set(ByVal value As Single)
            _software_correction_factor = value
        End Set
    End Property

    ' These are the controls that are passed in, to locate the battery control and its popup label in the main form.
    Private WithEvents my_panel As Panel
    Friend popup_label As Label
    Friend tooltip As ToolTip
    Friend WithEvents terminal_picturebox As PictureBox    ' The terminal picturebox is located in the main panel, to know where inside the panel to locate the battery.

    ' This is the container used to contain the shapes in the coulomb counter and the battery graphics.
    Friend ShapeContainer As ShapeContainer

    ' This is the container used to contain the lines in the legend that should never need to be redrawn.
    Friend Legend_ShapeContainer As ShapeContainer

    ' These are the objects created to represent the battery
    Friend WithEvents gnd_pictureBox As PictureBox
    Friend WithEvents Battery_Top_LineShape As LineShape
    Friend WithEvents Battery_Bottom_LineShape As LineShape
    Friend WithEvents Battery_Left_LineShape As LineShape
    Friend WithEvents Battery_Right_LineShape As LineShape
    Friend WithEvents Battery_Capacity_RectangleShape As RectangleShape
    Friend WithEvents Battery_Coulombs_Discharged_RectangleShape As RectangleShape
    Friend WithEvents Battery_Coulombs_Discharged_LineShape As LineShape
    Friend WithEvents Battery_Alarm_LineShape As LineShape
    Friend WithEvents Battery_Drag_Extents_RectangleShape As RectangleShape
    Friend Battery_Scale_Ticks_LineShape(SCALE_TICKS) As LineShape
    Friend Battery_Scale_Ticks_Label(SCALE_MAJOR_TICKS) As Label

    ' These are the objects created to represent the coulomb counter
    Friend WithEvents Coulomb_Counter_RectangleShape As RectangleShape
    Friend WithEvents Coulomb_Counter_Accumulator_LineShape As LineShape
    Friend WithEvents Coulomb_Counter_Alarm_LineShape As LineShape
    Friend WithEvents Coulomb_Counter_Capacity_LineShape As LineShape
    Friend WithEvents Coulomb_Counter_Drag_Extents_RectangleShape As RectangleShape
    Friend Coulomb_Counter_Scale_Ticks_LineShape(SCALE_TICKS) As LineShape
    Friend Coulomb_Counter_Scale_Ticks_Label(SCALE_MAJOR_TICKS) As Label

    ' Textboxes and labels created to show the Battery Coulombs and Alarm in Ah..
    Friend WithEvents Battery_Capacity_Ah_Label As Label
    Friend WithEvents Battery_Capacity_Ah_TextBox As TextBox
    Friend WithEvents Battery_Capacity_Ah_LineShape As LineShape
    Friend WithEvents Battery_Coulombs_Discharged_Ah_Label As Label
    Friend WithEvents Battery_Coulombs_Discharged_Ah_TextBox As TextBox
    Friend WithEvents Battery_Coulombs_Discharged_Ah_LineShape As LineShape
    Friend WithEvents Battery_Alarm_Ah_Label As Label
    Friend WithEvents Battery_Alarm_Ah_TextBox As TextBox
    Friend WithEvents Battery_Alarm_Ah_LineShape As LineShape

    ' Textboxes and labels created to show the Coluomb Comunter Range and Resolution.
    Friend WithEvents Coulomb_Counter_Range_Label As Label
    Friend WithEvents Coulomb_Counter_Range_TextBox As TextBox
    Friend WithEvents Coulomb_Counter_Resolution_Label As Label
    Friend WithEvents Coulomb_Counter_Resolution_TextBox As TextBox

    ' Callbacks to tie actions from the battery control to the form that contains it.
    Friend Double_Click_CallBack As Action(Of Double, Double, Double)   ' Callback that gets called after the Battery Control has been double clicked.
    Friend Drag_CallBack As Action(Of Double, Double, Double)           ' Callback that gets called after a property of the Battery Control has been changed by dragging.

    ' Use a timer to ensure that the popup label never stays on forever.
    Private WithEvents popup_label_timer As New Timer
    Private WithEvents tooltip_timer As New Timer
    Private tooltip_msg As String = ""

    Public Sub New(ByVal panel As Panel,
                   ByVal popup_label As Label,
                   ByVal tooltip As ToolTip,
                   ByVal resolution As Single,
                   Optional ByVal double_click_callback_function As Action(Of Double, Double, Double) = Nothing,
                   Optional ByVal drag_callback_function As Action(Of Double, Double, Double) = Nothing,
                   Optional ByVal battery_capacity_ah As Double = 0,
                   Optional ByVal battery_coulombs_discharged_ah As Double = 0,
                   Optional ByVal alarm_ah As Double = 0,
                   Optional ByVal use_software_correction As Boolean = False,
                   Optional ByVal software_correction_factor As Single = 0.0,
                   Optional ByVal maintain_percent As Boolean = False,
                   Optional ByVal debug_textbox As TextBox = Nothing)

        ' Remember the objects passed in by form to create this control
        Me.my_panel = panel
        Me.popup_label = popup_label
        Me.tooltip = tooltip
        Me.debug_textbox = debug_textbox

        ' Search panel for the PictureBox we will use for the Battery Control
        For Each control As Control In Me.my_panel.Controls
            Dim control_picturebox As PictureBox = DirectCast(control, PictureBox)
            Me.terminal_picturebox = control_picturebox
        Next control

        ' Create the graphics and controls
        Me.gnd_pictureBox = New PictureBox
        Me.Battery_Top_LineShape = New LineShape
        Me.Battery_Bottom_LineShape = New LineShape
        Me.Battery_Left_LineShape = New LineShape
        Me.Battery_Right_LineShape = New LineShape
        Me.Battery_Capacity_RectangleShape = New RectangleShape
        Me.Battery_Coulombs_Discharged_RectangleShape = New RectangleShape
        Me.Battery_Coulombs_Discharged_LineShape = New LineShape
        Me.Battery_Alarm_LineShape = New LineShape
        Me.Battery_Drag_Extents_RectangleShape = New RectangleShape
        Me.Coulomb_Counter_RectangleShape = New RectangleShape
        Me.Coulomb_Counter_Accumulator_LineShape = New LineShape
        Me.Coulomb_Counter_Alarm_LineShape = New LineShape
        Me.Coulomb_Counter_Capacity_LineShape = New LineShape
        Me.Coulomb_Counter_Drag_Extents_RectangleShape = New RectangleShape

        Me.Battery_Capacity_Ah_Label = New Label
        Me.Battery_Capacity_Ah_TextBox = New TextBox
        Me.Battery_Capacity_Ah_LineShape = New LineShape
        Me.Battery_Coulombs_Discharged_Ah_Label = New Label
        Me.Battery_Coulombs_Discharged_Ah_TextBox = New TextBox
        Me.Battery_Coulombs_Discharged_Ah_LineShape = New LineShape
        Me.Battery_Alarm_Ah_Label = New Label
        Me.Battery_Alarm_Ah_TextBox = New TextBox
        Me.Battery_Alarm_Ah_LineShape = New LineShape
        Me.Coulomb_Counter_Range_Label = New Label
        Me.Coulomb_Counter_Range_TextBox = New TextBox
        Me.Coulomb_Counter_Resolution_Label = New Label
        Me.Coulomb_Counter_Resolution_TextBox = New TextBox

        Me.ShapeContainer = New ShapeContainer()
        Me.ShapeContainer.Shapes.AddRange(New Shape() {Coulomb_Counter_RectangleShape, Coulomb_Counter_Accumulator_LineShape, Coulomb_Counter_Alarm_LineShape, _
                                               Coulomb_Counter_Capacity_LineShape, Battery_Top_LineShape, Battery_Bottom_LineShape, Battery_Left_LineShape, _
                                               Battery_Right_LineShape, Battery_Capacity_RectangleShape, Battery_Coulombs_Discharged_RectangleShape, Battery_Coulombs_Discharged_LineShape, _
                                               Battery_Alarm_LineShape, Coulomb_Counter_Drag_Extents_RectangleShape, Battery_Drag_Extents_RectangleShape})

        Me.Legend_ShapeContainer = New ShapeContainer()
        Me.Legend_ShapeContainer.Shapes.AddRange(New Shape() {Battery_Capacity_Ah_LineShape, Battery_Coulombs_Discharged_Ah_LineShape, Battery_Alarm_Ah_LineShape})

        For tick As Integer = 0 To SCALE_TICKS
            Me.Battery_Scale_Ticks_LineShape(tick) = New LineShape
            Me.ShapeContainer.Shapes.Add(Battery_Scale_Ticks_LineShape(tick))
            Me.Coulomb_Counter_Scale_Ticks_LineShape(tick) = New LineShape
            Me.ShapeContainer.Shapes.Add(Coulomb_Counter_Scale_Ticks_LineShape(tick))
        Next tick

        For tick As Integer = 0 To SCALE_MAJOR_TICKS
            Battery_Scale_Ticks_Label(tick) = New Label
            Coulomb_Counter_Scale_Ticks_Label(tick) = New Label
        Next tick

        ' Initialize drawing of control for MAX_LINES battery size.
        Me.init()

        ' Initialize the popup label and timer.
        Me.popup_label.Visible = False
        Me.popup_label.BringToFront()
        Me.popup_label_timer.Stop()
        Me.popup_label_timer.Interval = POPUP_TIMEOUT

        ' Initialize the tooltip timer.
        Me.tooltip_timer.Stop()
        Me.tooltip_timer.Interval = TOOLTIP_TIME_ON

        ' Initialize that we are not dragging.
        Me.battery_capacity_dragging = False
        Me.battery_coulombs_discharged_dragging = False
        Me.alarm_dragging = False

        ' Store the callback functions.
        Me.Double_Click_CallBack = double_click_callback_function
        Me.Drag_CallBack = drag_callback_function

        ' Store the other parameters
        Me.use_software_correction = use_software_correction
        Me.software_correction_factor = software_correction_factor
        Me.maintain_percent = maintain_percent

        ' Now draw the control for the actual values passed in.
        Me.Resolution_Set(resolution, True)
        Me.Battery_Capacity_Set(battery_capacity_ah, True)
        Me.Battery_Coulombs_Discharged_Set(battery_coulombs_discharged_ah, True)
        Me.Alarm_Set(alarm_ah, True)

    End Sub

    Public Sub Software_Correction_Set(ByVal use_software_correction As Boolean, ByVal software_correction_factor As Single)

        ' Save the software correction parameters
        Me.use_software_correction = use_software_correction
        Me.software_correction_factor = software_correction_factor

        ' Split the lines as the Ah and counts are no longer linked together if the software correction factor changes over time.
        If use_software_correction = True Then
            Me.Coulomb_Counter_Accumulator_LineShape.X2 = CInt(Me.Coulomb_Counter_RectangleShape.Location.X - BORDER_LINE_WIDTH / 2 - SCALE_TICK_LENGTH)
        Else
            Me.Coulomb_Counter_Accumulator_LineShape.X2 = Me.Battery_Coulombs_Discharged_LineShape.X2
        End If
        Me.Coulomb_Counter_Alarm_LineShape.X2 = Me.Coulomb_Counter_Accumulator_LineShape.X2

        Me.Battery_Capacity_Set(Me.battery_capacity_ah, True)

    End Sub

    Public Sub Battery_Capacity_Set(ByVal battery_capacity_ah As Double, Optional ByVal override As Boolean = False)

        ' Do nothing if nothing changed.  This prevents the constant "blinking" of the control that occurs as the LTC3335 is constantly polled.
        ' The dragging feature is screwed up when the PIC is being polled at the same time.
        ' Allow override for initialization, or for when fixing values out of range.
        If ((Me.battery_capacity_ah <> battery_capacity_ah) And (battery_capacity_dragging = False)) Or (override = True) Then

            ' Save New Capacity
            Me.battery_capacity_ah = battery_capacity_ah

            ' Convert Ah to LTC3335 Counts
            Dim battery_capacity_counts As Integer = Coulombs_Discharged_Ah_To_Ltc3335_Counts(battery_capacity_ah)

            ' Move the lines defining the battery.
            Me.Battery_Bottom_LineShape.Y1 = Me.Battery_Top_LineShape.Y1 + Math.Min(battery_capacity_counts, MAX_LINES) + 2 * (BORDER_LINE_WIDTH \ 2)
            Me.Battery_Bottom_LineShape.Y2 = Me.Battery_Bottom_LineShape.Y1
            Me.Battery_Left_LineShape.Y2 = CInt(Me.Battery_Bottom_LineShape.Y1 + If(battery_capacity_counts < MAX_LINES, BORDER_LINE_WIDTH / 2, BORDER_LINE_WIDTH))
            Me.Battery_Right_LineShape.Y2 = CInt(Me.Battery_Bottom_LineShape.Y2 + If(battery_capacity_counts < MAX_LINES, BORDER_LINE_WIDTH / 2, BORDER_LINE_WIDTH))
            Me.Battery_Bottom_LineShape.Visible = If(battery_capacity_counts < MAX_LINES, True, False)

            ' Resize the Rectangle defining the region inside the battery.
            Me.Battery_Capacity_RectangleShape.Height = Math.Min(battery_capacity_counts, MAX_LINES) + If(battery_capacity_counts < MAX_LINES, 0, BORDER_LINE_WIDTH)

            ' Move the GND Symbol.
            Me.gnd_pictureBox.Location = New Point(Me.gnd_pictureBox.Location.X, Me.Battery_Bottom_LineShape.Y2)
            Me.gnd_pictureBox.Visible = If(battery_capacity_counts < MAX_LINES, True, False)

            ' Move the lines showing the value between the battery and the coulomb counter.
            Me.Coulomb_Counter_Capacity_LineShape.Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + battery_capacity_counts)
            Me.Coulomb_Counter_Capacity_LineShape.Y2 = Me.Coulomb_Counter_Capacity_LineShape.Y1
            Me.Coulomb_Counter_Capacity_LineShape.Visible = If(battery_capacity_counts < MAX_LINES, True, False)

            ' Resize the battery scale.
            For tick As Integer = 0 To SCALE_MAJOR_TICKS
                With Battery_Scale_Ticks_Label(tick)
                    .Location = New Point(0, CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + tick * Math.Min(battery_capacity_counts, MAX_LINES) / SCALE_MAJOR_TICKS - .Height / 2))
                    .Text = Get_Percent_Discharged_String(tick * Math.Min(battery_capacity_ah, Me.range) / SCALE_MAJOR_TICKS)
                End With
            Next tick

            For tick As Integer = 0 To SCALE_TICKS
                Dim major_tick As Boolean = (tick Mod (SCALE_TICKS / SCALE_MAJOR_TICKS)) = 0

                With Battery_Scale_Ticks_LineShape(tick)
                    .Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + tick * Math.Min(battery_capacity_counts, MAX_LINES) / SCALE_TICKS)
                    .Y2 = .Y1
                End With
            Next tick

            ' Update the Textbox.
            Me.Battery_Capacity_Ah_TextBox.Text = Me.Get_Percent_Discharged_String(battery_capacity_ah) & Me.Get_Coulombs_Discharged_String(battery_capacity_ah)
            Update_Range_and_Resolution_TextBoxes()

            ' Update the Coulombs Discharge and Alarm, as they depend upon the Capacity
            Me.Battery_Coulombs_Discharged_Set(battery_coulombs_discharged_ah, True)
            Me.Alarm_Set(alarm_ah, True)

            ' Refresh the graphic
            Me.ShapeContainer.Parent.Refresh()
        End If
    End Sub

    Public Sub Battery_Coulombs_Discharged_Set(ByVal battery_coulombs_discharged_ah As Double, Optional ByVal override As Boolean = False, Optional ByVal accumulator As Integer = -1)

        ' Show debug text if configured.
        If (Not Me.debug_textbox Is Nothing) Then
            Me.debug_textbox.Text = Me.battery_coulombs_discharged_ah.ToString() + ", " + (Me.battery_coulombs_discharged_ah <> battery_coulombs_discharged_ah).ToString
        End If

        ' The dragging feature is screwed up when the PIC is being polled at the same time.
        ' Allow override for initialization, or for when fixing values out of range.
        If ((battery_capacity_dragging = False) And (battery_coulombs_discharged_dragging = False)) Or (override = True) Then

            ' Convert Ah to LTC3335 Counts
            Dim old_battery_coulombs_discharged_counts As Integer = Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.battery_coulombs_discharged_ah)
            Dim new_battery_coulombs_discharged_counts As Integer = Coulombs_Discharged_Ah_To_Ltc3335_Counts(battery_coulombs_discharged_ah)
            Dim battery_capacity_counts As Integer = Coulombs_Discharged_Ah_To_Ltc3335_Counts(battery_capacity_ah)

            ' Do not update graphic if value did not change enough for one pixel to be different.  This prevents the constant "blinking" of the control that occurs as the LTC3335 is constantly polled.
            If (old_battery_coulombs_discharged_counts <> new_battery_coulombs_discharged_counts) Or (override = True) Then

                ' Resize the Rectangle defining the region inside the battery.
                Me.Battery_Coulombs_Discharged_RectangleShape.Height = Math.Min(new_battery_coulombs_discharged_counts, Math.Min(battery_capacity_counts, MAX_LINES)) + If(new_battery_coulombs_discharged_counts < MAX_LINES, 0, BORDER_LINE_WIDTH)

                ' Move the lines showing the value between the battery and the coulomb counter.
                Me.Battery_Coulombs_Discharged_LineShape.Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + Math.Min(new_battery_coulombs_discharged_counts, MAX_LINES))
                Me.Battery_Coulombs_Discharged_LineShape.Y2 = Me.Battery_Coulombs_Discharged_LineShape.Y1
                Me.Battery_Coulombs_Discharged_LineShape.Visible = If(Me.battery_coulombs_discharged_ah <= Me.range, True, False)

                ' If not performing software correction or if doing software correction in the battery selection window, accumulator will be se to nonsensical value.
                If accumulator < 0 Then
                    Me.Coulomb_Counter_Accumulator_LineShape.Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + Math.Min(new_battery_coulombs_discharged_counts, MAX_LINES))
                    Me.Coulomb_Counter_Accumulator_LineShape.Y2 = Me.Coulomb_Counter_Accumulator_LineShape.Y1
                    Me.Coulomb_Counter_Accumulator_LineShape.Visible = If(new_battery_coulombs_discharged_counts < MAX_LINES, True, False)
                Else
                    ' If not performing software correction in the control panel, the accumulator and battery values can drift apart due to quiecent current and typ error changes over VBAT.
                    Me.Coulomb_Counter_Accumulator_LineShape.Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + Math.Min(accumulator, MAX_LINES))
                    Me.Coulomb_Counter_Accumulator_LineShape.Y2 = Me.Coulomb_Counter_Accumulator_LineShape.Y1
                    Me.Coulomb_Counter_Accumulator_LineShape.Visible = If(accumulator < MAX_LINES, True, False)
                End If

                ' Refresh the graphic
                Me.ShapeContainer.Parent.Refresh()
            End If

            ' Save New Coulombs Discharged.
            Me.battery_coulombs_discharged_ah = battery_coulombs_discharged_ah

            ' Finally Update the Textboxes.
            Me.Battery_Coulombs_Discharged_Ah_TextBox.Text = Me.Get_Percent_Discharged_String(battery_coulombs_discharged_ah) & Me.Get_Coulombs_Discharged_String(battery_coulombs_discharged_ah)

        End If

    End Sub

    Public Sub Alarm_Set(ByVal alarm_ah As Double, Optional ByVal override As Boolean = False)

        ' The dragging feature is screwed up when the PIC is being polled at the same time.
        ' Allow override for initialization, or for when fixing values out of range.
        If (alarm_dragging = False) Or (override = True) Then

            ' Convert Ah to LTC3335 Counts
            Dim old_battery_alarm_counts As Integer = Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.alarm_ah)
            Dim new_battery_alarm_counts As Integer = Coulombs_Discharged_Ah_To_Ltc3335_Counts(alarm_ah)

            ' Do not update graphic if value did not change enough for one pixel to be different.  This prevents the constant "blinking" of the control that occurs as the LTC3335 is constantly polled.
            If (Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.alarm_ah) <> Coulombs_Discharged_Ah_To_Ltc3335_Counts(alarm_ah)) Or (override = True) Then

                ' Move the lines showing the value between the battery and the coulomb counter.
                Me.Battery_Alarm_LineShape.Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + Math.Min(new_battery_alarm_counts, MAX_LINES))
                Me.Battery_Alarm_LineShape.Y2 = Me.Battery_Alarm_LineShape.Y1
                Me.Battery_Alarm_LineShape.Visible = If(Me.alarm_ah <= Me.range, True, False)
                Me.Coulomb_Counter_Alarm_LineShape.Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + Math.Min(new_battery_alarm_counts, MAX_LINES))
                Me.Coulomb_Counter_Alarm_LineShape.Y2 = Me.Coulomb_Counter_Alarm_LineShape.Y1
                Me.Coulomb_Counter_Alarm_LineShape.Visible = If(new_battery_alarm_counts < MAX_LINES, True, False)

                ' Refresh the graphic
                Me.ShapeContainer.Parent.Refresh()
            End If

            ' Save New Coulombs Discharged.
            Me.alarm_ah = alarm_ah

            ' Finally Update the Textboxes.
            Me.Battery_Alarm_Ah_TextBox.Text = Me.Get_Percent_Discharged_String(alarm_ah) & Me.Get_Coulombs_Discharged_String(alarm_ah)

        End If
    End Sub

    Public Sub Resolution_Set(ByVal resolution As Single, Optional ByVal override As Boolean = False)
        ' Save the new resolution
        Me.resolution = resolution

        ' Redraw all of the shapes based upon the new resolution
        Me.Battery_Capacity_Set(Me.battery_capacity_ah, True)
        Me.Battery_Coulombs_Discharged_Set(Me.battery_coulombs_discharged_ah, True)
        Me.Alarm_Set(Me._alarm_ah, True)

        ' Finally Update the Textboxes.
        Update_Range_and_Resolution_TextBoxes()

    End Sub

    Private Sub Update_Range_and_Resolution_TextBoxes()
        Me.Coulomb_Counter_Range_TextBox.Text = Me.Get_Percent_Discharged_String(Me.range) & Me.Get_Coulombs_Discharged_String(Me.range)
        If Me.battery_capacity_ah <= (Me.range) Then
            With Me.Coulomb_Counter_Range_TextBox
                .ReadOnly = True
                .ForeColor = Me.OK_COLOR
                .Font = CustomFont.GetFont(.Font.Name, .Font.Size, FontStyle.Regular)
            End With
        Else
            With Me.Coulomb_Counter_Range_TextBox
                .ReadOnly = False ' can not change the text color unless ReadOnly is false.
                .ForeColor = Me.ERROR_COLOR
                .Font = CustomFont.GetFont(.Font.Name, .Font.Size, FontStyle.Bold)
            End With
        End If

        Me.Coulomb_Counter_Resolution_TextBox.Text = Me.Get_Percent_Discharged_String(resolution) & Me.Get_Coulombs_Discharged_String(resolution)
        If Me.battery_capacity_ah >= (Me.range / 2) Then
            With Me.Coulomb_Counter_Resolution_TextBox
                .ReadOnly = True
                .ForeColor = Me.OK_COLOR
                .Font = CustomFont.GetFont(.Font.Name, .Font.Size, FontStyle.Regular)
            End With
        Else
            With Me.Coulomb_Counter_Resolution_TextBox
                .ReadOnly = False ' can not change the text color unless ReadOnly is false.
                .ForeColor = Me.ERROR_COLOR
                .Font = CustomFont.GetFont(.Font.Name, .Font.Size, FontStyle.Bold)
            End With
        End If
    End Sub

    Public Function Coulombs_Discharged_Ah_To_Percent_Discharged(ByVal coulombs_discharged_ah As Double) As Single
        If Me.battery_capacity_ah = 0 Then
            Return 0
        Else
            Return CSng(coulombs_discharged_ah * 100.0 / Me.battery_capacity_ah)
        End If
    End Function

    Public Function Percent_Discharged_To_Coulombs_Discharged_Ah(ByVal percent_discharged As Single) As Single
        Return CSng(percent_discharged * Me.battery_capacity_ah / 100.0)
    End Function

    Public Function Coulombs_Discharged_Ah_To_Ltc3335_Counts(ByVal coulombs_discharged As Double) As Integer
        If Me.resolution = 0 Then
            Return 0
        Else
            Const CLOSE_ENOUGH As Double = 0.01 ' If we are within this many counts, then round up.
            Return CInt(Math.Floor(coulombs_discharged / Me.resolution + CLOSE_ENOUGH))
        End If
    End Function

    Public Function Ltc3335_Counts_To_Coulombs_Discharged_Ah(ByVal Ltc3335_Counts As Integer) As Single
        Return Ltc3335_Counts * Me.resolution
    End Function

    Public Function Get_Percent_Discharged_String(ByVal coulombs_discharged As Double, Optional ByVal add_units As Boolean = True) As String
        Dim percent_discharged As Single = Me.Coulombs_Discharged_Ah_To_Percent_Discharged(coulombs_discharged)
        Dim percent_format As String

        If Math.Abs(percent_discharged) >= 100 Then
            percent_format = "000"
        ElseIf Math.Abs(percent_discharged) >= 10 Then
            percent_format = "00.0"
        ElseIf (Math.Abs(percent_discharged) >= 0.01) Or (Math.Abs(percent_discharged) = 0) Then
            percent_format = "0.00"
        Else
            Return "<" & 0.01.ToString("0.00") & If(add_units = True, "%", "")
        End If

        Return percent_discharged.ToString(percent_format) & If(add_units = True, "%", "")
    End Function

    Public Function Get_Coulombs_Discharged_String(ByVal coulombs_discharged As Double, Optional ByVal add_paran As Boolean = True) As String
        Dim charge_format As String
        Dim charge_units As String

        If Math.Abs(coulombs_discharged) < 0.001 Then
            coulombs_discharged *= 1000000
            charge_units = "µAh"
        ElseIf Math.Abs(coulombs_discharged) < 1 Then
            coulombs_discharged *= 1000
            charge_units = "mAh"
        Else
            charge_units = "Ah"
        End If

        If Math.Abs(coulombs_discharged) > 100 Then
            charge_format = "000"
        ElseIf Math.Abs(coulombs_discharged) > 10 Then
            charge_format = "00.0"
        ElseIf Math.Abs(coulombs_discharged) > 1 Then
            charge_format = "0.00"
        Else
            charge_format = "0.000"
        End If

        Return If(add_paran = True, "(", "") + coulombs_discharged.ToString(charge_format) + charge_units + If(add_paran = True, ")", "")

    End Function

    Private Sub Double_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles terminal_picturebox.DoubleClick, _
        gnd_pictureBox.DoubleClick, Battery_Top_LineShape.DoubleClick, Battery_Bottom_LineShape.DoubleClick, Battery_Left_LineShape.DoubleClick, Battery_Right_LineShape.DoubleClick, _
        Battery_Capacity_RectangleShape.DoubleClick, Battery_Coulombs_Discharged_RectangleShape.DoubleClick, Battery_Coulombs_Discharged_LineShape.DoubleClick, Battery_Alarm_LineShape.DoubleClick, _
        Coulomb_Counter_RectangleShape.DoubleClick, Coulomb_Counter_Accumulator_LineShape.DoubleClick, Coulomb_Counter_Alarm_LineShape.DoubleClick, Coulomb_Counter_Capacity_LineShape.DoubleClick, _
        Battery_Capacity_Ah_Label.DoubleClick, Battery_Capacity_Ah_TextBox.Click, Battery_Capacity_Ah_LineShape.DoubleClick,
        Battery_Coulombs_Discharged_Ah_Label.DoubleClick, Battery_Coulombs_Discharged_Ah_LineShape.DoubleClick, Battery_Alarm_Ah_Label.DoubleClick, Battery_Alarm_Ah_LineShape.DoubleClick, Coulomb_Counter_Range_Label.DoubleClick,
        Coulomb_Counter_Resolution_Label.DoubleClick, Coulomb_Counter_Resolution_TextBox.Click, Coulomb_Counter_Range_TextBox.Click, Battery_Alarm_Ah_TextBox.Click, Battery_Coulombs_Discharged_Ah_TextBox.Click

        ' Call a callback function if it's registered.
        If Not Me.Double_Click_CallBack Is Nothing Then
            tooltip_msg = ""
            tooltip_timer_tick(tooltip_timer, New EventArgs)

            Me.Double_Click_CallBack(Me.battery_capacity_ah, Me.battery_coulombs_discharged_ah, Me.alarm_ah)
        End If

    End Sub

    Private Sub TextBox_MouseEnter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Coulomb_Counter_Resolution_TextBox.MouseEnter, _
        Coulomb_Counter_Range_TextBox.MouseEnter, Battery_Alarm_Ah_TextBox.MouseEnter, Battery_Coulombs_Discharged_Ah_TextBox.MouseEnter, Battery_Capacity_Ah_TextBox.MouseEnter
        Dim textbox As TextBox = DirectCast(sender, TextBox)
        If Not Me.Double_Click_CallBack Is Nothing Then
            textbox.ReadOnly = False
        End If
    End Sub

    Private Sub TextBox_MouseHover(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Coulomb_Counter_Resolution_TextBox.MouseHover, _
    Coulomb_Counter_Range_TextBox.MouseHover, Battery_Alarm_Ah_TextBox.MouseHover, Battery_Coulombs_Discharged_Ah_TextBox.MouseHover, Battery_Capacity_Ah_TextBox.MouseHover,
    Battery_Capacity_RectangleShape.MouseHover, Battery_Coulombs_Discharged_RectangleShape.MouseHover, Coulomb_Counter_RectangleShape.MouseHover

        Dim shape_name As String = ""

        If (sender.GetType() Is GetType(TextBox)) Then
            Dim textbox As TextBox = DirectCast(sender, TextBox)

            If Not Me.Double_Click_CallBack Is Nothing Then
                tooltip_msg = "Click to open Battery Selection Window"
                Select Case textbox.Name
                    Case Battery_Alarm_Ah_TextBox.Name, Battery_Coulombs_Discharged_Ah_TextBox.Name, Battery_Capacity_Ah_TextBox.Name
                        tooltip_msg += " or " + ChrW(13) + Chr(10)
                    Case Else
                        tooltip_msg += ""
                End Select
            End If
            Select Case textbox.Name
                Case Battery_Alarm_Ah_TextBox.Name
                    tooltip_msg += "Click and drag red line to change value"
                Case Battery_Coulombs_Discharged_Ah_TextBox.Name
                    tooltip_msg += "Click and drag green line to change value"
                Case Battery_Capacity_Ah_TextBox.Name
                    tooltip_msg += "Click and drag black line to change value"
                Case Else
                    tooltip_msg += ""
            End Select

        ElseIf (sender.GetType() Is GetType(RectangleShape)) Then
            Dim rectangleshape As RectangleShape = DirectCast(sender, RectangleShape)

            If Not Me.Double_Click_CallBack Is Nothing Then
                tooltip_msg = "Double Click to open Battery Selection Window"
            End If
        End If

        If tooltip_msg <> "" Then
            Me.tooltip_timer.Stop()
            Me.tooltip_timer.Interval = TOOLTIP_TIME_ON
            Me.tooltip_timer.Start()
        End If
    End Sub

    Private Sub TextBox_MouseMove(ByVal sender As System.Object, ByVal e As MouseEventArgs) Handles Coulomb_Counter_Resolution_TextBox.MouseMove, _
    Coulomb_Counter_Range_TextBox.MouseMove, Battery_Alarm_Ah_TextBox.MouseMove, Battery_Coulombs_Discharged_Ah_TextBox.MouseMove, Battery_Capacity_Ah_TextBox.MouseMove
        Dim textbox As TextBox = DirectCast(sender, TextBox)
        If Not Me.Double_Click_CallBack Is Nothing Then
            textbox.ReadOnly = False
        End If
    End Sub

    Private Sub TextBox_MouseLeave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Coulomb_Counter_Resolution_TextBox.MouseLeave, _
        Coulomb_Counter_Range_TextBox.MouseLeave, Battery_Alarm_Ah_TextBox.MouseLeave, Battery_Coulombs_Discharged_Ah_TextBox.MouseLeave, Battery_Capacity_Ah_TextBox.MouseLeave
        Dim textbox As TextBox = DirectCast(sender, TextBox)
        If textbox.ForeColor <> Me.ERROR_COLOR Then
            textbox.ReadOnly = True
        End If
        tooltip_msg = ""
        tooltip_timer_tick(tooltip_timer, New EventArgs)
    End Sub

    ' Make popup invisible once the mouse exits the battery or coulomb counter.
    Private Sub Panel_PictureBox_MouseEnter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles my_panel.MouseEnter, _
    terminal_picturebox.MouseEnter, gnd_pictureBox.MouseEnter
        Me.popup_label.Visible = False
        MouseUp(sender, New MouseEventArgs(0, 0, 0, 0, 0))

        tooltip_msg = ""
        tooltip_timer_tick(tooltip_timer, New EventArgs)
    End Sub

    ' Since the mouse can be moved fast enough to skip entering the panels & pictureboxes, use a timer to ensure it is never turned on forever.
    Private Sub popup_label_timer_tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles popup_label_timer.Tick
        Me.popup_label.Visible = False
        Me.popup_label_timer.Stop()
    End Sub

    ' If the tooltip message was set, show the tooltip and set off to turn off in TOOLTIP_TIME_OFF time.
    Private Sub tooltip_timer_tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tooltip_timer.Tick
        If tooltip_msg <> "" Then
            Dim new_location As New Point
            new_location = Me.my_panel.PointToClient(Windows.Forms.Cursor.Position)
            tooltip.Show(tooltip_msg, Me.my_panel, New Point(new_location.X + TOOLTIP_OFFSET, new_location.Y + TOOLTIP_OFFSET))

            tooltip_msg = ""
            Me.tooltip_timer.Stop()
            Me.tooltip_timer.Interval = TOOLTIP_TIME_OFF
            Me.tooltip_timer.Start()
        Else
            Me.tooltip_timer.Stop()
            tooltip.Hide(Me.my_panel)
        End If
    End Sub

    ' Make popup visible once the mouse enters the battery and display the percent coulombs discharged/coulombs discharged.
    Private Sub Battery_MouseMove(ByVal sender As System.Object, ByVal e As MouseEventArgs) Handles Battery_Capacity_RectangleShape.MouseMove, _
    Battery_Coulombs_Discharged_RectangleShape.MouseMove, Battery_Coulombs_Discharged_LineShape.MouseMove, Battery_Alarm_LineShape.MouseMove, _
    Battery_Bottom_LineShape.MouseMove, Battery_Drag_Extents_RectangleShape.MouseMove

        Dim shape_name As String = ""
        Dim temp_lineshape As New LineShape
        Dim temp_rectangleshape As New RectangleShape

        If (sender.GetType() Is GetType(LineShape)) Then
            temp_lineshape = DirectCast(sender, LineShape)
            shape_name = temp_lineshape.Name
        ElseIf (sender.GetType() Is GetType(RectangleShape)) Then
            temp_rectangleshape = DirectCast(sender, RectangleShape)
            shape_name = temp_rectangleshape.Name
        End If

        ' Only show popup if on the coulomb counter graphic or dragging a line.
        If ((shape_name <> Me.Battery_Drag_Extents_RectangleShape.Name) Or is_dragging()) Then
            Me.popup_label.Visible = True
            Me.popup_label_timer.Start()
        Else
            Me.popup_label.Visible = False
        End If


        ' Translate the position into a coulombs discharged.
        Dim coulombs_discharged As Double = Ltc3335_Counts_To_Coulombs_Discharged_Ah(e.Y)

        ' Create label displaying charge value under mouse.
        Dim label_text As String = ""
        Dim new_location As New Point

        Select Case shape_name
            Case Me.Battery_Bottom_LineShape.Name
                label_text = "Battery Capacity: " + ChrW(13) + ChrW(10)
                coulombs_discharged = Me.battery_capacity_ah
                new_location.Y = Battery_Bottom_LineShape.Y1 + temp_lineshape.Parent.Parent.Location.Y - LABEL_OFFSET
                new_location.X = e.X + Me.Battery_Bottom_LineShape.X1 + temp_lineshape.Parent.Parent.Location.X + LABEL_OFFSET
                tooltip_msg = ""
                tooltip_timer_tick(tooltip_timer, New EventArgs)
            Case Me.Battery_Coulombs_Discharged_LineShape.Name
                label_text = "Coulombs Discharged: " + ChrW(13) + ChrW(10)
                coulombs_discharged = Me.battery_coulombs_discharged_ah
                new_location.Y = Battery_Coulombs_Discharged_LineShape.Y1 + temp_lineshape.Parent.Parent.Location.Y - LABEL_OFFSET
                new_location.X = e.X + Me.Battery_Coulombs_Discharged_LineShape.X1 + temp_lineshape.Parent.Parent.Location.X + LABEL_OFFSET
                tooltip_msg = ""
                tooltip_timer_tick(tooltip_timer, New EventArgs)
            Case Me.Battery_Alarm_LineShape.Name
                label_text = "Alarm Coulombs: " + ChrW(13) + ChrW(10)
                coulombs_discharged = Me.alarm_ah
                new_location.Y = Battery_Alarm_LineShape.Y1 + temp_lineshape.Parent.Parent.Location.Y - LABEL_OFFSET
                new_location.X = e.X + Me.Battery_Alarm_LineShape.X1 + temp_lineshape.Parent.Parent.Location.X + LABEL_OFFSET
                tooltip_msg = ""
                tooltip_timer_tick(tooltip_timer, New EventArgs)
            Case Me.Battery_Drag_Extents_RectangleShape.Name
                new_location.Y = e.Y + Me.Battery_Capacity_RectangleShape.Location.Y + temp_rectangleshape.Parent.Parent.Location.Y - LABEL_OFFSET
                new_location.X = e.X + Me.Battery_Drag_Extents_RectangleShape.Location.X + temp_rectangleshape.Parent.Parent.Location.X + LABEL_OFFSET
            Case Else
                new_location.Y = e.Y + Me.Battery_Capacity_RectangleShape.Location.Y + temp_rectangleshape.Parent.Parent.Location.Y - LABEL_OFFSET
                new_location.X = e.X + Me.Battery_Capacity_RectangleShape.Location.X + temp_rectangleshape.Parent.Parent.Location.X + LABEL_OFFSET
        End Select

        ' If we're moving a line, update to selected charge level and maintain the label.
        If Me.battery_capacity_dragging = True Then
            If (maintain_percent) Then
                Dim cell_charge_percent_discharged As Single = Me.Coulombs_Discharged_Ah_To_Percent_Discharged(Me.battery_coulombs_discharged_ah)
                Dim alarm_percent_discharged As Single = Me.Coulombs_Discharged_Ah_To_Percent_Discharged(Me.alarm_ah)
                Me.Battery_Capacity_Set(coulombs_discharged, True)
                Me.Battery_Coulombs_Discharged_Set(Me.Percent_Discharged_To_Coulombs_Discharged_Ah(cell_charge_percent_discharged), True)
                Me.Alarm_Set(Me.Percent_Discharged_To_Coulombs_Discharged_Ah(alarm_percent_discharged), True)
            Else
                Me.Battery_Capacity_Set(coulombs_discharged, True)
            End If
            label_text = "Battery Capacity: " + ChrW(13) + ChrW(10)
            tooltip_msg = ""
            tooltip_timer_tick(tooltip_timer, New EventArgs)
        ElseIf Me.battery_coulombs_discharged_dragging = True Then
            coulombs_discharged = Math.Min(coulombs_discharged, Me.range)
            Me.Battery_Coulombs_Discharged_Set(coulombs_discharged, True)
            label_text = "Coulombs Discharged: " + ChrW(13) + ChrW(10)
            tooltip_msg = ""
            tooltip_timer_tick(tooltip_timer, New EventArgs)
        ElseIf Me.alarm_dragging = True Then
            coulombs_discharged = Math.Min(coulombs_discharged, Me.range)
            Me.Alarm_Set(coulombs_discharged, True)
            label_text = "Alarm Coulombs: " + ChrW(13) + ChrW(10)
            tooltip_msg = ""
            tooltip_timer_tick(tooltip_timer, New EventArgs)
        End If

        ' Update the popup text.
        label_text += Get_Percent_Discharged_String(coulombs_discharged) + Get_Coulombs_Discharged_String(coulombs_discharged)
        Me.popup_label.Text = label_text
        Me.popup_label.Location = new_location

    End Sub

    ' Make popup visible once the mouse enters the coulomb counter and display the coulomb count/coulombs discharged.
    Private Sub Coulomb_Counter_MouseMove(ByVal sender As System.Object, ByVal e As MouseEventArgs) Handles Coulomb_Counter_RectangleShape.MouseMove,
        Coulomb_Counter_Accumulator_LineShape.MouseMove, Coulomb_Counter_Alarm_LineShape.MouseMove, Coulomb_Counter_Capacity_LineShape.MouseMove,
        Coulomb_Counter_Drag_Extents_RectangleShape.MouseMove

        Dim shape_name As String = ""
        Dim temp_lineshape As New LineShape
        Dim temp_rectangleshape As New RectangleShape

        If (sender.GetType() Is GetType(LineShape)) Then
            temp_lineshape = DirectCast(sender, LineShape)
            shape_name = temp_lineshape.Name
        ElseIf (sender.GetType() Is GetType(RectangleShape)) Then
            temp_rectangleshape = DirectCast(sender, RectangleShape)
            shape_name = temp_rectangleshape.Name
        End If

        ' Only show popup if on the coulomb counter graphic or dragging a line.
        If ((shape_name <> Me.Coulomb_Counter_Drag_Extents_RectangleShape.Name) Or is_dragging()) Then
            Me.popup_label.Visible = True
            Me.popup_label_timer.Start()
        Else
            Me.popup_label.Visible = False
        End If

        ' Translate the position into a coulomb count.
        Dim coulomb_count As Integer = CInt(Math.Min(If(e.Y >= BORDER_LINE_WIDTH / 2, e.Y - BORDER_LINE_WIDTH / 2, 0), MAX_LINES - 1))

        ' Create label displaying charge value under mouse.
        Dim label_text As String = ""
        Dim new_location As New Point

        Select Case shape_name
            Case Me.Coulomb_Counter_Capacity_LineShape.Name
                label_text = "Capacity Count: " + ChrW(13) + ChrW(10)
                coulomb_count = Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.battery_capacity_ah)
                new_location.Y = Coulomb_Counter_Capacity_LineShape.Y1 + temp_lineshape.Parent.Parent.Location.Y - LABEL_OFFSET
                new_location.X = e.X + Me.Coulomb_Counter_Capacity_LineShape.X2 + temp_lineshape.Parent.Parent.Location.X + LABEL_OFFSET
                tooltip_msg = ""
                tooltip_timer_tick(tooltip_timer, New EventArgs)
            Case Me.Coulomb_Counter_Accumulator_LineShape.Name
                label_text = "Coulomb Count: " + ChrW(13) + ChrW(10)
                If Me.use_software_correction = False Then
                    coulomb_count = Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.battery_coulombs_discharged_ah)
                Else
                    coulomb_count = CInt(Me.Coulomb_Counter_Accumulator_LineShape.Y1 - (Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2))
                End If
                new_location.Y = Coulomb_Counter_Accumulator_LineShape.Y1 + temp_lineshape.Parent.Parent.Location.Y - LABEL_OFFSET
                new_location.X = e.X + Me.Coulomb_Counter_Accumulator_LineShape.X2 + temp_lineshape.Parent.Parent.Location.X + LABEL_OFFSET
                tooltip_msg = ""
                tooltip_timer_tick(tooltip_timer, New EventArgs)
            Case Me.Coulomb_Counter_Alarm_LineShape.Name
                label_text = "Alarm Count: " + ChrW(13) + ChrW(10)
                If Me.use_software_correction = False Then
                    coulomb_count = Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.alarm_ah)
                Else
                    coulomb_count = CInt(Coulomb_Counter_Alarm_LineShape.Y1 - (Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2))
                End If
                new_location.Y = Coulomb_Counter_Alarm_LineShape.Y1 + temp_lineshape.Parent.Parent.Location.Y - LABEL_OFFSET
                new_location.X = e.X + Me.Coulomb_Counter_Alarm_LineShape.X2 + temp_lineshape.Parent.Parent.Location.X + LABEL_OFFSET
                tooltip_msg = ""
                tooltip_timer_tick(tooltip_timer, New EventArgs)
            Case Me.Coulomb_Counter_Drag_Extents_RectangleShape.Name
                new_location.Y = e.Y + Me.Coulomb_Counter_RectangleShape.Location.Y + temp_rectangleshape.Parent.Parent.Location.Y - LABEL_OFFSET
                new_location.X = e.X + Me.Coulomb_Counter_Drag_Extents_RectangleShape.Location.X + temp_rectangleshape.Parent.Parent.Location.X + LABEL_OFFSET
            Case Else
                new_location.Y = e.Y + Me.Coulomb_Counter_RectangleShape.Location.Y + temp_rectangleshape.Parent.Parent.Location.Y - LABEL_OFFSET
                new_location.X = e.X + Me.Coulomb_Counter_RectangleShape.Location.X + temp_rectangleshape.Parent.Parent.Location.X + LABEL_OFFSET
        End Select

        ' If we're moving a line, update to selected charge level and maintain the label.
        If Me.battery_capacity_dragging = True Then
            If (maintain_percent) Then
                Dim cell_charge_percent_discharged As Single = Me.Coulombs_Discharged_Ah_To_Percent_Discharged(Me.battery_coulombs_discharged_ah)
                Dim alarm_percent_discharged As Single = Me.Coulombs_Discharged_Ah_To_Percent_Discharged(Me.alarm_ah)
                Me.Battery_Capacity_Set(Ltc3335_Counts_To_Coulombs_Discharged_Ah(coulomb_count), True)
                Me.Battery_Coulombs_Discharged_Set(Me.Percent_Discharged_To_Coulombs_Discharged_Ah(cell_charge_percent_discharged), True)
                Me.Alarm_Set(Me.Percent_Discharged_To_Coulombs_Discharged_Ah(alarm_percent_discharged), True)
            Else
                Me.Battery_Capacity_Set(Ltc3335_Counts_To_Coulombs_Discharged_Ah(coulomb_count), True)
            End If
            label_text = "Capacity Count: " + ChrW(13) + ChrW(10)
            tooltip_msg = ""
            tooltip_timer_tick(tooltip_timer, New EventArgs)
        ElseIf Me.battery_coulombs_discharged_dragging = True Then
            coulomb_count = Coulombs_Discharged_Ah_To_Ltc3335_Counts(Math.Min(Ltc3335_Counts_To_Coulombs_Discharged_Ah(coulomb_count), Me.range))
            Me.Battery_Coulombs_Discharged_Set(Ltc3335_Counts_To_Coulombs_Discharged_Ah(coulomb_count), True)
            label_text = "Coulomb Count: " + ChrW(13) + ChrW(10)
            tooltip_msg = ""
            tooltip_timer_tick(tooltip_timer, New EventArgs)
        ElseIf Me.alarm_dragging = True Then
            coulomb_count = Coulombs_Discharged_Ah_To_Ltc3335_Counts(Math.Min(Ltc3335_Counts_To_Coulombs_Discharged_Ah(coulomb_count), Me.range))
            Me.Alarm_Set(Ltc3335_Counts_To_Coulombs_Discharged_Ah(coulomb_count), True)
            label_text = "Alarm Count: " + ChrW(13) + ChrW(10)
            tooltip_msg = ""
            tooltip_timer_tick(tooltip_timer, New EventArgs)
        End If

        ' Update the popup text.
        label_text += coulomb_count.ToString(COULOMBS_FORMAT) + " counts " + Get_Coulombs_Discharged_String(Ltc3335_Counts_To_Coulombs_Discharged_Ah(CInt(coulomb_count)))
        Me.popup_label.Text = label_text
        Me.popup_label.Location = new_location

    End Sub

    Private Sub Battery_Capacity_LineShape_MouseDown(ByVal sender As System.Object, ByVal e As MouseEventArgs) Handles Coulomb_Counter_Capacity_LineShape.MouseDown, _
        Battery_Bottom_LineShape.MouseDown
        Me.battery_capacity_dragging = True

    End Sub

    Private Sub Battery_Coulombs_Discharged_LineShape_MouseDown(ByVal sender As System.Object, ByVal e As MouseEventArgs) Handles Coulomb_Counter_Accumulator_LineShape.MouseDown, _
        Battery_Coulombs_Discharged_LineShape.MouseDown
        Me.battery_coulombs_discharged_dragging = True
    End Sub

    Private Sub Alarm_LineShape_MouseDown(ByVal sender As System.Object, ByVal e As MouseEventArgs) Handles Coulomb_Counter_Alarm_LineShape.MouseDown, _
        Battery_Alarm_LineShape.MouseDown
        Me.alarm_dragging = True
    End Sub

    Private Sub MouseUp(ByVal sender As System.Object, ByVal e As MouseEventArgs) Handles Battery_Capacity_RectangleShape.MouseUp, _
    Battery_Coulombs_Discharged_RectangleShape.MouseUp, Battery_Coulombs_Discharged_LineShape.MouseUp, Battery_Alarm_LineShape.MouseUp, Battery_Bottom_LineShape.MouseUp, _
    Coulomb_Counter_RectangleShape.MouseUp, Coulomb_Counter_Accumulator_LineShape.MouseUp, Coulomb_Counter_Alarm_LineShape.MouseUp, Coulomb_Counter_Capacity_LineShape.MouseUp,
    my_panel.MouseUp, Coulomb_Counter_Drag_Extents_RectangleShape.MouseUp, Battery_Drag_Extents_RectangleShape.MouseUp

        If is_dragging() Then

            ' Quit dragging lines once we leave battery control.
            Me.battery_capacity_dragging = False
            Me.battery_coulombs_discharged_dragging = False
            Me.alarm_dragging = False

            ' Call a callback function if it's registered.
            If Not Me.Drag_CallBack Is Nothing Then
                Me.Drag_CallBack(Me.battery_capacity_ah, Me.battery_coulombs_discharged_ah, Me._alarm_ah)
            End If
        End If
    End Sub

    Private Function is_dragging() As Boolean
        Return ((Me.battery_capacity_dragging = True) Or (Me.battery_coulombs_discharged_dragging = True) Or (Me.alarm_dragging = True))
    End Function

    ' Create the initial drawing of the battery, coulomb counter, lines, and textboxes.
    Private Sub init()

        ' Temp Point useful when calculating positions for the components of the control.
        Dim temp_point As New Point

        ' Temp Height and Width useful when using autosize on labels to get their minimum possible size, then setting all of the other labels to the size of the biggest one.
        Dim temp_height As Integer
        Dim temp_width As Integer

        Const STARTING_COULOMBS_DISCHARGED As Single = 25   ' in %
        Const STARTING_ALARM As Single = 75

        ' Create battery scale at the far left of the panel

        ' Create the tick labels first
        For tick As Integer = 0 To SCALE_MAJOR_TICKS
            With Battery_Scale_Ticks_Label(tick)
                .Parent = Me.my_panel
                .AutoSize = True
                .BackColor = Color.Silver
                .Margin = New Padding(0)
                .Padding = New Padding(0)
                .Font = New Font("Linear Helv Cond", 8.0!, FontStyle.Regular)
                .Location = New Point(0, CInt(Me.terminal_picturebox.Location.Y + Me.terminal_picturebox.Size.Height + BORDER_LINE_WIDTH + BORDER_LINE_WIDTH / 2 + tick * MAX_LINES / SCALE_MAJOR_TICKS))
                .TextAlign = ContentAlignment.MiddleRight
                .Text = "<0.01%" ' Make textbox large enough for when "<" is appeneded to the text
            End With
            D3335.Set_Font(DirectCast(Battery_Scale_Ticks_Label(tick), Control))
        Next tick

        ' Now rescale the labels so that they're all the same height and width
        temp_height = Battery_Scale_Ticks_Label(SCALE_MAJOR_TICKS).Height
        temp_width = Battery_Scale_Ticks_Label(SCALE_MAJOR_TICKS).Width
        For tick As Integer = 0 To SCALE_MAJOR_TICKS
            With Battery_Scale_Ticks_Label(tick)
                .AutoSize = False
                .Height = temp_height
                .Width = temp_width
                .Location = New Point(.Location.X, CInt(.Location.Y - .Height / 2))
                .Text = Get_Percent_Discharged_String(tick * Me.range / SCALE_MAJOR_TICKS)
            End With
        Next tick

        For tick As Integer = 0 To SCALE_TICKS
            Dim major_tick As Boolean = (tick Mod (SCALE_TICKS / SCALE_MAJOR_TICKS)) = 0

            With Battery_Scale_Ticks_LineShape(tick)
                .BorderWidth = If(major_tick, SCALE_MAJOR_TICK_WIDTH, SCALE_TICK_WIDTH)
                .X1 = If(major_tick, 0, SCALE_MAJOR_TICK_LENGTH - SCALE_TICK_LENGTH) + Battery_Scale_Ticks_Label(SCALE_MAJOR_TICKS).Location.X + Battery_Scale_Ticks_Label(SCALE_MAJOR_TICKS).Width
                .X2 = .X1 + If(major_tick, SCALE_MAJOR_TICK_LENGTH, SCALE_TICK_LENGTH)
                .Y1 = CInt(Me.terminal_picturebox.Location.Y + Me.terminal_picturebox.Size.Height + BORDER_LINE_WIDTH + BORDER_LINE_WIDTH / 2 + tick * MAX_LINES / SCALE_TICKS)
                .Y2 = .Y1
                .SelectionColor = Color.Transparent
            End With
        Next tick

        ' Create coulomb counter scale at the far right of the panel

        ' Create the tick labels first
        For tick As Integer = 0 To SCALE_MAJOR_TICKS
            With Coulomb_Counter_Scale_Ticks_Label(tick)
                .Parent = Me.my_panel
                .AutoSize = True
                .BackColor = Color.Silver
                .Margin = New Padding(0)
                .Padding = New Padding(0)
                .Font = New Font("Linear Helv Cond", 8.0!, FontStyle.Regular)
                .Location = New Point(Me.my_panel.Width, Battery_Scale_Ticks_Label(tick).Location.Y)
                .TextAlign = ContentAlignment.MiddleLeft
                .Text = (tick * (MAX_LINES - 1) \ SCALE_MAJOR_TICKS).ToString("0")
            End With
            D3335.Set_Font(DirectCast(Coulomb_Counter_Scale_Ticks_Label(tick), Control))
        Next tick

        ' Now rescale the labels so that they're all the same height and width
        temp_height = Coulomb_Counter_Scale_Ticks_Label(SCALE_MAJOR_TICKS).Height
        temp_width = Coulomb_Counter_Scale_Ticks_Label(SCALE_MAJOR_TICKS).Width
        For tick As Integer = 0 To SCALE_MAJOR_TICKS
            With Coulomb_Counter_Scale_Ticks_Label(tick)
                .AutoSize = False
                .Height = temp_height
                .Width = temp_width
                .Location = New Point(.Location.X - .Width, .Location.Y)
            End With
        Next tick

        For tick As Integer = 0 To SCALE_TICKS
            Dim major_tick As Boolean = (tick Mod (SCALE_TICKS / SCALE_MAJOR_TICKS)) = 0

            With Coulomb_Counter_Scale_Ticks_LineShape(tick)
                .BorderWidth = If(major_tick, SCALE_MAJOR_TICK_WIDTH, SCALE_TICK_WIDTH)
                .X1 = Me.Coulomb_Counter_Scale_Ticks_Label(0).Location.X - SCALE_MAJOR_TICK_LENGTH
                .X2 = .X1 + If(major_tick, SCALE_MAJOR_TICK_LENGTH, SCALE_TICK_LENGTH)
                .Y1 = Me.Battery_Scale_Ticks_LineShape(tick).Y1
                .Y2 = .Y1
                .SelectionColor = Color.Transparent
            End With
        Next tick

        ' Create battery next to the scale centered around the PictureBox

        ' Create outline of battery
        With Me.Battery_Top_LineShape
            .Name = "Battery_Top_LineShape"
            .BorderWidth = BORDER_LINE_WIDTH
            .X1 = CInt(BORDER_LINE_WIDTH / 2 + Me.Battery_Scale_Ticks_LineShape(0).X2)
            .X2 = CInt(Me.Battery_Top_LineShape.X1 + 2 * (Me.terminal_picturebox.Location.X + Me.terminal_picturebox.Size.Width / 2 - Me.Battery_Top_LineShape.X1))
            .Y1 = CInt(Me.Battery_Scale_Ticks_LineShape(0).Y1 - BORDER_LINE_WIDTH / 2)
            .Y2 = Me.Battery_Top_LineShape.Y1
            .SelectionColor = Color.Transparent
        End With

        With Me.Battery_Bottom_LineShape
            .Name = "Battery_Bottom_LineShape"
            .BorderWidth = BORDER_LINE_WIDTH
            .X1 = Me.Battery_Top_LineShape.X1
            .X2 = Me.Battery_Top_LineShape.X2
            .Y1 = Me.Battery_Top_LineShape.Y1 + MAX_LINES + 2 * (BORDER_LINE_WIDTH \ 2)
            .Y2 = Me.Battery_Bottom_LineShape.Y1
            .Name = "Battery_Bottom_LineShape"
            .SelectionColor = Color.Transparent
        End With

        With Me.Battery_Left_LineShape
            .Name = "Battery_Left_LineShape"
            .BorderWidth = BORDER_LINE_WIDTH
            .X1 = Me.Battery_Top_LineShape.X1
            .X2 = Me.Battery_Bottom_LineShape.X1
            .Y1 = Me.Battery_Top_LineShape.Y1 - (BORDER_LINE_WIDTH \ 2)
            .Y2 = Me.Battery_Bottom_LineShape.Y1 + (BORDER_LINE_WIDTH \ 2)
            .SelectionColor = Color.Transparent
        End With

        With Me.Battery_Right_LineShape
            .Name = "Battery_Right_LineShape"
            .BorderWidth = BORDER_LINE_WIDTH
            .X1 = Me.Battery_Top_LineShape.X2
            .X2 = Me.Battery_Bottom_LineShape.X2
            .Y1 = Me.Battery_Top_LineShape.Y2 - (BORDER_LINE_WIDTH \ 2)
            .Y2 = Me.Battery_Bottom_LineShape.Y2 + (BORDER_LINE_WIDTH \ 2)
            .SelectionColor = Color.Transparent
        End With

        Me.my_panel.Parent.Controls.Add(Me.gnd_pictureBox)
        Me.my_panel.Controls.Add(Me.gnd_pictureBox)
        Me.gnd_pictureBox.Image = Global.LinearTechnology.DemoBoards.DC2343A.My.Resources.Resources.LTC3335_ground
        Me.gnd_pictureBox.Size = New Size(35, 31)
        Me.gnd_pictureBox.SizeMode = PictureBoxSizeMode.Zoom
        temp_point.X = CInt(Me.terminal_picturebox.Location.X + Me.terminal_picturebox.Size.Width / 2 - Me.gnd_pictureBox.Size.Width / 2)
        temp_point.Y = Me.Battery_Bottom_LineShape.Y2
        Me.gnd_pictureBox.Location = temp_point

        ' Next create Coulomb Counter

        ' Define the block that shows the extent of the Coloumb Counter, spaced from the text between the Battery and the Coulomb Counter.
        With Me.Coulomb_Counter_RectangleShape
            .Name = "Coulomb_Counter_RectangleShape"
            .BorderWidth = BORDER_LINE_WIDTH
            .Location = New Point(Me.Battery_Top_LineShape.X2 + MID_MARGIN, Me.Battery_Top_LineShape.Y2)
            .Height = Me.Battery_Bottom_LineShape.Y1 - Me.Battery_Top_LineShape.Y1
            .Width = CInt(Me.Coulomb_Counter_Scale_Ticks_LineShape(0).X1 - Me.Coulomb_Counter_RectangleShape.Location.X - BORDER_LINE_WIDTH / 2)
            .FillStyle = FillStyle.Solid
            .FillColor = Color.FromArgb(CType(CType(228, Byte), Integer), CType(CType(193, Byte), Integer), CType(CType(188, Byte), Integer))
            .SelectionColor = Color.Transparent
        End With

        ' Now define the rectangles and lines that show the capacity, charge, and alarm values.
        With Me.Battery_Capacity_RectangleShape
            .Name = "Battery_Capacity_RectangleShape"
            .BorderWidth = 0
            .Location = New Point(CInt(Me.Battery_Left_LineShape.X1 + BORDER_LINE_WIDTH / 2), Me.Battery_Left_LineShape.Y1 + BORDER_LINE_WIDTH)
            .Height = Me.Battery_Left_LineShape.Y2 - Me.Battery_Left_LineShape.Y1 - 2 * BORDER_LINE_WIDTH
            .Width = Me.Battery_Right_LineShape.X1 - Me.Battery_Left_LineShape.X1 - BORDER_LINE_WIDTH
            .FillStyle = FillStyle.Solid
            .FillColor = Color.LightGreen
            .SelectionColor = Color.Transparent
        End With

        With Me.Battery_Coulombs_Discharged_RectangleShape
            .Name = "Battery_Coulombs_Discharged_RectangleShape"
            .BorderWidth = 0
            .Location = Me.Battery_Capacity_RectangleShape.Location
            .Height = CInt(Me.Battery_Capacity_RectangleShape.Height * (STARTING_COULOMBS_DISCHARGED / 100))
            .Width = Me.Battery_Capacity_RectangleShape.Width
            .FillStyle = FillStyle.Solid
            .FillColor = Color.Silver
            .SelectionColor = Color.Transparent
        End With

        ' Now draw the lines showing the Capacity, Coulombs Discharged, and Alarm Values.
        ' Note that the Coulomb Counter lines are drawn from right to left, but the Battery lines are drawn from left to right.
        ' This forces a complete dash to be started at the edges of each rectangle.
        With Battery_Coulombs_Discharged_LineShape
            .Name = "Battery_Coulombs_Discharged_LineShape"
            .BorderColor = Color.Green
            .BorderStyle = Drawing2D.DashStyle.Dash
            .BorderWidth = CONTROL_LINE_WIDTH
            .X1 = CInt(Me.Battery_Bottom_LineShape.X1 - BORDER_LINE_WIDTH / 2 - SCALE_TICK_LENGTH)
            .X2 = CInt(Me.Battery_Right_LineShape.X2 + BORDER_LINE_WIDTH / 2 + SCALE_TICK_LENGTH)
            .Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + MAX_LINES * (STARTING_COULOMBS_DISCHARGED / 100))
            .Y2 = .Y1
            .SelectionColor = Color.Transparent
        End With

        With Battery_Alarm_LineShape
            .Name = "Battery_Alarm_LineShape"
            .BorderColor = Color.Red
            .BorderStyle = Drawing2D.DashStyle.Dash
            .BorderWidth = CONTROL_LINE_WIDTH
            .X1 = CInt(Me.Battery_Bottom_LineShape.X1 - BORDER_LINE_WIDTH / 2 - SCALE_TICK_LENGTH)
            .X2 = Me.Battery_Coulombs_Discharged_LineShape.X2
            .Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + MAX_LINES * (STARTING_ALARM / 100))
            .Y2 = .Y1
            .SelectionColor = Color.Transparent
        End With

        With Me.Coulomb_Counter_Capacity_LineShape
            .Name = "Coulomb_Counter_Capacity_LineShape"
            .BorderColor = Color.Black
            .BorderStyle = Drawing2D.DashStyle.Dash
            .BorderWidth = CONTROL_LINE_WIDTH
            .X1 = CInt(Me.Coulomb_Counter_RectangleShape.Location.X + Me.Coulomb_Counter_RectangleShape.Width + BORDER_LINE_WIDTH / 2 + SCALE_TICK_LENGTH)
            .X2 = CInt(Me.Battery_Bottom_LineShape.X2 + BORDER_LINE_WIDTH / 2 + SCALE_TICK_LENGTH)
            .Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + MAX_LINES)
            .Y2 = .Y1
            .SelectionColor = Color.Transparent
        End With

        With Me.Coulomb_Counter_Accumulator_LineShape
            .Name = "Coulomb_Counter_Accumulator_LineShape"
            .BorderColor = Color.Green
            .BorderStyle = Drawing2D.DashStyle.Dash
            .BorderWidth = CONTROL_LINE_WIDTH
            .X1 = Me.Coulomb_Counter_Capacity_LineShape.X1
            If use_software_correction = True Then
                .X2 = CInt(Me.Coulomb_Counter_RectangleShape.Location.X - BORDER_LINE_WIDTH / 2 - SCALE_TICK_LENGTH)
            Else
                .X2 = Me.Battery_Coulombs_Discharged_LineShape.X2
            End If
            .Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + MAX_LINES * (STARTING_COULOMBS_DISCHARGED / 100))
            .Y2 = .Y1
            .SelectionColor = Color.Transparent
        End With

        With Coulomb_Counter_Alarm_LineShape
            .Name = "Coulomb_Counter_Alarm_LineShape"
            .BorderColor = Color.Red
            .BorderStyle = Drawing2D.DashStyle.Dash
            .BorderWidth = CONTROL_LINE_WIDTH
            .X1 = Me.Coulomb_Counter_Accumulator_LineShape.X1
            .X2 = Me.Coulomb_Counter_Accumulator_LineShape.X2
            .Y1 = CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2 + MAX_LINES * (STARTING_ALARM / 100))
            .Y2 = .Y1
            .SelectionColor = Color.Transparent
        End With

        ' Create the transparent RectangleShapes so that we can drag lines between the Coulomb Counter and the Battery, and below the Battery.
        With Battery_Drag_Extents_RectangleShape
            .Name = "Battery_Drag_Extents_RectangleShape"
            .BorderWidth = 0
            .BorderStyle = Drawing2D.DashStyle.Custom
            .Location = New Point(Me.Battery_Coulombs_Discharged_LineShape.X1, CInt(Me.Battery_Top_LineShape.Y1 + BORDER_LINE_WIDTH / 2))
            .Height = MAX_LINES
            .Width = Me.Battery_Coulombs_Discharged_LineShape.X2 - Me.Battery_Coulombs_Discharged_LineShape.X1
            .FillStyle = FillStyle.Transparent
            .FillColor = Color.Transparent
            .SelectionColor = Color.Transparent
        End With

        With Coulomb_Counter_Drag_Extents_RectangleShape
            .Name = "Coulomb_Counter_Drag_Extents_RectangleShape"
            .BorderWidth = 0
            .BorderStyle = Drawing2D.DashStyle.Custom
            .Location = New Point(Me.Coulomb_Counter_Capacity_LineShape.X2, CInt(Me.Coulomb_Counter_RectangleShape.Location.Y + BORDER_LINE_WIDTH / 2))
            .Height = MAX_LINES
            .Width = Me.Coulomb_Counter_Capacity_LineShape.X1 - Me.Coulomb_Counter_Capacity_LineShape.X2
            .FillStyle = FillStyle.Transparent
            .FillColor = Color.Transparent
            .SelectionColor = Color.Transparent
        End With

        ' Add all of the shapes to the shape container and bring to the front
        Me.ShapeContainer.Parent = Me.my_panel
        Me.ShapeContainer.BringToFront()

        Me.Legend_ShapeContainer.Parent = Me.my_panel
        Me.Legend_ShapeContainer.BringToFront()

        'Bring objects to the front, in the order that we want them shown
        Me.Battery_Capacity_RectangleShape.BringToFront()
        Me.Battery_Coulombs_Discharged_RectangleShape.BringToFront()
        Me.Battery_Coulombs_Discharged_LineShape.BringToFront()
        Me.Battery_Alarm_LineShape.BringToFront()
        Me.Coulomb_Counter_Accumulator_LineShape.BringToFront()
        Me.Coulomb_Counter_Capacity_LineShape.BringToFront()
        Me.Coulomb_Counter_Alarm_LineShape.BringToFront()

        With Me.Battery_Capacity_Ah_TextBox
            .Name = "Battery_Capacity_Ah_TextBox"
            .Parent = Me.my_panel
            .Margin = New Padding(0)
            .MaxLength = Len("Coulombs Discharged")
            .Text = 100.ToString(PERCENT_DISCHARGED_FORMAT) & "% (168mAh)"
            .Width = 110
            .TextAlign = HorizontalAlignment.Center
            .ReadOnly = True
            .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Regular)
            temp_point.X = 0
            temp_point.Y = Me.my_panel.Height - .Height
            .Location = temp_point
            .TabStop = False
        End With
        D3335.Set_Font(DirectCast(Me.Battery_Capacity_Ah_TextBox, Control))
        D3335.Set_TextBox_Height_and_Width(DirectCast(Me.Battery_Capacity_Ah_TextBox, Control))

        With Me.Battery_Capacity_Ah_Label
            .Name = "Battery_Capacity_Ah_Label"
            .Parent = Me.my_panel
            .AutoSize = True
            .BackColor = Color.Silver
            .Margin = New Padding(0)
            .Padding = New Padding(0)
            .Text = "Battery" + ChrW(13) + ChrW(10) + "Capacity"
            .TextAlign = ContentAlignment.BottomLeft
            .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Bold)
            temp_point.X = 0
            temp_point.Y = Me.Battery_Capacity_Ah_TextBox.Location.Y - .Height
            .Location = temp_point
            .TabStop = False
        End With
        D3335.Set_Font(DirectCast(Me.Battery_Capacity_Ah_Label, Control))

        ' Create Textboxes and Labels created to show the Battery Coulombs and Alarm in Ah.
        With Me.Battery_Alarm_Ah_TextBox
            .Name = "Battery_Alarm_Ah_TextBox"
            .Parent = Me.my_panel
            .Margin = New Padding(0)
            .MaxLength = Battery_Capacity_Ah_TextBox.MaxLength
            .Text = STARTING_ALARM.ToString(PERCENT_DISCHARGED_FORMAT) & "% (39.2mAh)"
            .Width = Battery_Capacity_Ah_TextBox.Width
            .TextAlign = HorizontalAlignment.Center
            .ReadOnly = True
            .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Regular)
            temp_point.X = 0
            temp_point.Y = Me.Battery_Capacity_Ah_Label.Location.Y - .Height
            .Location = temp_point
            .TabStop = False
        End With
        D3335.Set_Font(DirectCast(Me.Battery_Alarm_Ah_TextBox, Control))
        D3335.Set_TextBox_Height_and_Width(DirectCast(Me.Battery_Alarm_Ah_TextBox, Control))

        With Me.Battery_Alarm_Ah_Label
            .Name = "Battery_Alarm_Ah_Label"
            .Parent = Me.my_panel
            .AutoSize = True
            .BackColor = Color.Silver
            .Margin = New Padding(0)
            .Padding = New Padding(0)
            .Text = "Alarm" + ChrW(13) + ChrW(10) + "Coulombs"
            .TextAlign = ContentAlignment.BottomLeft
            .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Bold)
            temp_point.X = 0
            temp_point.Y = Me.Battery_Alarm_Ah_TextBox.Location.Y - .Height
            .Location = temp_point
        End With
        D3335.Set_Font(DirectCast(Me.Battery_Alarm_Ah_Label, Control))

        With Me.Battery_Coulombs_Discharged_Ah_TextBox
            .Name = "Battery_Coulombs_Discharged_Ah_TextBox"
            .Parent = Me.my_panel
            .Margin = New Padding(0)
            .MaxLength = Battery_Alarm_Ah_TextBox.MaxLength
            .Text = STARTING_COULOMBS_DISCHARGED.ToString(PERCENT_DISCHARGED_FORMAT) & "% (16.8mAh)"
            .Width = Battery_Alarm_Ah_TextBox.Width
            .TextAlign = HorizontalAlignment.Center
            .ReadOnly = True
            .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Regular)
            temp_point.X = 0
            temp_point.Y = Me.Battery_Alarm_Ah_Label.Location.Y - .Height
            .Location = temp_point
            .TabStop = False
        End With
        D3335.Set_Font(DirectCast(Me.Battery_Coulombs_Discharged_Ah_TextBox, Control))
        D3335.Set_TextBox_Height_and_Width(DirectCast(Me.Battery_Coulombs_Discharged_Ah_TextBox, Control))

        With Me.Battery_Coulombs_Discharged_Ah_Label
            .Name = "Battery_Coulombs_Discharged_Ah_Label"
            .Parent = Me.my_panel
            .AutoSize = True
            .BackColor = Color.Silver
            .Margin = New Padding(0)
            .Padding = New Padding(0)
            .Text = "Coulombs" + ChrW(13) + ChrW(10) + "Discharged"
            .TextAlign = ContentAlignment.BottomLeft
            .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Bold)
            temp_point.X = 0
            temp_point.Y = Me.Battery_Coulombs_Discharged_Ah_TextBox.Location.Y - .Height
            .Location = temp_point
        End With
        D3335.Set_Font(DirectCast(Me.Battery_Coulombs_Discharged_Ah_Label, Control))

        ' Create the lines that create the key for graphic.
        With Battery_Alarm_Ah_LineShape
            .Name = "Battery_Alarm_Ah_LineShape"
            .BorderColor = Color.Red
            .BorderStyle = Drawing2D.DashStyle.Dash
            .BorderWidth = CONTROL_LINE_WIDTH
            .X1 = Me.Battery_Alarm_Ah_Label.Location.X + Me.Battery_Alarm_Ah_Label.Width
            .X2 = Me.Battery_Alarm_Ah_Label.Location.X + Me.Battery_Alarm_Ah_TextBox.Width
            .Y1 = CInt(Me.Battery_Alarm_Ah_Label.Location.Y + Me.Battery_Alarm_Ah_Label.Height * 3 / 4)
            .Y2 = .Y1
            .SelectionColor = Color.Transparent
        End With

        With Battery_Coulombs_Discharged_Ah_LineShape
            .Name = "Battery_Coulombs_Discharged_Ah_LineShape"
            .BorderColor = Color.Green
            .BorderStyle = Drawing2D.DashStyle.Dash
            .BorderWidth = CONTROL_LINE_WIDTH
            .X1 = Me.Battery_Coulombs_Discharged_Ah_Label.Location.X + Me.Battery_Coulombs_Discharged_Ah_Label.Width
            .X2 = Me.Battery_Coulombs_Discharged_Ah_Label.Location.X + Me.Battery_Coulombs_Discharged_Ah_TextBox.Width
            .Y1 = CInt(Me.Battery_Coulombs_Discharged_Ah_Label.Location.Y + Me.Battery_Coulombs_Discharged_Ah_Label.Height * 3 / 4)
            .Y2 = .Y1
            .SelectionColor = Color.Transparent
        End With

        ' Create the lines that create the key for graphic.
        With Battery_Capacity_Ah_LineShape
            .Name = "Battery_Capacity_Ah_LineShape"
            .BorderColor = Color.Black
            .BorderStyle = Drawing2D.DashStyle.Dash
            .BorderWidth = CONTROL_LINE_WIDTH
            .X1 = Me.Battery_Capacity_Ah_Label.Location.X + Me.Battery_Capacity_Ah_Label.Width
            .X2 = Me.Battery_Capacity_Ah_Label.Location.X + Me.Battery_Capacity_Ah_TextBox.Width
            .Y1 = CInt(Me.Battery_Capacity_Ah_Label.Location.Y + Me.Battery_Capacity_Ah_Label.Height * 3 / 4)
            .Y2 = .Y1
            .SelectionColor = Color.Transparent
        End With

        ' Create Textboxes and Labels created to show the Coluomb Comunter Range and Resolution.
        D3335.Set_TextBox_Height_and_Width(DirectCast(Me.Coulomb_Counter_Range_TextBox, Control))
        With Me.Coulomb_Counter_Range_TextBox
            .Name = "Coulomb_Counter_Range_TextBox"
            .Parent = Me.my_panel
            .Margin = New Padding(0)
            .MaxLength = Battery_Capacity_Ah_TextBox.MaxLength
            .Text = "121% (111mAh)"
            .Width = Battery_Capacity_Ah_TextBox.Width
            .TextAlign = HorizontalAlignment.Center
            .ReadOnly = True
            .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Regular)
            temp_point.X = my_panel.Width - .Width
            temp_point.Y = Me.Battery_Alarm_Ah_TextBox.Location.Y
            .Location = temp_point
            .TabStop = False
        End With
        D3335.Set_Font(DirectCast(Me.Coulomb_Counter_Range_TextBox, Control))

        With Me.Coulomb_Counter_Range_Label
            .Name = "Coulomb_Counter_Range_Label"
            .Parent = Me.my_panel
            .AutoSize = True
            .BackColor = Color.Silver
            .Margin = New Padding(0)
            .Padding = New Padding(0)
            .Text = "LTC3335" + ChrW(13) + ChrW(10) + "Range"
            .TextAlign = ContentAlignment.BottomCenter
            .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Bold)
            temp_point.X = CInt(Me.Coulomb_Counter_Range_TextBox.Location.X + (Me.Coulomb_Counter_Range_TextBox.Width - .Width) / 2)
            temp_point.Y = Me.Coulomb_Counter_Range_TextBox.Location.Y - .Height
            .Location = temp_point
        End With
        D3335.Set_Font(DirectCast(Me.Coulomb_Counter_Range_Label, Control))

        D3335.Set_TextBox_Height_and_Width(DirectCast(Me.Coulomb_Counter_Resolution_TextBox, Control))
        With Me.Coulomb_Counter_Resolution_TextBox
            .Name = "Coulomb_Counter_Resolution_TextBox"
            .Parent = Me.my_panel
            .Margin = New Padding(0)
            .MaxLength = Battery_Coulombs_Discharged_Ah_TextBox.MaxLength
            .Text = "0.78% (434µAh)"
            .Width = Battery_Alarm_Ah_TextBox.Width
            .TextAlign = HorizontalAlignment.Center
            .ReadOnly = True
            .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Regular)
            temp_point.X = Me.Coulomb_Counter_Range_TextBox.Location.X
            temp_point.Y = Me.Battery_Coulombs_Discharged_Ah_TextBox.Location.Y
            .Location = temp_point
            .TabStop = False
        End With
        D3335.Set_Font(DirectCast(Me.Coulomb_Counter_Resolution_TextBox, Control))

        With Me.Coulomb_Counter_Resolution_Label
            .Name = "Coulomb_Counter_Resolution_Label"
            .Parent = Me.my_panel
            .AutoSize = True
            .BackColor = Color.Silver
            .Margin = New Padding(0)
            .Padding = New Padding(0)
            .Text = "LTC3335" + ChrW(13) + ChrW(10) + "Resolution"
            .TextAlign = ContentAlignment.BottomCenter
            .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Bold)
            temp_point.X = CInt(Me.Coulomb_Counter_Resolution_TextBox.Location.X + (Me.Coulomb_Counter_Resolution_TextBox.Width - .Width) / 2)
            temp_point.Y = Me.Coulomb_Counter_Resolution_TextBox.Location.Y - .Height
            .Location = temp_point
        End With
        D3335.Set_Font(DirectCast(Me.Coulomb_Counter_Resolution_Label, Control))

    End Sub

End Class