﻿Option Strict On
Option Explicit On

Imports Microsoft.VisualBasic
Imports System.Windows.Forms
Imports System.Drawing

Public Class Battery_Selection

    Public WithEvents Prescaler_Selection_CenteredComboBox As CenteredComboBox ' Public as its used by the Data Log and Pipe Server.  todo - clean that up
    Public battery_cc_graphic As Battery_Coulomb_Counter_Graphic ' Public as its used by the Data Log and Pipe Server.  todo - clean that up
    Private usb_comm As D3335_USB
    Private tooltip As ToolTip

    Private Ipeak_Selection As LTC3335.IPEAK_SELECTION_OPTIONS
    Private handle_exception_callback As Action(Of Exception)
    Private ok_button_callback As Action(Of Integer, Double, Double, Double, Boolean, Single)

    Private OK_COLOR As Color = SystemColors.WindowText
    Private ERROR_COLOR As Color = Color.Red

    ' Popup window should always be on top of GUI Control Panel, but not always on Top of all Windows programs.
    Private Declare Function BringWindowToTop Lib "user32" Alias "BringWindowToTop" (ByVal hwnd As Long) As Long
    Private Sub Battery_Selection_Activated(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        BringWindowToTop(CLng(Me.Handle))
    End Sub

    Friend Sub New(ByVal usb_comm As D3335_USB,
                   Optional ByVal handle_exception_callback As Action(Of Exception) = Nothing,
                   Optional ByVal ok_button_callback As Action(Of Integer, Double, Double, Double, Boolean, Single) = Nothing)

        InitializeComponent()

        Me.usb_comm = usb_comm
        Me.handle_exception_callback = handle_exception_callback
        Me.ok_button_callback = ok_button_callback

        ' Populate the combo boxes if they haven't been already
        If Me.Prescaler_Selection_ComboBox.Items.Count < (LTC3335.PRESCALER_SELECTION_MAX - LTC3335.PRESCALER_SELECTION_MIN + 1) Then
            For prescaler_selection As Integer = LTC3335.PRESCALER_SELECTION_MIN To LTC3335.PRESCALER_SELECTION_MAX
                Me.Prescaler_Selection_ComboBox.Items.Add(prescaler_selection.ToString())
            Next prescaler_selection
        End If

        ' Turn the combobox into a centered combobox if it hasn't been already.
        If Prescaler_Selection_CenteredComboBox Is Nothing Then
            Prescaler_Selection_CenteredComboBox = New CenteredComboBox(Prescaler_Selection_ComboBox)
            Prescaler_Selection_CenteredComboBox.TabStop = True
            Prescaler_Selection_CenteredComboBox.TabIndex = Battery_Alarm_Percent_Discharged_TextBox.TabIndex + 1
            Prescaler_Selection_ComboBox.Visible = False
        End If

        tooltip = New ToolTip

        Dim tooltip_msg As String = "Software Correction to be Enabled in Future GUI Version"
        tooltip.SetToolTip(Me.Software_Correction_CheckBox, tooltip_msg)
        tooltip.SetToolTip(Me.Software_Correction_CheckBox_Label, tooltip_msg)

        D3335.Set_TextBox_Height_and_Width(DirectCast(Me.Correction_Factor_Value_TextBox, Control))

    End Sub

    Friend Overloads Sub Show(ByVal Ipeak_Selection As LTC3335.IPEAK_SELECTION_OPTIONS,
                          Optional ByVal battery_capacity_ah As Double = 0,
                          Optional ByVal battery_coulombs_discharged_ah As Double = 0,
                          Optional ByVal alarm_ah As Double = 255,
                          Optional ByVal use_software_correction As Boolean = False,
                          Optional ByVal software_correction_factor As Single = 0.0,
                          Optional ByVal prescaler As Integer = -1)

        ' Save the ipeak selection, as we'll need it to recalculate values each time the prescaler is changed.
        Me.Ipeak_Selection = Ipeak_Selection

        ' Calculate the default battery capacity if not passed in (should only happen once in the factory)
        If battery_capacity_ah <= 0 Then       ' 0 is not valid battery capacity.  Use the default instead.
            If prescaler >= 0 Then
                battery_capacity_ah = LTC3335.Get_Charge_Range(Me.Ipeak_Selection, prescaler)
            Else
                battery_capacity_ah = LTC3335.Get_Charge_Range(Me.Ipeak_Selection, LTC3335.PRESCALER_SELECTION_MIN)
            End If

            ' Initialize to some value.  This should rarely be used outside of the factory.
            battery_coulombs_discharged_ah = battery_capacity_ah * 50 / 100
            alarm_ah = battery_capacity_ah * 90 / 100

        End If

        ' Calculate the best prescaler if it wasn't passed in.
        If (prescaler < 0) Then
            prescaler = LTC3335.Get_Prescaler(Me.Ipeak_Selection, CSng(battery_capacity_ah), use_software_correction, software_correction_factor)
        End If

        ' Create the battery control from the values passed in
        If Me.battery_cc_graphic Is Nothing Then
            ' todo - I can't seem to loop through all forms, so copy this code to each form for now.
            ' First set the Use Compatible Text Rendering property.
            For Each control As Control In MyBase.Controls
                D3335.Set_UseCompatibleTextRendering(control)
            Next control

            ' Next set all of the fonts to Linear Helv Cond.
            For Each control As Control In MyBase.Controls
                D3335.Set_Font(control)
            Next control

            Me.battery_cc_graphic = New Battery_Coulomb_Counter_Graphic(Battery_Control_Panel, Battery_Popup_Label, New ToolTip, _
                                                     LTC3335.Get_Charge_Resolution(Me.Ipeak_Selection, prescaler), _
                                                     Nothing, AddressOf Drag_CallBack, _
                                                     battery_capacity_ah, battery_coulombs_discharged_ah, alarm_ah, _
                                                     use_software_correction, software_correction_factor,
                                                     maintain_percent:=True,
                                                     debug_textbox:=D3335.Debug2_TextBox)
        Else
            Me.battery_cc_graphic.Software_Correction_Set(use_software_correction, software_correction_factor)
            Me.battery_cc_graphic.Resolution_Set(LTC3335.Get_Charge_Resolution(Me.Ipeak_Selection, prescaler))
            Me.battery_cc_graphic.Battery_Capacity_Set(battery_capacity_ah)
            Me.battery_cc_graphic.Battery_Coulombs_Discharged_Set(battery_coulombs_discharged_ah)
            Me.battery_cc_graphic.Alarm_Set(alarm_ah)
        End If

        ' If the Alarm and Coulombs Discharged were initially set below the capacity, do not pick the best prescaler for the given capacity but the highest prescaler
        ' that allows the Alarm and Coulombs Discharged to have real values.
        While ((Me.battery_cc_graphic.alarm_ah > Me.battery_cc_graphic.range) Or (Me.battery_cc_graphic.battery_coulombs_discharged_ah > Me.battery_cc_graphic.range)) And (prescaler > 0)
            prescaler -= 1
            Me.battery_cc_graphic.Resolution_Set(LTC3335.Get_Charge_Resolution(Me.Ipeak_Selection, prescaler))
        End While

        ' Suggest a prescaler if it seems like the LTC3335 just powered up.
        If (Me.battery_cc_graphic.alarm_ah = Me.battery_cc_graphic.range) And (Me.battery_cc_graphic.battery_coulombs_discharged_ah = 0) Then
            prescaler = LTC3335.Get_Prescaler(Me.Ipeak_Selection, CSng(battery_capacity_ah), use_software_correction, software_correction_factor)
            Me.battery_cc_graphic.Resolution_Set(LTC3335.Get_Charge_Resolution(Me.Ipeak_Selection, prescaler))
            Me.battery_cc_graphic.Alarm_Set(Me.battery_cc_graphic.range)
        End If

        ' Set the Software Correction Controls
        Software_Correction_CheckBox.Checked = use_software_correction
        Correction_Factor_Value_TextBox.Text = software_correction_factor.ToString(D3335.CC_ERROR_FORMAT) + "%"

        ' Populate the Battery Characteristics textboxes
        Battery_Capacity_TextBox.Text = get_coulombs_ah_string(battery_capacity_ah)
        Battery_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(Me.battery_cc_graphic.battery_coulombs_discharged_ah, False)
        bound_textbox(Battery_Percent_Discharged_TextBox, 100, True)
        Battery_Alarm_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(Me.battery_cc_graphic.alarm_ah, False)
        bound_textbox(Battery_Alarm_Percent_Discharged_TextBox, 100, True)

        ' Calculate the coulomb counter settings from the battery characteristics
        Prescaler_Selection_CenteredComboBox.SelectedIndex = prescaler
        Accumulator_TextBox.Text = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.battery_cc_graphic.battery_coulombs_discharged_ah).ToString()
        bound_textbox(Accumulator_TextBox, D3335_USB.ACCUMULATOR_MAX)
        Alarm_Threshold_TextBox.Text = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.battery_cc_graphic.alarm_ah).ToString()
        bound_textbox(Alarm_Threshold_TextBox, D3335_USB.ALARM_THRESHOLD_MAX)

        Me.AcceptButton = OK_Button

        Me.Show()

        Me.Battery_Capacity_TextBox.Focus()

    End Sub

    Private Sub BatterySelection_FormClosing(ByVal sender As Object, ByVal e As FormClosingEventArgs) Handles Me.FormClosing

        If e.CloseReason = CloseReason.UserClosing Then
            e.Cancel = True
            Me.Hide()
        End If

    End Sub

    Friend Sub OK_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OK_Button.Click

        ' Call callback to pass the new battery configuration to the parent form.
        If Not ok_button_callback Is Nothing Then
            ok_button_callback(Prescaler_Selection_CenteredComboBox.SelectedIndex, _
                               Me.battery_cc_graphic.battery_capacity_ah, Me.battery_cc_graphic.battery_coulombs_discharged_ah, Me.battery_cc_graphic.alarm_ah, _
                               Me.battery_cc_graphic.use_software_correction, Me.battery_cc_graphic.software_correction_factor)
        End If

        ' Hide the popup
        Me.Hide()
    End Sub

    Friend Sub Cancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cancel_Button.Click
        ' Revert this popup to show the Battery Selection before the user started to change controls.

        ' Hide the popup
        Me.Hide()
    End Sub

    Private Sub TextBox_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Battery_Percent_Discharged_TextBox.TextChanged, _
        Battery_Alarm_Percent_Discharged_TextBox.TextChanged, Battery_Capacity_TextBox.TextChanged, Accumulator_TextBox.TextChanged, Alarm_Threshold_TextBox.TextChanged
        Me.AcceptButton = Nothing
    End Sub

    Private Sub TextBox_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Battery_Percent_Discharged_TextBox.Click, _
        Battery_Alarm_Percent_Discharged_TextBox.Click, Battery_Capacity_TextBox.Click, Accumulator_TextBox.Click, Alarm_Threshold_TextBox.Click
        Dim textbox As TextBox = DirectCast(sender, TextBox)
        If textbox.ForeColor = Me.ERROR_COLOR Then
            textbox.Text = ""
            textbox.ForeColor = Me.OK_COLOR
        End If
        Me.AcceptButton = Nothing
    End Sub

    Private Sub TextBox_KeyPress(ByVal sender As System.Object, ByVal e As KeyPressEventArgs) Handles Battery_Percent_Discharged_TextBox.KeyPress, _
        Battery_Alarm_Percent_Discharged_TextBox.KeyPress, Battery_Capacity_TextBox.KeyPress, Accumulator_TextBox.KeyPress, Alarm_Threshold_TextBox.KeyPress
        If e.KeyChar = Convert.ToChar(Keys.Enter) Then
            Battery_Selection.ActiveForm.Focus()
        End If
    End Sub

    Friend Sub Battery_Capacity_TextBox_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Battery_Capacity_TextBox.LostFocus
        Dim battery_capacity_last As Double = Me.battery_cc_graphic.battery_capacity_ah

        If Battery_Capacity_TextBox.Text <> "" Then
            Try
                ' Bounds Check the setting
                Dim battery_capacity As Single = CSng(Battery_Capacity_TextBox.Text)
                Dim max_capacity As Single = LTC3335.Get_Charge_Range(Me.Ipeak_Selection, LTC3335.PRESCALER_SELECTION_MIN)
                Dim min_capacity As Single = 0

                If battery_capacity > max_capacity Then
                    battery_capacity = max_capacity
                ElseIf battery_capacity < min_capacity Then
                    battery_capacity = min_capacity
                End If
                Battery_Capacity_TextBox.Text = get_coulombs_ah_string(battery_capacity)

                ' Change the other GUI elements that depend upon the Battery_Capacity

                ' Keep the % coulombs discharged the same before and after the capacity changes
                Dim cell_charge_percent_discharged As Single = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Percent_Discharged(Me.battery_cc_graphic.battery_coulombs_discharged_ah)
                Dim alarm_percent_discharged As Single = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Percent_Discharged(Me.battery_cc_graphic.alarm_ah)

                Me.battery_cc_graphic.Battery_Capacity_Set(battery_capacity)
                Me.battery_cc_graphic.Battery_Coulombs_Discharged_Set(Me.battery_cc_graphic.Percent_Discharged_To_Coulombs_Discharged_Ah(cell_charge_percent_discharged))
                Me.battery_cc_graphic.Alarm_Set(Me.battery_cc_graphic.Percent_Discharged_To_Coulombs_Discharged_Ah(alarm_percent_discharged))

                Dim prescaler As Integer = LTC3335.Get_Prescaler(Me.Ipeak_Selection, battery_capacity, Me.battery_cc_graphic.use_software_correction, Me.battery_cc_graphic.software_correction_factor)
                ' Change the Prescaler value and force event to fire and update the rest of the dependent fields.
                If Prescaler_Selection_CenteredComboBox.SelectedIndex <> prescaler Then
                    Prescaler_Selection_CenteredComboBox.SelectedIndex = prescaler
                Else
                    Prescaler_Selection_CenteredComboBox_SelectedIndexChanged(sender, e)
                End If

            Catch ex As Exception
                Handle_Exception(ex)
                ' Number couldn't be converted, revert back to old value
                Battery_Capacity_TextBox.Text = get_coulombs_ah_string(battery_capacity_last)
            End Try
        Else
            ' Number couldn't be converted, revert back to old value
            Battery_Capacity_TextBox.Text = get_coulombs_ah_string(battery_capacity_last)
        End If

        Me.AcceptButton = OK_Button
    End Sub

    Friend Sub Battery_Percent_Discharged_TextBox_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Battery_Percent_Discharged_TextBox.LostFocus

        If Battery_Percent_Discharged_TextBox.Text <> "" Then
            Try
                ' Convert percent to ah to compare to old reading
                Dim battery_coulombs_discharged_ah As Single = Me.battery_cc_graphic.Percent_Discharged_To_Coulombs_Discharged_Ah(CSng(Battery_Percent_Discharged_TextBox.Text))

                ' Bounds Check the setting
                Dim battery_coulombs_discharged_ah_max As Single = Me.battery_cc_graphic.range
                Dim battery_coulombs_discharged_ah_min As Single = 0

                If battery_coulombs_discharged_ah <> Me.battery_cc_graphic.battery_coulombs_discharged_ah Then
                    If battery_coulombs_discharged_ah > battery_coulombs_discharged_ah_max Then
                        battery_coulombs_discharged_ah = battery_coulombs_discharged_ah_max
                    ElseIf battery_coulombs_discharged_ah < battery_coulombs_discharged_ah_min Then
                        battery_coulombs_discharged_ah = battery_coulombs_discharged_ah_min
                    End If

                    Me.battery_cc_graphic.Battery_Coulombs_Discharged_Set(battery_coulombs_discharged_ah)
                    Battery_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(battery_coulombs_discharged_ah, False)
                    bound_textbox(Battery_Percent_Discharged_TextBox, 100, True)
                    Accumulator_TextBox.Text = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Ltc3335_Counts(battery_coulombs_discharged_ah).ToString
                    bound_textbox(Accumulator_TextBox, D3335_USB.ACCUMULATOR_MAX)
                End If
            Catch ex As Exception
                Handle_Exception(ex)
                ' Number couldn't be converted, revert back to old value
                Battery_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(Me.battery_cc_graphic.battery_coulombs_discharged_ah, False)
                bound_textbox(Battery_Percent_Discharged_TextBox, 100, True)
            End Try
        Else
            ' Number couldn't be converted, revert back to old value
            Battery_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(Me.battery_cc_graphic.battery_coulombs_discharged_ah, False)
            bound_textbox(Battery_Percent_Discharged_TextBox, 100, True)
        End If

        Me.AcceptButton = OK_Button

    End Sub

    Friend Sub Battery_Alarm_Percent_Discharged_TextBox_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Battery_Alarm_Percent_Discharged_TextBox.LostFocus

        If Battery_Alarm_Percent_Discharged_TextBox.Text <> "" Then
            Try
                ' Bounds Check the setting
                Dim alarm_coulombs_discharged_ah As Single = Me.battery_cc_graphic.Percent_Discharged_To_Coulombs_Discharged_Ah(CSng(Battery_Alarm_Percent_Discharged_TextBox.Text))
                Dim alarm_coulombs_discharged_ah_max As Single = Me.battery_cc_graphic.range
                Dim alarm_coulombs_discharged_ah_min As Single = 0

                If alarm_coulombs_discharged_ah <> Me.battery_cc_graphic.alarm_ah Then
                    If alarm_coulombs_discharged_ah > alarm_coulombs_discharged_ah_max Then
                        alarm_coulombs_discharged_ah = alarm_coulombs_discharged_ah_max
                    ElseIf alarm_coulombs_discharged_ah < alarm_coulombs_discharged_ah_min Then
                        alarm_coulombs_discharged_ah = alarm_coulombs_discharged_ah_min
                    End If

                    Me.battery_cc_graphic.Alarm_Set(alarm_coulombs_discharged_ah)
                    Battery_Alarm_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(alarm_coulombs_discharged_ah, False)
                    bound_textbox(Battery_Alarm_Percent_Discharged_TextBox, 100, True)
                    Alarm_Threshold_TextBox.Text = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.battery_cc_graphic.alarm_ah).ToString
                    bound_textbox(Alarm_Threshold_TextBox, D3335_USB.ALARM_THRESHOLD_MAX)

                End If
            Catch ex As Exception
                Handle_Exception(ex)
                ' Number couldn't be converted, revert back to old value
                Battery_Alarm_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(Me.battery_cc_graphic.alarm_ah, False)
                bound_textbox(Battery_Alarm_Percent_Discharged_TextBox, 100, True)
            End Try
        Else
            ' Number couldn't be converted, revert back to old value
            Battery_Alarm_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(Me.battery_cc_graphic.alarm_ah, False)
            bound_textbox(Battery_Alarm_Percent_Discharged_TextBox, 100, True)
        End If

        Me.AcceptButton = OK_Button

    End Sub

    Friend Sub Prescaler_Selection_CenteredComboBox_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Prescaler_Selection_CenteredComboBox.SelectedIndexChanged

        ' Set the resolution and range for this prescaler
        Me.battery_cc_graphic.Resolution_Set(LTC3335.Get_Charge_Resolution(Me.Ipeak_Selection, Prescaler_Selection_CenteredComboBox.SelectedIndex))

        Accumulator_TextBox.Text = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.battery_cc_graphic.battery_coulombs_discharged_ah).ToString()
        bound_textbox(Accumulator_TextBox, D3335_USB.ACCUMULATOR_MAX)
        Alarm_Threshold_TextBox.Text = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.battery_cc_graphic.alarm_ah).ToString()
        bound_textbox(Alarm_Threshold_TextBox, D3335_USB.ALARM_THRESHOLD_MAX)

        Me.AcceptButton = OK_Button

    End Sub

    Friend Sub Accumulator_TextBox_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Accumulator_TextBox.LostFocus
        Dim accumulator_last As Integer = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.battery_cc_graphic.battery_coulombs_discharged_ah)
        If Accumulator_TextBox.Text <> "" Then
            Try
                ' Bounds Check the setting
                Dim accumulator As Integer = CInt(Accumulator_TextBox.Text)

                If accumulator > D3335_USB.ACCUMULATOR_MAX Then
                    accumulator = D3335_USB.ACCUMULATOR_MAX
                ElseIf accumulator < D3335_USB.ACCUMULATOR_MIN Then
                    accumulator = D3335_USB.ACCUMULATOR_MIN
                End If
                Accumulator_TextBox.Text = accumulator.ToString()
                bound_textbox(Accumulator_TextBox, D3335_USB.ACCUMULATOR_MAX)

                ' Change the other GUI elements that depend upon the Accumulator
                Me.battery_cc_graphic.Battery_Coulombs_Discharged_Set(Me.battery_cc_graphic.Ltc3335_Counts_To_Coulombs_Discharged_Ah(accumulator))
                Battery_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(Me.battery_cc_graphic.battery_coulombs_discharged_ah, False)
                bound_textbox(Battery_Percent_Discharged_TextBox, 100, True)

            Catch ex As Exception
                Handle_Exception(ex)
                ' Number couldn't be converted, revert back to old value
                Accumulator_TextBox.Text = accumulator_last.ToString()
                bound_textbox(Accumulator_TextBox, D3335_USB.ACCUMULATOR_MAX)
            End Try
        Else
            ' Number couldn't be converted, revert back to old value
            Accumulator_TextBox.Text = accumulator_last.ToString()
            bound_textbox(Accumulator_TextBox, D3335_USB.ACCUMULATOR_MAX)
        End If

        Me.AcceptButton = OK_Button
    End Sub

    Friend Sub Alarm_Threshold_TextBox_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Alarm_Threshold_TextBox.LostFocus
        Dim alarm_threshold_last As Integer = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Ltc3335_Counts(Me.battery_cc_graphic.alarm_ah)
        If Alarm_Threshold_TextBox.Text <> "" Then
            Try
                ' Bounds Check the setting
                Dim alarm_threshold As Integer = CInt(Alarm_Threshold_TextBox.Text)

                If alarm_threshold > D3335_USB.ALARM_THRESHOLD_MAX Then
                    alarm_threshold = D3335_USB.ALARM_THRESHOLD_MAX
                ElseIf alarm_threshold < D3335_USB.ALARM_THRESHOLD_MIN Then
                    alarm_threshold = D3335_USB.ALARM_THRESHOLD_MIN
                End If
                Alarm_Threshold_TextBox.Text = alarm_threshold.ToString()
                bound_textbox(Alarm_Threshold_TextBox, D3335_USB.ALARM_THRESHOLD_MAX)

                ' Change the other GUI elements that depend upon the Alarm Threshold
                Me.battery_cc_graphic.Alarm_Set(Me.battery_cc_graphic.Ltc3335_Counts_To_Coulombs_Discharged_Ah(alarm_threshold))
                Battery_Alarm_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(Me.battery_cc_graphic.alarm_ah, False)
                bound_textbox(Battery_Alarm_Percent_Discharged_TextBox, 100, True)

            Catch ex As Exception
                Handle_Exception(ex)
                ' Number couldn't be converted, revert back to old value
                Alarm_Threshold_TextBox.Text = alarm_threshold_last.ToString()
                bound_textbox(Alarm_Threshold_TextBox, D3335_USB.ALARM_THRESHOLD_MAX)
            End Try
        Else
            ' Number couldn't be converted, revert back to old value
            Alarm_Threshold_TextBox.Text = alarm_threshold_last.ToString()
            bound_textbox(Alarm_Threshold_TextBox, D3335_USB.ALARM_THRESHOLD_MAX)
        End If

        Me.AcceptButton = OK_Button
    End Sub

    ' Callback Function when the Battery Control has been used to change the battery parameters, and they need to be copied back to the textboxes.
    Friend Sub Drag_CallBack(ByVal battery_capacity_ah As Double, ByVal battery_coulombs_discharged_ah As Double, ByVal alarm_ah As Double)

        ' Populate the Battery Characteristics textboxes
        Battery_Capacity_TextBox.Text = get_coulombs_ah_string(battery_capacity_ah)
        Battery_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(Me.battery_cc_graphic.battery_coulombs_discharged_ah, False)
        bound_textbox(Battery_Percent_Discharged_TextBox, 100, True)
        Battery_Alarm_Percent_Discharged_TextBox.Text = Me.battery_cc_graphic.Get_Percent_Discharged_String(Me.battery_cc_graphic.alarm_ah, False)
        bound_textbox(Battery_Alarm_Percent_Discharged_TextBox, 100, True)

        ' Calculate the ltc_settings from the battery characteristics
        ' Prescaler_Selection_CenteredComboBox.SelectedIndex = prescaler
        Accumulator_TextBox.Text = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Ltc3335_Counts(battery_coulombs_discharged_ah).ToString()
        bound_textbox(Accumulator_TextBox, D3335_USB.ACCUMULATOR_MAX)
        Alarm_Threshold_TextBox.Text = Me.battery_cc_graphic.Coulombs_Discharged_Ah_To_Ltc3335_Counts(alarm_ah).ToString()
        bound_textbox(Alarm_Threshold_TextBox, D3335_USB.ALARM_THRESHOLD_MAX)

        Me.AcceptButton = OK_Button

    End Sub

    Private Function get_coulombs_ah_string(ByVal charge As Double) As String
        Dim charge_format As String

        If charge > 1 Then
            charge_format = "##0.000"
        Else
            charge_format = "0.000000"
        End If

        Return charge.ToString(charge_format)
    End Function

    Private Sub bound_textbox(ByRef textbox As TextBox, ByVal max_value As Integer, Optional ByVal color_only As Boolean = False)
        If (CDbl(textbox.Text.Replace("<", "")) <= max_value) Then
            With textbox
                .ForeColor = Me.OK_COLOR
                .Font = CustomFont.GetFont(.Font.Name, .Font.Size, FontStyle.Regular)
            End With

        Else
            With textbox
                If color_only = False Then
                    .Text = "Out of Range"
                End If
                .ForeColor = Me.ERROR_COLOR
                .Font = CustomFont.GetFont(.Font.Name, .Font.Size, FontStyle.Bold)
            End With
        End If
    End Sub

    Private Sub Handle_Exception(ByVal ex As Exception)
        If Not handle_exception_callback Is Nothing Then
            handle_exception_callback(ex)
        End If
    End Sub

    Private Sub Software_Correction_CheckBox_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Software_Correction_CheckBox.CheckedChanged

        If Me.Software_Correction_CheckBox.Checked Then
            Me.Correction_Factor_Label.Text = "Coulomb Count Error Corrected:"
            Me.Correction_Factor_Value_TextBox.BackColor = Color.PaleGoldenrod
        Else
            Me.Correction_Factor_Label.Text = "Typical Coulomb Count Error:"
            Me.Correction_Factor_Value_TextBox.BackColor = SystemColors.Control
        End If

        Me.battery_cc_graphic.Software_Correction_Set(Software_Correction_CheckBox.Checked, CSng(Correction_Factor_Value_TextBox.Text.Replace("%", "")))

        ' Recalculate everything that depends upon the capacity
        Battery_Capacity_TextBox_LostFocus(Battery_Capacity_TextBox, New EventArgs)

    End Sub

End Class
