﻿Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices
Imports System.Drawing

Public Class CenteredComboBox
    Inherits ComboBox

    Public ClickedIndex As Integer ' The combobox SelectedIndex actually changes as each selection is highlighted (but not selected by the user!)
    Private MyFont As Font

    Sub New()
        Me.DrawMode = Windows.Forms.DrawMode.OwnerDrawFixed
        Me.DropDownStyle = ComboBoxStyle.DropDownList
        Me.ClickedIndex = -1
    End Sub

    Sub New(ByVal combobox As ComboBox)
        Me.New()
        Me.Size = combobox.Size
        Me.Location = combobox.Location
        Me.BackColor = combobox.BackColor
        Me.Font = combobox.Font
        Me.FormattingEnabled = combobox.FormattingEnabled
        Me.Parent = combobox.Parent
        Me.MyFont = CustomFont.GetFont("Linear Helv Cond", Me.Font.Size, FontStyle.Regular)

        For index As Integer = 0 To combobox.Items.Count - 1
            Me.Items.Add(combobox.Items(index))
        Next index
        Me.BringToFront()
    End Sub

    Private Sub CenteredComboBox_DrawItem(ByVal sender As Object, ByVal e As System.Windows.Forms.DrawItemEventArgs) Handles Me.DrawItem
        e.DrawBackground()
        Dim txt As String = ""
        If e.Index >= 0 Then txt = Me.Items(e.Index).ToString
        TextRenderer.DrawText(e.Graphics, txt, Me.MyFont, e.Bounds, e.ForeColor, TextFormatFlags.HorizontalCenter)
        e.DrawFocusRectangle()
    End Sub

    Private Sub CenteredComboBox_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Me.SelectedIndexChanged
        Me.ClickedIndex = Me.SelectedIndex
    End Sub

End Class
