﻿Option Strict On
Option Explicit On

Imports System.Collections
Imports System.Collections.Generic
Imports System.Drawing
Imports System.Drawing.Text
Imports System.Runtime.InteropServices


Module CustomFont
    'PRIVATE FONT COLLECTION TO HOLD THE DYNAMIC FONT
    Private _pfc As PrivateFontCollection = Nothing

    Public Function GetFonts() As List(Of String)
        'IF THIS IS THE FIRST TIME GETTING AN INSTANCE
        'LOAD THE FONT FROM RESOURCES
        If _pfc Is Nothing Then
            LoadFonts()
        End If

        Dim FontList As New List(Of String)

        For Each item As FontFamily In _pfc.Families
            FontList.Add(item.Name)
        Next item

        Return FontList
    End Function

    Public Function GetFonts(ByVal filepath As String) As List(Of String)
        'IF THIS IS THE FIRST TIME GETTING AN INSTANCE
        'LOAD THE FONT FROM RESOURCES
        If _pfc Is Nothing Then
            LoadFonts(filepath)
        End If

        Dim FontList As New List(Of String)

        For Each item As FontFamily In _pfc.Families
            FontList.Add(item.Name)
        Next item

        Return FontList
    End Function

    Public ReadOnly Property GetFont(ByVal Name As String,
                                     ByVal Size As Single, _
                                     Optional ByVal style As FontStyle = Nothing) As Font
        Get

            'IF THIS IS THE FIRST TIME GETTING AN INSTANCE
            'LOAD THE FONT FROM RESOURCES
            If _pfc Is Nothing Then
                LoadFonts()
            End If

            Dim Index As Integer = 0

            For test_idx As Integer = 0 To _pfc.Families.Length - 1
                If _pfc.Families(test_idx).Name = Name Then
                    Index = test_idx
                    Exit For
                End If
            Next test_idx

            Return GetFont(Index, Size, style)
        End Get

    End Property


    Public ReadOnly Property GetFont(ByVal Index As Integer,
                                         ByVal Size As Single, _
                                         Optional ByVal style As FontStyle = Nothing) As Font
        Get
            'IF THIS IS THE FIRST TIME GETTING AN INSTANCE
            'LOAD THE FONT FROM RESOURCES
            If _pfc Is Nothing Then
                LoadFonts()
            End If

            'RETURN A NEW FONT OBJECT BASED ON THE SIZE AND STYLE PASSED IN
            Return New Font(_pfc.Families(Index), Size, style)
        End Get
    End Property

    Private Sub LoadFonts(ByVal filepath As String)

        'INIT THE FONT COLLECTION
        If _pfc Is Nothing Then
            _pfc = New PrivateFontCollection
        End If

        For Each fontfile As String In System.IO.Directory.GetFiles(filepath & "\Fonts", "*.ttf")
            _pfc.AddFontFile(fontfile)
        Next fontfile
    End Sub


    Private Sub LoadFonts()
        Dim runTimeResourceSet As System.Resources.ResourceSet
        Dim dictEntry As DictionaryEntry

        'INIT THE FONT COLLECTION
        If _pfc Is Nothing Then
            _pfc = New PrivateFontCollection
        End If

        runTimeResourceSet = My.Resources.ResourceManager.GetResourceSet(System.Globalization.CultureInfo.CurrentCulture, True, True)
        For Each dictEntry In runTimeResourceSet
            If (dictEntry.Value.GetType() Is GetType(Byte())) Then
                If LoadFont(CType(dictEntry.Value, Byte())) = True Then
                End If
            End If
        Next dictEntry

    End Sub

    Private Function LoadFont(ByVal Font As Byte()) As Boolean
        Try
            'LOAD MEMORY POINTER FOR FONT RESOURCE
            Dim fontMemPointer As IntPtr = Marshal.AllocCoTaskMem(Font.Length)

            'COPY THE DATA TO THE MEMORY LOCATION
            Marshal.Copy(Font, 0, fontMemPointer, Font.Length)

            'LOAD THE MEMORY FONT INTO THE PRIVATE FONT COLLECTION
            _pfc.AddMemoryFont(fontMemPointer, Font.Length)

            'FREE UNSAFE MEMORY
            Marshal.FreeCoTaskMem(fontMemPointer)
        Catch ex As Exception
            'ERROR LOADING FONT. HANDLE EXCEPTION HERE
            Return False
        End Try

        Return True
    End Function
End Module
