﻿Option Strict On
Option Explicit On

Imports System.Windows.Forms

Public Class Data_Log
    Private Const TIMESTAMP_FORMAT As String = "0.00"

    Private WithEvents CheckBox As CheckBox
    Private WithEvents event_log As Event_Log
    Private logging As Boolean
    Private log_now_flag As Boolean

    Private filename As String

    Private datapoint_count As Integer
    Private timestamp_start As Single
    Private timestamp_next As Single

    Public Sub New(ByVal CheckBox As CheckBox, ByVal event_log As Event_Log)
        Me.CheckBox = CheckBox
        Me.event_log = event_log
        Me.filename = ""
        Me.logging = False
        Me.log_now_flag = False
    End Sub

    Public Sub Close()
        If (Me.logging = False) Then
            Return
        End If

        Me.Stop_Log()
    End Sub

    Private Sub Data_Log_CheckBox_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBox.CheckedChanged
        If CheckBox.Checked = True Then
            Dim save_file_dialog As New SaveFileDialog()
            Dim save_file_name As String

            save_file_dialog.Filter = "CSV Files (*.csv)|*.csv|All Files (*.*)|*.*"
            save_file_dialog.Title = "Choose Data Log File Location"
            save_file_dialog.FileName = "DC2343A_Data_Log" + DateTime.Now.ToString("_yymmdd_HHmm") + ".csv"
            If save_file_dialog.ShowDialog() = DialogResult.OK Then
                If save_file_dialog.FileName.Substring(save_file_dialog.FileName.Length - ".csv".Length, ".csv".Length) = ".csv" Then
                    save_file_name = save_file_dialog.FileName
                Else
                    save_file_name = save_file_dialog.FileName + ".csv"
                End If
            Else
                CheckBox.Checked = False
                Exit Sub
            End If

            Me.Start(save_file_name)
        Else
            Me.Stop_Log()
        End If

    End Sub

    Private Sub Start(ByVal filename As String)
        Me.filename = filename
        Try
            Me.logging = True

            ' Set up timestamp to start when first data point is taken.
            Me.timestamp_start = -1

            ' Clear the datapoint counter
            datapoint_count = 0

            ' Create the header
            Me.Create_Header()

        Catch ex As Exception
            D3335.Handle_Exception(ex)
        End Try
        event_log.Add(event_log.Event_Code.Message, "Data Log started at: " + Me.filename)
    End Sub

    Private Sub Stop_Log()
        Me.logging = False
        event_log.Add(event_log.Event_Code.Message, "Data Log stopped at: " + Me.filename)
    End Sub

    Private Sub Create_Header()
        Dim writer As System.IO.StreamWriter = Nothing

        Try
            Dim textline As String
            ' Open the writer.  Overwrite old file.
            writer = New System.IO.StreamWriter(Me.filename, False)

            ' Save the filename, date, time, interval, and length
            Dim temp_datetime As DateTime = DateTime.Now
            writer.WriteLine(Me.filename)
            writer.WriteLine("Date Started:," + String.Format("{0:MM/dd/yyyy}", temp_datetime))
            writer.WriteLine("Time Started:," + String.Format("{0:HH:mm}", temp_datetime))
            writer.WriteLine("Serial Number:," + D3335.SN_TextBox.Text)
            writer.WriteLine("FW Revision:," + D3335.FW_Rev_TextBox.Text)
            writer.WriteLine("Voltage Setting:," + D3335.Voltage_Selection_CenteredComboBox.SelectedItem.ToString)
            writer.WriteLine("IPEAK Setting:," + D3335.Ipeak_Selection.Ipeak_Selection_ComboBox.SelectedItem.ToString)
            writer.WriteLine("Prescaler Setting:," + D3335.Prescaler_Selection_CenteredComboBox.SelectedItem.ToString)
            writer.WriteLine("Battery Capacity:," + D3335.battery_cc_graphic.battery_capacity_ah.ToString)

            writer.WriteLine("")

            ' Save the datapoint number and PIC timestamp
            writer.Write("Datapoint,")
            writer.Write("Time (s),")

            ' Save the accumulator data 
            textline = "Accumulator Value (counts),"
            writer.Write(textline)

            ' Save the accumulator data 
            textline = "Alarm Value (counts),"
            writer.Write(textline)

            ' Save the alarms data 
            textline = "Alarm: ac_on_time_overflow,"
            textline += "Alarm: Coulomb_Counter_Overflow,"
            textline += "Alarm: Alarm_Threshold,"
            writer.Write(textline)

            ' Save the GPIO data 
            textline = "/IRQ Value,"
            textline += "PGOOD Value,"
            writer.Write(textline)

            ' Save the ADC data 
            textline = "VBAT Value,"
            textline += "VOUT Value,"
            writer.Write(textline)

            ' Save the Software Correction and Coulomb Count Data 
            textline = "Software Correction,"
            textline += "Typ Error (%),"
            textline += "Resolution (coulombs/count),"
            textline += "Coulomb Count (Ah),"
            textline += "Coulomb Count (%),"
            writer.Write(textline)

            ' Start Next Line
            writer.WriteLine("")

        Catch ex As Exception
            D3335.Handle_Exception(ex)
        Finally
        End Try

        If writer IsNot Nothing Then
            writer.Flush()
            writer.Close()
        End If

    End Sub

    Public Sub Arm_Data(ByVal timestamp As Single)

        If (Me.logging = False) Then
            Return
        End If

        Me.log_now_flag = True
        Me.timestamp_next = timestamp

        ' If this is the first data taken, set the 0-time point for the timestamp.
        If Me.timestamp_start < 0 Then
            Me.timestamp_start = Me.timestamp_next
        End If
    End Sub

    Public Sub Add_Data()

        Dim writer As System.IO.StreamWriter = Nothing

        If (Me.logging = False) Then
            Return
        End If

        If (Me.log_now_flag = False) Then
            Return
        End If
        Me.log_now_flag = False

        Try
            Dim textline As String = ""

            ' Open the writer.  Append to file.
            writer = System.IO.File.AppendText(Me.filename)

            ' Save the datapoint number and PIC timestamp
            Me.datapoint_count += 1
            writer.Write(Me.datapoint_count.ToString() + ",")
            writer.Write((Me.timestamp_next - Me.timestamp_start).ToString(TIMESTAMP_FORMAT) + ",")

            ' Save the accumulator data 
            textline = D3335.Accumulator_TextBox.Text + ","
            writer.Write(textline)

            ' Save the alarm threshold data 
            textline = D3335.Alarm_Threshold_TextBox.Text + ","
            writer.Write(textline)

            ' Save the alarms data 
            textline = (-CInt(D3335.Alarm_AC_ON_Time_CheckBox.Checked)).ToString() + ","
            textline += (-CInt(D3335.Alarm_Coulomb_Counter_Overflow_CheckBox.Checked)).ToString() + ","
            textline += (-CInt(D3335.Alarm_Alarm_Trip_CheckBox.Checked)).ToString() + ","
            writer.Write(textline)

            ' Save the GPIO data 
            textline = D3335.LTC3335_nIRQ_Textbox.Text + ","
            textline += D3335.LTC3335_PGOOD_TextBox.Text + ","
            writer.Write(textline)

            ' Save the ADC data 
            textline = D3335.Battery_Voltage_TextBox.Text.Replace("V", "") + ","
            textline += D3335.Output_Voltage_TextBox.Text.Replace("V", "") + ","
            writer.Write(textline)

            ' Save the Software Correction and Coulomb Count Data 
            textline = (-CInt(D3335.battery_cc_graphic.use_software_correction)).ToString + ","
            textline += D3335.battery_cc_graphic.software_correction_factor.ToString + ","
            textline += D3335.battery_cc_graphic.resolution.ToString + ","
            textline += D3335.battery_cc_graphic.battery_coulombs_discharged_ah.ToString + ","
            textline += D3335.battery_cc_graphic.Coulombs_Discharged_Ah_To_Percent_Discharged(D3335.battery_cc_graphic.battery_coulombs_discharged_ah).ToString + ","
            writer.Write(textline)

            ' Start Next Line
            writer.WriteLine("")

        Catch ex As Exception
            D3335.Handle_Exception(ex)
        End Try

        If writer IsNot Nothing Then
            writer.Flush()
            writer.Close()
        End If

    End Sub

End Class
