﻿Option Strict On
Option Explicit On

Imports Microsoft.VisualBasic
Imports System.Windows.Forms
Imports System.Drawing
Imports Microsoft.VisualBasic.PowerPacks

Public Class En_Selection

    Private usb_comm As D3335_USB
    Friend selection As LTC3335.EN_CONFIGURATION_OPTIONS

    Public Class en_option
        Private Const SELECTION_WIDTH As Integer = 1

        Private en_label As Label
        Private description_label As Label
        Private _selected As Boolean = False
        Private _originally_selected As Boolean
        Private hardware_warning_label As Label
        Private option_list() As en_option

        Public Sub New(ByVal en_label As Label, ByVal description_label As Label, ByRef option_list() As en_option, ByVal hardware_warning_label As Label)
            Me.en_label = en_label
            Me.description_label = description_label
            Me.option_list = option_list
            Me.hardware_warning_label = hardware_warning_label
            Me.selected = False
            Me.originally_selected = False
        End Sub

        Public Property selected() As Boolean
            Get
                Return _selected
            End Get

            Set(ByVal value As Boolean)
                _selected = value
                If _selected = True Then
                    Me.en_label.Font = New Font("Linear Helv Cond", 12.0!, FontStyle.Bold)
                    D3335.Set_Font(DirectCast(Me.en_label, Control))

                    Me.description_label.Font = New Font("Linear Helv Cond", 12.0!, FontStyle.Bold)
                    D3335.Set_Font(DirectCast(Me.description_label, Control))

                    ' Add handlers for painting the box around the selection.
                    AddHandler Me.en_label.Paint, AddressOf EN_Label_Paint
                    AddHandler Me.description_label.Paint, AddressOf Description_Label_Paint
                    AddHandler Me.description_label.Parent.Paint, AddressOf EN_TableLayoutPanel_Paint

                    ' Remove handlers for clicking now that this is already selected.
                    RemoveHandler Me.en_label.Click, AddressOf Label_Click
                    RemoveHandler Me.description_label.Click, AddressOf Label_Click

                Else
                    Me.en_label.Font = New Font("Linear Helv Cond", 12.0!, FontStyle.Regular)
                    D3335.Set_Font(DirectCast(Me.en_label, Control))

                    Me.description_label.Font = New Font("Linear Helv Cond", 12.0!, FontStyle.Regular)
                    D3335.Set_Font(DirectCast(Me.description_label, Control))

                    ' Remove handlers for painting the box around the selection.
                    RemoveHandler Me.en_label.Paint, AddressOf EN_Label_Paint
                    RemoveHandler Me.description_label.Paint, AddressOf Description_Label_Paint
                    RemoveHandler Me.description_label.Parent.Paint, AddressOf EN_TableLayoutPanel_Paint

                    ' Add handlers for clicking now so that this option can be selected.
                    AddHandler Me.en_label.Click, AddressOf Label_Click
                    AddHandler Me.description_label.Click, AddressOf Label_Click

                End If
            End Set

        End Property

        Public Property originally_selected() As Boolean
            Get
                Return _originally_selected
            End Get

            Set(ByVal value As Boolean)
                _originally_selected = value
            End Set

        End Property

        Private Sub EN_Label_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs)

            If Me.selected = True Then
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(SELECTION_WIDTH - 1, SELECTION_WIDTH - 1), _
                                    New Point(SELECTION_WIDTH - 1, en_label.Height - SELECTION_WIDTH))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(SELECTION_WIDTH - 1, SELECTION_WIDTH - 1), _
                                    New Point(en_label.Width, SELECTION_WIDTH - 1))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(SELECTION_WIDTH - 1, en_label.Height - SELECTION_WIDTH), _
                                    New Point(en_label.Width, en_label.Height - SELECTION_WIDTH))
            End If

        End Sub

        Private Sub Description_Label_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs)

            If Me.selected = True Then
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(0, SELECTION_WIDTH - 1), _
                                    New Point(description_label.Width - SELECTION_WIDTH, SELECTION_WIDTH - 1))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(0, description_label.Height - SELECTION_WIDTH), _
                                    New Point(description_label.Width - SELECTION_WIDTH, description_label.Height - SELECTION_WIDTH))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(description_label.Width - SELECTION_WIDTH, SELECTION_WIDTH - 1), _
                                    New Point(description_label.Width - SELECTION_WIDTH, description_label.Height - SELECTION_WIDTH))
            End If
        End Sub

        Private Sub EN_TableLayoutPanel_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs)

            If Me.selected = True Then
                ' Draw line to screen.
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(en_label.Location.X, en_label.Location.Y + SELECTION_WIDTH - 1),
                                    New Point(description_label.Location.X + description_label.Width - SELECTION_WIDTH, description_label.Location.Y + SELECTION_WIDTH - 1))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(en_label.Location.X, en_label.Location.Y + en_label.Height - SELECTION_WIDTH), _
                                    New Point(description_label.Location.X + description_label.Width - SELECTION_WIDTH, description_label.Location.Y + description_label.Height - SELECTION_WIDTH - 1))
            End If
        End Sub

        Private Sub Label_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)

            ' First unselect the option that's already selected
            For Each other_option As en_option In option_list
                If other_option.selected = True Then
                    other_option.selected = False
                End If
            Next other_option

            ' Now selected this option
            Me.selected = True

            ' Show warning that jumpers must be set.
            Me.hardware_warning_label.Visible = Not Me.originally_selected

        End Sub

    End Class

    Public en_options(LTC3335.EN_CONFIGURATION_OPTIONS.NUM_EN_CONFIGURATIONS - 1) As en_option

    ' Popup window should always be on top of GUI Control Panel, but not always on Top of all Windows programs.
    Private Declare Function BringWindowToTop Lib "user32" Alias "BringWindowToTop" (ByVal hwnd As Long) As Long
    Private Sub En_Selection_Activated(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        BringWindowToTop(CLng(Me.Handle))
    End Sub

    Friend Overloads Sub Show(ByVal selection As LTC3335.EN_CONFIGURATION_OPTIONS)
        For en_option As LTC3335.EN_CONFIGURATION_OPTIONS = LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_ON To LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_OFF
            en_options(en_option).originally_selected = If(selection = en_option, True, False)
        Next en_option

        ' Start with warning that jumpers must be set hidden.
        Me.Hardware_Warning_Label.Visible = False

        Me.Show()
    End Sub

    Friend Sub New(ByVal usb_comm As D3335_USB)

        InitializeComponent()

        Me.usb_comm = usb_comm

        ' Set the Use Compatible Text Rendering property.
        For Each control As Control In MyBase.Controls
            D3335.Set_UseCompatibleTextRendering(control)
        Next control

        ' Next set all of the fonts to Linear Helv Cond.
        For Each control As Control In MyBase.Controls
            D3335.Set_Font(control)
        Next control

        ' Turn off all of the tabstops on this page.
        For Each control As Control In MyBase.Controls
            D3335.Set_TabStops(control, False)
        Next control

        ' Set up all of the en option objects.
        en_options(LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_ON) = New en_option(EN_ON_Label, Description_ON_Label, en_options, Hardware_Warning_Label)
        en_options(LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_EXT) = New en_option(EN_EXT_Label, Description_EXT_Label, en_options, Hardware_Warning_Label)
        en_options(LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_OFF) = New en_option(EN_OFF_Label, Description_OFF_Label, en_options, Hardware_Warning_Label)

        ' The 3.3V setting with EN on is the default from the factory.
        Set_Selection(LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_ON)

    End Sub

    Friend Sub Set_Selection(ByVal selection As LTC3335.EN_CONFIGURATION_OPTIONS)
        Me.selection = selection
        Set_Selection_GUI(selection)
    End Sub

    Friend Sub Set_Selection_GUI(ByVal selection As LTC3335.EN_CONFIGURATION_OPTIONS)
        For en_option As LTC3335.EN_CONFIGURATION_OPTIONS = LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_ON To LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_OFF
            en_options(en_option).selected = If(selection = en_option, True, False)
        Next en_option
    End Sub

    Friend Sub OK_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OK_Button.Click

        ' Set EN selection to that specified by this popup.
        Me.selection = LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_ON
        For Each this_option As en_option In en_options
            If this_option.selected = True Then
                Exit For
            Else
                Me.selection = CType(Me.selection + 1, LTC3335.EN_CONFIGURATION_OPTIONS)
            End If
        Next this_option

        ' Send command to FW to set the EN selection.
        Dim data As New D3335_USB.EN_CONFIGURATION_COMMAND_WRITE
        data.en_configuration = Me.selection
        Me.usb_comm.SendCommand(D3335_USB.USB_DATA_ID.EN_CONFIGURATION, data)

        ' Close the popup
        Me.Hide()
    End Sub

    Friend Sub Cancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cancel_Button.Click
        ' Revert this popup to show the EN selection before the user started to change controls.
        Me.Set_Selection_GUI(Me.selection)

        ' Close the popup
        Me.Hide()
    End Sub

    Protected Overrides Sub OnFormClosing(ByVal e As System.Windows.Forms.FormClosingEventArgs)
        Cancel_Button_Click(Cancel_Button, New EventArgs)
        e.Cancel = True
        MyBase.OnFormClosing(e)
    End Sub

End Class
