﻿Option Strict On
Option Explicit On

Imports System.Drawing
Imports Microsoft.VisualBasic
Imports Microsoft.VisualBasic.PowerPacks

Public Class Estimated_Current_Graphic

    Private Const NUM_LINE_SEGMENTS As Integer = 6
    Private Const LINE_SPACING As Integer = 2
    Private Const MAX_SCROLL_SPEED As Integer = 100          ' in Hz

    Private panel As Panel
    Private label As Label
    Private picturebox As PictureBox

    Private shapecontainer As ShapeContainer
    Private lineshapes(NUM_LINE_SEGMENTS) As LineShape  ' Note that 1 extra line is made, since one of the segments is going to be split between the front and the back.

    Private WithEvents timer As Timer
    Private line_height As Integer
    Private line_length As Integer

    Private textbox As TextBox
    Private textbox_string As String
    Private textbox_toggle As Boolean

    Public Sub New(ByVal panel As Panel, Optional ByVal textbox As TextBox = Nothing)

        ' Search  panel for a Label and a PictureBox to use for the Current Control
        Me.panel = panel
        Me.textbox = textbox
        For Each control As Control In Me.panel.Controls
            If (control.GetType() Is GetType(Label)) Then
                Me.label = CType(control, Windows.Forms.Label)
            End If
            If (control.GetType() Is GetType(PictureBox)) Then
                Me.picturebox = CType(control, Windows.Forms.PictureBox)
            End If
        Next control

        ' Draw lines to picturebox to animate as current flow
        Me.shapecontainer = New ShapeContainer()
        Me.shapecontainer.Parent = Me.panel
        Me.line_height = CInt(Me.picturebox.Location.Y + Me.picturebox.Height / 2)
        Me.line_length = CInt((Me.picturebox.Location.X - NUM_LINE_SEGMENTS * LINE_SPACING) / NUM_LINE_SEGMENTS)
        Dim line_position As Integer = 0

        For line_index As Integer = 0 To NUM_LINE_SEGMENTS
            Me.lineshapes(line_index) = New LineShape
            With Me.lineshapes(line_index)
                .Parent = Me.shapecontainer
                .BorderColor = SystemColors.ControlText
                .BorderWidth = 4
                .Name = "LineShape" + line_index.ToString
                .Enabled = False
                .X1 = line_position
                .X2 = Me.lineshapes(line_index).X1 + If(line_index <> NUM_LINE_SEGMENTS, Me.line_length, 0)
                .Y1 = Me.line_height
                .Y2 = Me.line_height
                .SelectionColor = Color.Transparent
            End With
            line_position += (Me.line_length + LINE_SPACING)
        Next line_index

        ' Start with label clear
        Me.label.Text = ""

        ' Create timer for the animation
        Me.timer = New Timer()
        Me.timer.Stop()
        Me.textbox_toggle = False

        ' Bring all of the shapes to the front
        Me.shapecontainer.BringToFront()
        Me.panel.Visible = False

    End Sub

    Public Sub Show(ByVal current_ma As Single, ByVal max_current_ma As Single, ByVal textbox_toggle As Boolean, ByVal show_graphic As Boolean)

        ' If we're toggling the /IRQ textbox, remember the old value so that it can be restored when test is over.
        If (Not Me.textbox Is Nothing) And (textbox_string = "") Then
            Me.textbox_string = Me.textbox.Text
        End If
        Me.textbox_toggle = textbox_toggle

        ' Set timer to toggle at a frequency proportional to the current.
        If (current_ma = 0) Or (max_current_ma = 0) Then
            Me.timer.Stop()
        Else
            Dim frequency As Single = MAX_SCROLL_SPEED * current_ma / max_current_ma
            Me.timer.Interval = Math.Max(CInt(1000 / frequency), 1)
            Me.timer.Start()
        End If

        ' Display the current.
        Me.label.Text = get_current_string(current_ma)
        Me.panel.Visible = show_graphic
    End Sub

    Public Sub Hide()
        ' Restore the /IRQ textbox and make the control invisible.
        If Not Me.textbox Is Nothing Then
            Me.textbox.Text = textbox_string
            textbox_string = ""
        End If
        Me.timer.Stop()
        Me.panel.Visible = False
    End Sub

    Private Function get_current_string(ByVal current_ma As Single) As String

        Dim current_ma_format As String
        Dim current_ma_units As String

        If current_ma < 1 Then
            current_ma *= 1000
            current_ma_units = "µA"
        Else
            current_ma_units = "mA"
        End If

        If current_ma > 100 Then
            current_ma_format = "000"
        ElseIf current_ma > 10 Then
            current_ma_format = "00.0"
        ElseIf current_ma > 1 Then
            current_ma_format = "0.00"
        Else
            current_ma_format = "0.000"
        End If

        Return current_ma.ToString(current_ma_format) + current_ma_units

    End Function

    Private Sub timer_tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer.Tick

        ' The first segment is going to be a fraction of a line.
        If Me.lineshapes(0).X2 = Me.line_length Then
            Me.lineshapes(0).X2 = 0
        Else
            Me.lineshapes(0).X2 += 1
        End If

        ' The middle segments are all full lines.
        Dim line_position As Integer = Me.lineshapes(0).X2 + LINE_SPACING
        For line_index As Integer = 1 To NUM_LINE_SEGMENTS - 1
            Me.lineshapes(line_index).X1 = line_position
            Me.lineshapes(line_index).X2 = Me.lineshapes(line_index).X1 + Me.line_length
            line_position += (Me.line_length + LINE_SPACING)
        Next line_index

        ' The last segment is going to combine with the first segment to total a full line.
        Me.lineshapes(NUM_LINE_SEGMENTS).X1 = line_position
        Me.lineshapes(NUM_LINE_SEGMENTS).X2 = Me.lineshapes(NUM_LINE_SEGMENTS).X1 + (Me.line_length - Me.lineshapes(0).X2)

        ' If an /IRQ textbox was passed, toggle it at the same rate that is used to animate the lines.
        If (Not Me.textbox Is Nothing) And (Me.textbox_toggle = True) Then
            If Me.textbox.Text = "0" Then
                Me.textbox.Text = "1"
            Else
                Me.textbox.Text = "0"
            End If
        End If

    End Sub
End Class
