﻿Option Strict On
Option Explicit On

Imports Microsoft.VisualBasic
Imports System.Windows.Forms

Public Class Ipeak_Selection

    Private usb_comm As D3335_USB
    Private selection As LTC3335.IPEAK_SELECTION_OPTIONS
    Private nonstandard_inductor As Boolean
    Private nonstandard_inductor_text As String
    Private tooltip As ToolTip

    ' Popup window should always be on top of GUI Control Panel, but not always on Top of all Windows programs.
    Private Declare Function BringWindowToTop Lib "user32" Alias "BringWindowToTop" (ByVal hwnd As Long) As Long
    Private Sub Ipeak_Selection_Activated(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        BringWindowToTop(CLng(Me.Handle))
    End Sub

    Friend Sub New(ByVal usb_comm As D3335_USB)

        InitializeComponent()

        Me.usb_comm = usb_comm

        ' Populate the combo boxes if they haven't been already
        If Me.Ipeak_Selection_ComboBox.Items.Count < LTC3335.IPEAK_SELECTION_OPTIONS.NUM_IPEAK_SELECTION_OPTIONS Then
            For selection = 0 To CType(LTC3335.IPEAK_SELECTION_OPTIONS.NUM_IPEAK_SELECTION_OPTIONS - 1, LTC3335.IPEAK_SELECTION_OPTIONS)
                Me.Ipeak_Selection_ComboBox.Items.Add(CType(selection, LTC3335.IPEAK_SELECTION_OPTIONS).ToString())
                Me.Inductor_ComboBox.Items.Add(Get_Inductor_String(CType(selection, LTC3335.IPEAK_SELECTION_OPTIONS)))
            Next selection
        End If

        nonstandard_inductor_text = Inductor_TextBox.Text
        nonstandard_inductor = NonStandard_Inductor_CheckBox.Checked

        ' The 100mA setting is the default from the factory.
        Set_Selection(LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_100MA)

        tooltip = New ToolTip

        Dim tooltip_msg As String = "Select Value of IPeak"
        tooltip.SetToolTip(Me.Ipeak_Selection_ComboBox, tooltip_msg)

        tooltip_msg = "Double Click Populated Resistors for R1-R6"
        For Each control As Control In Me.IPK_GroupBox.Controls
            If (control.GetType() Is GetType(PictureBox)) Then
                tooltip.SetToolTip(control, tooltip_msg)
            End If
        Next control

        tooltip_msg = "Select Value of Inductor"
        tooltip.SetToolTip(Me.Inductor_TextBox, tooltip_msg)
        tooltip.SetToolTip(Me.Inductor_ComboBox, tooltip_msg)

    End Sub

    Friend Function Get_Selection() As LTC3335.IPEAK_SELECTION_OPTIONS
        Return Me.selection
    End Function

    Friend Function Get_Inductor_String() As String
        Return Me.Get_Inductor_String(Me.selection)
    End Function

    Friend Sub Set_Inductor_String(ByVal inductor_string As String)

        nonstandard_inductor_text = inductor_string
        Inductor_TextBox.Text = nonstandard_inductor_text

        Dim blank_inductor_response As New D3335_USB.NONSTANDARD_INDUCTOR_RESPONSE
        NonStandard_Inductor_CheckBox.Checked = (nonstandard_inductor_text <> "") 'blank_inductor_response.nonstandard_inductor)

    End Sub

    Private Function get_inductor_string(ByVal selection As LTC3335.IPEAK_SELECTION_OPTIONS) As String
        If NonStandard_Inductor_CheckBox.Checked = False Then
            Dim inductor_value As Integer
            Select Case selection
                Case LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_5MA
                    inductor_value = 2200
                Case LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_10MA
                    inductor_value = 1000
                Case LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_15MA
                    inductor_value = 680
                Case LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_25MA
                    inductor_value = 470
                Case LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_50MA
                    inductor_value = 220
                Case LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_100MA
                    inductor_value = 100
                Case LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_150MA
                    inductor_value = 68
                Case LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_250MA
                    inductor_value = 47
                Case Else
                    inductor_value = 47
            End Select

            Return inductor_value.ToString + " µH"
        Else
            Try
                Dim test As Single = CSng(Inductor_TextBox.Text)
                Return Inductor_TextBox.Text.TrimEnd + " µH"
            Catch ex As Exception
                Return Inductor_TextBox.Text
            End Try
        End If

    End Function

    Friend Sub Set_Selection(ByVal selection As LTC3335.IPEAK_SELECTION_OPTIONS)
        Me.selection = selection
        Set_Selection_GUI(selection)
    End Sub

    Friend Sub Set_Selection_GUI(ByVal selection As LTC3335.IPEAK_SELECTION_OPTIONS)
        Me.Ipeak_Selection_ComboBox.SelectedItem = selection.ToString()
        Me.Inductor_ComboBox.SelectedItem = Me.Get_Inductor_String(selection)
        Me.LTC3335_IPK2_TextBox.Text = ((CInt(selection) And &H4) >> 2).ToString
        Me.LTC3335_IPK1_TextBox.Text = ((CInt(selection) And &H2) >> 1).ToString
        Me.LTC3335_IPK0_TextBox.Text = ((CInt(selection) And &H1) >> 0).ToString
        If Me.LTC3335_IPK2_TextBox.Text = "0" Then
            R1_PictureBox.Image = Nothing
            R2_PictureBox.Image = My.Resources.Resources.LTC3335_resistor
        Else
            R1_PictureBox.Image = My.Resources.Resources.LTC3335_resistor
            R2_PictureBox.Image = Nothing
        End If
        If Me.LTC3335_IPK1_TextBox.Text = "0" Then
            R3_PictureBox.Image = Nothing
            R4_PictureBox.Image = My.Resources.Resources.LTC3335_resistor
        Else
            R3_PictureBox.Image = My.Resources.Resources.LTC3335_resistor
            R4_PictureBox.Image = Nothing
        End If
        If Me.LTC3335_IPK0_TextBox.Text = "0" Then
            R5_PictureBox.Image = Nothing
            R6_PictureBox.Image = My.Resources.Resources.LTC3335_resistor
        Else
            R5_PictureBox.Image = My.Resources.Resources.LTC3335_resistor
            R6_PictureBox.Image = Nothing
        End If

        Hardware_Warning_Label.Visible = Me.selection <> selection

        Me.AcceptButton = OK_Button
    End Sub

    Private Sub Ipeak_Selection_ComboBox_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ipeak_Selection_ComboBox.SelectedIndexChanged
        If Ipeak_Selection_ComboBox.SelectedIndex <> -1 Then
            Dim selection As LTC3335.IPEAK_SELECTION_OPTIONS
            selection = DirectCast([Enum].Parse(GetType(LTC3335.IPEAK_SELECTION_OPTIONS), Ipeak_Selection_ComboBox.SelectedItem.ToString), LTC3335.IPEAK_SELECTION_OPTIONS)
            Me.Set_Selection_GUI(selection)
        Else
            Me.Set_Selection_GUI(Me.selection)
        End If
    End Sub

    Private Sub LTC3335_IPK2_TextBox_Enter(ByVal sender As System.Object, ByVal e As KeyPressEventArgs) Handles LTC3335_IPK2_TextBox.KeyPress, _
        LTC3335_IPK1_TextBox.KeyPress, LTC3335_IPK0_TextBox.KeyPress
        If e.KeyChar = Convert.ToChar(Keys.Enter) Then
            LTC3335_IPK_TextBox_LostFocus(sender, e)
        End If
    End Sub

    Private Sub LTC3335_IPK_TextBox_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LTC3335_IPK2_TextBox.LostFocus, _
        LTC3335_IPK1_TextBox.LostFocus, LTC3335_IPK0_TextBox.LostFocus

        If ((LTC3335_IPK2_TextBox.Text = "0") Or (LTC3335_IPK2_TextBox.Text = "1")) And _
            ((LTC3335_IPK1_TextBox.Text = "0") Or (LTC3335_IPK1_TextBox.Text = "1")) And _
            ((LTC3335_IPK0_TextBox.Text = "0") Or (LTC3335_IPK0_TextBox.Text = "1")) Then
            Dim temp_int As Integer = (CInt(Me.LTC3335_IPK0_TextBox.Text) << 0) + (CInt(Me.LTC3335_IPK1_TextBox.Text) << 1) + (CInt(Me.LTC3335_IPK2_TextBox.Text) << 2)
            Dim selection As LTC3335.IPEAK_SELECTION_OPTIONS = CType(temp_int, LTC3335.IPEAK_SELECTION_OPTIONS)
            Me.Set_Selection_GUI(selection)
        Else
            Me.Set_Selection_GUI(Me.selection)
        End If
        Me.AcceptButton = OK_Button
    End Sub

    Private Sub LTC3335_IPK_TextBox_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LTC3335_IPK2_TextBox.TextChanged, _
        LTC3335_IPK1_TextBox.TextChanged, LTC3335_IPK0_TextBox.TextChanged, Inductor_TextBox.TextChanged
        Me.AcceptButton = Nothing
    End Sub

    Private Sub Inductor_TextBox_TextBox_Enter(ByVal sender As System.Object, ByVal e As KeyPressEventArgs) Handles Inductor_TextBox.KeyPress
        If e.KeyChar = Convert.ToChar(Keys.Enter) Then
            Inductor_TextBox_LostFocus(sender, e)
        End If
    End Sub

    Private Sub Inductor_TextBox_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Inductor_TextBox.LostFocus
        Me.AcceptButton = OK_Button
    End Sub

    Private Sub Inductor_ComboBox_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Inductor_ComboBox.SelectedIndexChanged
        If Inductor_ComboBox.SelectedIndex <> -1 Then
            Dim selection As LTC3335.IPEAK_SELECTION_OPTIONS = CType(Inductor_ComboBox.SelectedIndex, LTC3335.IPEAK_SELECTION_OPTIONS)
            Me.Set_Selection_GUI(selection)
        Else
            Me.Set_Selection_GUI(Me.selection)
        End If
    End Sub

    Friend Sub OK_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OK_Button.Click

        ' Set IPEAK selection to that specified by this popup.
        Me.selection = DirectCast([Enum].Parse(GetType(LTC3335.IPEAK_SELECTION_OPTIONS), Ipeak_Selection_ComboBox.SelectedItem.ToString), LTC3335.IPEAK_SELECTION_OPTIONS)
        Me.nonstandard_inductor_text = If(Inductor_TextBox.Text <> "", Inductor_TextBox.Text, " ")
        Me.nonstandard_inductor = NonStandard_Inductor_CheckBox.Checked

        ' Send command to FW to set the IPEAK selection.
        Dim data As New D3335_USB.Ipeak_Selection_COMMAND_WRITE()
        data.Ipeak_Selection = Me.selection
        Me.usb_comm.SendCommand(D3335_USB.USB_DATA_ID.IPEAK_CONFIGURATION, data)

        ' Send command to FW to set the NONSTANDARD_INDUCTOR selection.
        Dim data2 As New D3335_USB.NONSTANDARD_INDUCTOR_COMMAND_WRITE()
        data2.nonstandard_inductor = If(Me.nonstandard_inductor, Me.nonstandard_inductor_text, "")
        Me.usb_comm.SendCommand(D3335_USB.USB_DATA_ID.NONSTANDARD_INDUCTOR, data2)

        ' Close the popup
        Me.Hide()
    End Sub

    Friend Sub Cancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cancel_Button.Click
        ' Revert this popup to show the IPEAK selection before the user started to change controls.
        Me.Set_Selection_GUI(Me.selection)
        Inductor_TextBox.Text = nonstandard_inductor_text
        NonStandard_Inductor_CheckBox.Checked = nonstandard_inductor

        ' Close the popup
        Me.Hide()
    End Sub

    Protected Overrides Sub OnFormClosing(ByVal e As System.Windows.Forms.FormClosingEventArgs)
        Cancel_Button_Click(Cancel_Button, New EventArgs)
        e.Cancel = True
        MyBase.OnFormClosing(e)
    End Sub

    Private Sub R_PictureBox_DoubleClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles R1_PictureBox.DoubleClick, _
        R2_PictureBox.DoubleClick, R3_PictureBox.DoubleClick, R4_PictureBox.DoubleClick, R5_PictureBox.DoubleClick, R6_PictureBox.DoubleClick

        Dim this_picturebox As PictureBox = DirectCast(sender, PictureBox)

        If this_picturebox.Name.IndexOf("R1") >= 0 Then
            LTC3335_IPK2_TextBox.Text = "1"
        ElseIf this_picturebox.Name.IndexOf("R2") >= 0 Then
            LTC3335_IPK2_TextBox.Text = "0"
        ElseIf this_picturebox.Name.IndexOf("R3") >= 0 Then
            LTC3335_IPK1_TextBox.Text = "1"
        ElseIf this_picturebox.Name.IndexOf("R4") >= 0 Then
            LTC3335_IPK1_TextBox.Text = "0"
        ElseIf this_picturebox.Name.IndexOf("R5") >= 0 Then
            LTC3335_IPK0_TextBox.Text = "1"
        ElseIf this_picturebox.Name.IndexOf("R6") >= 0 Then
            LTC3335_IPK0_TextBox.Text = "0"
        End If
        LTC3335_IPK_TextBox_LostFocus(sender, e)

    End Sub

    Private Sub NonStandard_Inductor_CheckBox_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NonStandard_Inductor_CheckBox.CheckedChanged
        Dim tooltip_msg As String

        Inductor_TextBox.Visible = NonStandard_Inductor_CheckBox.Checked
        Inductor_ComboBox.Visible = Not NonStandard_Inductor_CheckBox.Checked
        Or_2_Label.Visible = Not NonStandard_Inductor_CheckBox.Checked

        If NonStandard_Inductor_CheckBox.Checked = False Then
            Inductor_Label.Text = "Select the value of inductor L1."
            tooltip_msg = "Select Value of Inductor"
        Else
            Inductor_Label.Text = "IPEAK can not be determined from" + ChrW(13) + ChrW(10) + "non-standard inductor size."
            tooltip_msg = "IPEAK can not be determined from non-standard inductor size."
        End If
        tooltip.SetToolTip(Me.Inductor_TextBox, tooltip_msg)
        tooltip.SetToolTip(Me.Inductor_ComboBox, tooltip_msg)
    End Sub

    Private Sub Ipeak_Selection_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' todo - I can't seem to loop through all forms, so copy this code to each form for now.
        ' First set the Use Compatible Text Rendering property.
        For Each control As Control In MyBase.Controls
            D3335.Set_UseCompatibleTextRendering(control)
        Next control

        ' Next set all of the fonts to Linear Helv Cond.
        For Each control As Control In MyBase.Controls
            D3335.Set_Font(control)
        Next control

        ' Turn off all of the tabstops on this page.
        For Each control As Control In MyBase.Controls
            D3335.Set_TabStops(control, False)
        Next control

    End Sub

End Class
