﻿Option Strict On
Option Explicit On

Imports Microsoft.VisualBasic

Public Class Mfg_Test
    Private Enum State
        Idle
        Read_VBAT
        Calibrate_VBAT
        Read_VOUT
        Calibrate_VOUT
        I2C_Comm
        Config_Ipeak_Vout_En
        Config_Capacity
        IRQ
        PGOOD
        Write_SN
    End Enum

    Private Const INTERVAL As Integer = 200 ' in ms

    Private test_state As State
    Private test_state_timer As Integer
    Private test_cal_count As Integer

    Private WithEvents test_timer As Timer
    Private WithEvents test_button As Button
    Private test_texbox As TextBox
    Private test_popup As Popup
    Private mfg_file As Mfg_File
    Private mfg_file_data As Mfg_File.Mfg_File_Data

    Public Sub New(ByVal test_button As Button, ByVal test_texbox As TextBox, ByVal test_popup As Popup, ByVal mfg_file As Mfg_File)
        Me.test_button = test_button
        Me.test_button.Visible = True

        Me.test_texbox = test_texbox
        Me.test_texbox.Visible = True

        Me.test_popup = test_popup

        Me.test_timer = New Timer
        Me.test_timer.Interval = INTERVAL

        Me.mfg_file = mfg_file

        Me.stop_test()
    End Sub

    Public Sub Attached(ByVal attached As Boolean)
        Me.test_button.Enabled = attached
        Me.test_texbox.Enabled = attached
        If attached = False Then
            Me.stop_test()
        End If
    End Sub

    Public Sub start_test()
        test_state = CType(Me.test_state + 1, Mfg_Test.State)
        Me.test_button.Text = "Stop Manufacturing Test"
        test_state_timer = 0
        test_cal_count = 0
        test_timer.Start()
        mfg_file_data = New Mfg_File.Mfg_File_Data(mfg_file, D3335.SN_TextBox.Text)
    End Sub

    Public Sub stop_test(Optional ByVal send_usb As Boolean = False, Optional ByVal print_label As Boolean = False)
        test_state = Mfg_Test.State.Idle
        Me.test_button.Text = "Start Manufacturing Test"
        test_state_timer = 0
        test_cal_count = 0
        test_timer.Stop()

        If send_usb = True Then
            ' When creating the Mfg_Test object, the centered combobox mightnot be created yet.
            D3335.Voltage_Selection_CenteredComboBox.SelectedItem = LTC3335.VOLTAGE_SELECTION_OPTIONS.HARDWARE_CONTROL.ToString()
            D3335.Accumulator_TextBox.Text = (0).ToString()
            D3335.Accumulator_TextBox_LostFocus(D3335.Accumulator_TextBox, New System.EventArgs)
            D3335.Alarm_Threshold_TextBox.Text = (255).ToString()
            D3335.Alarm_Threshold_TextBox_LostFocus(D3335.Alarm_Threshold_TextBox, New System.EventArgs)
            D3335.Clear_Int_Button_Click(D3335.Clear_Int_Button, New System.EventArgs)
            mfg_file.Append(mfg_file_data, print_label)
        End If

        Me.test_texbox.Text = test_state.ToString + " : " + test_state_timer.ToString
    End Sub

    Private Sub test_button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles test_button.Click
        If (test_state = Mfg_Test.State.Idle) Then
            Me.start_test()
        Else
            Me.stop_test(True)
        End If
    End Sub

    Private Sub test_timer_tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles test_timer.Tick

        Dim error_msg As String = ""

        ' Increment timer
        test_state_timer += INTERVAL

        Select Case test_state

            Case Mfg_Test.State.Idle
                ' Do nothing until test is started

            Case Mfg_Test.State.Read_VBAT
                ' Read VBAT from PIC.  Fail if not within 10% after 5 seconds.
                Dim vbat As Single = CSng(D3335.Battery_Voltage_TextBox.Text.Replace("V", ""))
                mfg_file_data.vbat_start = vbat

                If (vbat >= 5 * 0.9) And (vbat <= 5 * 1.1) Then
                    test_state = CType(Me.test_state + 1, Mfg_Test.State)
                    test_state_timer = 0
                    test_cal_count = 0
                ElseIf test_state_timer > 5000 Then
                    error_msg = "VBAT is out of range: " + vbat.ToString() + "V"
                    error_msg += ChrW(13) + ChrW(10)
                    error_msg += "Either power supply is not within range, or ADC circuit has fault."
                End If

            Case Mfg_Test.State.Calibrate_VBAT
                'Write VBAT to 5.0V and read back after 2 seconds.  Acceptable to repeat 2 times until VBAT is within 1%.
                If test_state_timer = INTERVAL Then
                    D3335.Battery_Voltage_TextBox.Text = (5).ToString()
                    D3335.Battery_Voltage_TextBox_LostFocus(D3335.Battery_Voltage_TextBox, New System.EventArgs)
                End If

                Dim vbat As Single = CSng(D3335.Battery_Voltage_TextBox.Text.Replace("V", ""))
                mfg_file_data.vbat_end = vbat

                If (vbat >= 5 * 0.99) And (vbat <= 5 * 1.01) Then
                    test_state = CType(Me.test_state + 1, Mfg_Test.State)
                    test_state_timer = 0
                ElseIf test_state_timer > 5000 Then
                    If (test_cal_count < 2) Then
                        test_cal_count += 1
                        test_state_timer = 0
                    Else
                        error_msg = "VBAT could not calibrate: " + vbat.ToString() + "V"
                        error_msg += ChrW(13) + ChrW(10)
                        error_msg += "Either power supply is not stable, or ADC circuit has fault."
                    End If
                End If

            Case Mfg_Test.State.Read_VOUT
                ' Read VOUT from PIC.  Fail if not within 10% after 5 seconds.
                Dim vout As Single = CSng(D3335.Output_Voltage_TextBox.Text.Replace("V", ""))
                mfg_file_data.vout_start = vout

                If (vout >= 3.3 * 0.9) And (vout <= 3.3 * 1.1) Then
                    test_state = CType(Me.test_state + 1, Mfg_Test.State)
                    test_state_timer = 0
                    test_cal_count = 0
                ElseIf test_state_timer > 5000 Then
                    error_msg = "VOUT is out of range: " + vout.ToString() + "V"
                    error_msg += ChrW(13) + ChrW(10)
                    error_msg += "Either JP1-JP4 are incorrectly set, ADC circuit has fault, or LTC3335 circuit has a fault."
                End If

            Case Mfg_Test.State.Calibrate_VOUT
                ' Write VBAT to 3.3V and read back after 5 seconds.  Acceptable to repeat 2 times until VBAT is within 1%.
                If test_state_timer = INTERVAL Then
                    D3335.Output_Voltage_TextBox.Text = (3.3).ToString()
                    D3335.Output_Voltage_TextBox_LostFocus(D3335.Output_Voltage_TextBox, New System.EventArgs)
                End If

                Dim vout As Single = CSng(D3335.Output_Voltage_TextBox.Text.Replace("V", ""))
                mfg_file_data.vout_end = vout

                If (vout >= 3.3 * 0.99) And (vout <= 3.3 * 1.01) Then
                    test_state = CType(Me.test_state + 1, Mfg_Test.State)
                    test_state_timer = 0
                ElseIf test_state_timer > 5000 Then
                    If (test_cal_count < 2) Then
                        test_cal_count += 1
                        test_state_timer = 0
                    Else
                        error_msg = "VOUT could not calibrate: " + vout.ToString() + "V"
                        error_msg += ChrW(13) + ChrW(10)
                        error_msg += "Either ADC circuit has or LTC3335 circuit has a fault."
                    End If
                End If

            Case Mfg_Test.State.I2C_Comm
                ' Set output to 1.8V to verify register can be written.
                If test_state_timer = INTERVAL Then
                    D3335.Voltage_Selection_CenteredComboBox.SelectedItem = LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_1_8V.ToString
                End If

                Dim vout As Single = CSng(D3335.Output_Voltage_TextBox.Text.Replace("V", ""))
                If (vout >= 1.75) And (vout <= 1.85) Then
                    test_state = CType(Me.test_state + 1, Mfg_Test.State)
                    test_state_timer = 0
                ElseIf test_state_timer > 5000 Then
                    error_msg = "Could not set voltage to 1.8V through I2C: "
                    error_msg += ChrW(13) + ChrW(10)
                    error_msg += "Is Resistor Attached to Output, Jumper PCB on rows B and C of JP5, and the EN Jumper in the ""ON"" Position?"
                End If

            Case Mfg_Test.State.Config_Ipeak_Vout_En
                ' Write IPEAK setting to 100mA.
                ' Write VOUT setting to 3.3V.
                ' Write EN setting to ON.
                If test_state_timer = INTERVAL Then
                    D3335.Open_IPEAK_Setting_MouseClick(D3335.LTC3335_IPK0_TextBox, New MouseEventArgs(0, 0, 0, 0, 0))
                    D3335.Ipeak_Selection.Set_Selection(LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_100MA)
                    D3335.Ipeak_Selection.OK_Button_Click(D3335.Ipeak_Selection, New System.EventArgs)

                    D3335.Open_VOUT_Setting_MouseClick(D3335.LTC3335_OUT0_TextBox, New MouseEventArgs(0, 0, 0, 0, 0))
                    D3335.Vout_Selection.Set_Selection(LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_3_3V)
                    D3335.Vout_Selection.OK_Button_Click(D3335.Vout_Selection, New System.EventArgs)

                    D3335.Open_EN_Setting_MouseClick(D3335.LTC3335_EN_TextBox, New MouseEventArgs(0, 0, 0, 0, 0))
                    D3335.En_Selection.Set_Selection(LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_ON)
                    D3335.En_Selection.OK_Button_Click(D3335.En_Selection, New System.EventArgs)
                Else
                    If (D3335.Ipeak_Selection.Get_Selection() = LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_100MA) And
                       (D3335.Vout_Selection.selection = LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_3_3V) And
                       (D3335.En_Selection.selection = LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_ON) Then
                        test_state = CType(Me.test_state + 1, Mfg_Test.State)
                        test_state_timer = 0
                    Else
                        error_msg = "Could not configure Ipeak, Vout, or En: "
                        error_msg += ChrW(13) + ChrW(10)
                        error_msg += "USB or PIC Error."
                    End If
                End If

            Case Mfg_Test.State.Config_Capacity
                ' Write Battery Capacity to 2.4Ah.

                Dim CAPACITY As Single = 2.4

                If test_state_timer = INTERVAL Then
                    D3335.Double_Click_CallBack(D3335.battery_cc_graphic.battery_capacity_ah, D3335.battery_cc_graphic.battery_coulombs_discharged_ah, D3335.battery_cc_graphic.alarm_ah)
                    D3335.Battery_Selection.battery_cc_graphic.Battery_Capacity_Set(CAPACITY, True)
                    D3335.Battery_Selection.OK_Button_Click(D3335.Battery_Selection, New System.EventArgs)
                Else
                    If D3335.battery_cc_graphic.battery_capacity_ah = CAPACITY Then
                        test_state = CType(Me.test_state + 1, Mfg_Test.State)
                        test_state_timer = 0
                    Else
                        error_msg = "Could not configure Capacity: "
                        error_msg += ChrW(13) + ChrW(10)
                        error_msg += "USB or PIC Error."
                    End If
                End If

            Case Mfg_Test.State.IRQ
                ' Read /IRQ from GUI.  Fail if low (0).
                ' Set LTC3335 Accumulator and Alarm to 255.
                ' Read /IRQ from GUI.  Fail if high (1).
                If test_state_timer = INTERVAL Then
                    If D3335.LTC3335_nIRQ_Textbox.Text = "1" Then
                        D3335.Accumulator_TextBox.Text = (255).ToString()
                        D3335.Accumulator_TextBox_LostFocus(D3335.Accumulator_TextBox, New System.EventArgs)
                        D3335.Alarm_Threshold_TextBox.Text = (255).ToString()
                        D3335.Alarm_Threshold_TextBox_LostFocus(D3335.Alarm_Threshold_TextBox, New System.EventArgs)
                    Else
                        error_msg = "/IRQ started low: "
                        error_msg += ChrW(13) + ChrW(10)
                        error_msg += "LTC3335 circuit has a fault."
                    End If
                ElseIf D3335.LTC3335_nIRQ_Textbox.Text = "0" Then
                    test_state = CType(Me.test_state + 1, Mfg_Test.State)
                    test_state_timer = 0
                ElseIf test_state_timer > 1000 Then
                    error_msg = "/IRQ did not fall: "
                    error_msg += ChrW(13) + ChrW(10)
                    error_msg += "LTC3335 circuit has a fault."
                End If

            Case Mfg_Test.State.PGOOD
                ' Read PGOOD from GUI.  Fail if low (0).
                ' Set LTC3335 Voltage Selection To SOFTWARE_CONTROL_5_0_V.
                ' Read PGOOD from GUI.  Fail if high (1).
                If test_state_timer = INTERVAL Then
                    If D3335.LTC3335_PGOOD_TextBox.Text = "1" Then
                        D3335.Voltage_Selection_CenteredComboBox.SelectedItem = LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_5_0V.ToString
                    Else
                        error_msg = "PGOOD started low: "
                        error_msg += ChrW(13) + ChrW(10)
                        error_msg += "LTC3335 circuit has a fault."
                    End If
                ElseIf D3335.LTC3335_PGOOD_TextBox.Text = "0" Then
                    test_state = CType(Me.test_state + 1, Mfg_Test.State)
                    test_state_timer = 0
                ElseIf test_state_timer > 1000 Then
                    error_msg = "PGOOD did not fall: "
                    error_msg += ChrW(13) + ChrW(10)
                    error_msg += "LTC3335 circuit has a fault."
                End If

            Case Mfg_Test.State.Write_SN
                ' Generate S/N if board does not already have S/N.
                If test_state_timer > 1000 Then
                    mfg_file_data.Set_Serial_Number_End(mfg_file)
                    mfg_file_data.comment = "Passed"
                    If mfg_file_data.serial_number_start <> mfg_file_data.serial_number_end Then
                        stop_test(True, True)
                        D3335.SN_TextBox.Text = mfg_file_data.serial_number_end
                        D3335.SN_TextBox_LostFocus(D3335.SN_TextBox, New EventArgs)
                    Else
                        stop_test(True, False)
                    End If

                    test_popup.Mfg_Test("Serial Number: " + D3335.SN_TextBox.Text, True)
                End If

        End Select

        ' If error, display popup and stop test
        If error_msg <> "" Then
            mfg_file_data.comment = error_msg
            Me.stop_test(True)
            '            If test_popup.Visible = False Then
            test_popup.Mfg_Test(error_msg)
            'End If
        End If

        ' display state
        Me.test_texbox.Text = test_state.ToString + " : " + test_state_timer.ToString

    End Sub

End Class

Public Class Mfg_File
    Private Const MFG_FILE_NAME As String = "MfgData.txt"
    Private Const MFG_LABEL_PRINT_COMMAND As String = "Mfg_Print_SN_Label.exe"

    Private Const start_of_record_delimiter As String = "#"
    Private Const record_field_delimiter As String = ","
    Private Const comment_delimiter As String = ";"

    Private filepath As String
    Private filename As String
    Private location As Byte
    Private product As String
    Private Const INVALID_LOCATION As Byte = &HFF

    Public Class Mfg_File_Data

        Private board_number As UInteger
        Private test_date As String
        Private test_time As String
        Public serial_number_start As String
        Public vbat_start As Single
        Public vbat_end As Single
        Public vout_start As Single
        Public vout_end As Single
        Public serial_number_end As String
        Public comment As String

        ' Generates new line of mfg data.
        Public Sub New(ByVal mfg_file As Mfg_File, ByVal serial_number_start As String)
            Dim serial_number As Serial_Number
            serial_number = New Serial_Number(serial_number_start)
            If (serial_number.valid = False) Then
                Me.board_number = 0 ' do not assign serial number until the board passes
            Else
                Me.board_number = serial_number.board_number
            End If
            Me.test_date = String.Format("{0:MM/dd/yyyy}", DateTime.Now)
            Me.test_time = String.Format("{0:HH:mm}", DateTime.Now)
            Me.serial_number_start = serial_number_start
            Me.vbat_start = -1
            Me.vbat_end = -1
            Me.vout_start = -1
            Me.vout_end = -1
            Me.serial_number_end = serial_number_start
            Me.comment = ""
        End Sub

        ' Formats line of mfg data into string to write to mfg file.
        Public Function Get_MfgData_String() As String
            Dim mfg_data_string As String

            mfg_data_string = start_of_record_delimiter + Me.board_number.ToString()
            mfg_data_string += record_field_delimiter + Me.test_date
            mfg_data_string += record_field_delimiter + Me.test_time
            mfg_data_string += record_field_delimiter + Me.serial_number_start
            mfg_data_string += record_field_delimiter + Me.vbat_start.ToString()
            mfg_data_string += record_field_delimiter + Me.vbat_end.ToString()
            mfg_data_string += record_field_delimiter + Me.vout_start.ToString()
            mfg_data_string += record_field_delimiter + Me.vout_end.ToString()
            mfg_data_string += record_field_delimiter + Me.serial_number_end
            mfg_data_string += comment_delimiter + Me.comment

            Return mfg_data_string
        End Function

        ' Sets the end serial number for the PCB, unless the start serial number is valid.
        Public Sub Set_Serial_Number_End(ByVal mfg_file As Mfg_File)
            Dim serial_number As Serial_Number
            serial_number = New Serial_Number(Me.serial_number_start)
            If (serial_number.valid = False) Then
                Me.board_number = mfg_file.Get_Max_Board_Number() + 1UI
                serial_number = New Serial_Number(Me.board_number, mfg_file.location)
                Me.serial_number_end = serial_number.Get_String()
            End If
        End Sub

    End Class

    Public Sub New(ByVal filepath As String)
        Dim reader As System.IO.StreamReader
        Dim textline As String

        Me.filepath = filepath
        Me.filename = filepath + MFG_FILE_NAME

        reader = New System.IO.StreamReader(Me.filename)
        textline = reader.ReadLine()
        reader.Close()

        ' Determine if file contains path to actual manufacturing data, or if it is the actual manufacturing data itself.
        If (textline.IndexOf(MFG_FILE_NAME) >= 0) Then
            Me.filename = textline
        Else
            Me.filename = filename
        End If
        reader = New System.IO.StreamReader(Me.filename)

        ' Search for product and save
        Dim product_string As String = "Product:"
        Me.product = ""
        While (Not reader.EndOfStream) And (Me.product = "")
            textline = reader.ReadLine()
            If (textline.IndexOf(product_string) >= 0) Then
                textline = textline.Remove(textline.IndexOf(product_string), product_string.Length)
                Me.product = Microsoft.VisualBasic.LTrim(textline)
            End If
        End While

        ' Search for location and save
        Dim location_string As String = "Location:"
        Me.location = INVALID_LOCATION
        While (Not reader.EndOfStream) And (Me.location = INVALID_LOCATION)
            textline = reader.ReadLine()
            If (textline.IndexOf(location_string) >= 0) Then
                textline = textline.Remove(textline.IndexOf(location_string), location_string.Length)
                textline = Microsoft.VisualBasic.LTrim(textline)
                textline = textline.Substring(0, textline.IndexOf(" "))
                Me.location = Byte.Parse(textline)
            End If
        End While
        reader.Close()

    End Sub

    Public Function Is_Valid(ByVal product As String) As Boolean
        If Me.location = INVALID_LOCATION Then
            Return False
        ElseIf Me.product <> product Then
            Return False
        Else
            Return True
        End If
    End Function

    Public Sub Append(ByVal mfg_file_data As Mfg_File_Data, Optional ByVal print_label As Boolean = False)
        Dim writer As System.IO.StreamWriter
        Dim textline As String
        Try
            textline = mfg_file_data.Get_MfgData_String()
            writer = System.IO.File.AppendText(Me.filename)
            writer.WriteLine(textline)
            writer.Flush()
            writer.Close()
            If print_label = True Then
                Me.Print_Label(mfg_file_data.serial_number_end)
            End If
        Catch ex As Exception
            D3335.Handle_Exception(ex)
        End Try

        Return
    End Sub

    Private Sub Print_Label(ByVal serial_number As String)
        Try
            System.Diagnostics.Process.Start(Me.filepath + MFG_LABEL_PRINT_COMMAND, serial_number)
        Catch ex As Exception
            D3335.Handle_Exception(ex)
        End Try
    End Sub


    Public Function Get_Max_Board_Number() As UInteger
        Dim max_board_number As UInteger = 0
        Dim textline As String
        Dim end_of_count_index As Integer
        Dim reader As System.IO.StreamReader

        Try
            reader = New System.IO.StreamReader(Me.filename)

            While (Not reader.EndOfStream)
                textline = reader.ReadLine()
                If (textline.Length > 0) AndAlso (textline(0) = start_of_record_delimiter) Then
                    end_of_count_index = textline.IndexOf(record_field_delimiter)
                    If (end_of_count_index > 0) Then
                        Try
                            Dim this_count As UInteger = UInteger.Parse(textline.Substring(1, end_of_count_index - 1))
                            If this_count > max_board_number Then
                                max_board_number = this_count
                            End If
                        Catch ex As Exception
                            D3335.Handle_Exception(ex)
                        End Try
                    End If
                End If
            End While
            reader.Close()
        Catch ex As Exception
            D3335.Handle_Exception(ex)
        End Try

        Return max_board_number

    End Function

End Class

Public Class Serial_Number
    Public company As String
    Public location As Byte
    Public month As Integer
    Public day As Integer
    Public year As Integer
    Public board_number As UInteger
    Public valid As Boolean

    Public Sub New(ByVal board_number As UInteger, ByVal location As Byte)
        Dim current_date As Date

        Me.company = "LT"
        Me.location = location
        current_date = Microsoft.VisualBasic.Now
        Me.month = Microsoft.VisualBasic.Month(current_date)
        Me.day = Microsoft.VisualBasic.Day(current_date)
        Me.year = Microsoft.VisualBasic.Year(current_date)
        Me.board_number = board_number

        Me.valid = True
    End Sub

    Public Sub New(ByVal sernum_string As String)

        ' Assume string is valid serial number, set to false if any fields are proven false
        Me.valid = True

        ' Try converting serial number into its components, and set valid to False is any copmonent is wrong.
        Try
            If sernum_string.Substring(0, 2) <> "LT" Then
                Me.valid = False
            End If

            Me.location = Byte.Parse(sernum_string.Substring(2, 2))

            Me.month = Integer.Parse(sernum_string.Substring(4, 2))
            If Me.month = 0 Or Me.month > 12 Then
                Me.valid = False
            End If

            Me.day = Integer.Parse(sernum_string.Substring(6, 2))

            Me.year = Integer.Parse(sernum_string.Substring(8, 4))
            If Me.year < 2014 Or Me.year > 2020 Then
                Me.valid = False
            End If

            Dim days_per_month() As Integer = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
            If Me.day <= 0 Or Me.day > days_per_month(Me.month) Then
                If (Me.month = 2) And (Me.day = 29) And DateTime.IsLeapYear(Me.year) Then
                Else
                    Me.valid = False
                End If
            End If

            Me.board_number = UInteger.Parse(sernum_string.Substring(12, 5))
        Catch ex As Exception
            Me.valid = False
        End Try

    End Sub

    Public Function Get_String() As String
        Dim sernum_string As String

        If Me.valid = True Then
            sernum_string = Me.company
            sernum_string += Me.location.ToString("D2")
            sernum_string += Me.month.ToString("D2")
            sernum_string += Me.day.ToString("D2")
            sernum_string += Me.year.ToString("D4")
            sernum_string += Me.board_number.ToString("D5")
        Else
            sernum_string = "N/A"
        End If

        Return sernum_string

    End Function

End Class
