﻿Option Strict On
Option Explicit On

Module PipeServer_Commands

    Public Const PIPE_NAME As String = "DC2343_Pipe.txt"
    Public Const COMMAND_DELIMITER As String = "#"
    Public Const DATA_DELIMITER As String = ";"

    Public Const COMMAND_NOT_A_CONTROL_START As Integer = 0
    Public Const COMMAND_NOT_A_CONTROL_END As Integer = COMMAND_BUTTON_START - 1
    Public Const COMMAND_BUTTON_START As Integer = 100
    Public Const COMMAND_BUTTON_END As Integer = COMMAND_READ_LABEL_START - 1
    Public Const COMMAND_READ_LABEL_START As Integer = 200
    Public Const COMMAND_READ_LABEL_END As Integer = COMMAND_SET_TEXTBOX_START - 1
    Public Const COMMAND_SET_TEXTBOX_START As Integer = 300
    Public Const COMMAND_SET_TEXTBOX_END As Integer = COMMAND_READ_TEXTBOX_START - 1
    Public Const COMMAND_READ_TEXTBOX_START As Integer = 400
    Public Const COMMAND_READ_TEXTBOX_END As Integer = COMMAND_SET_COMBOBOX_START - 1
    Public Const COMMAND_SET_COMBOBOX_START As Integer = 500
    Public Const COMMAND_SET_COMBOBOX_END As Integer = COMMAND_READ_COMBOBOX_START - 1
    Public Const COMMAND_READ_COMBOBOX_START As Integer = 600
    Public Const COMMAND_READ_COMBOBOX_END As Integer = COMMAND_SET_CHECKBOX_START - 1
    Public Const COMMAND_SET_CHECKBOX_START As Integer = 700
    Public Const COMMAND_SET_CHECKBOX_END As Integer = COMMAND_READ_CHECKBOX_START - 1
    Public Const COMMAND_READ_CHECKBOX_START As Integer = 800
    Public Const COMMAND_READ_CHECKBOX_END As Integer = COMMAND_LAST
    Public Const COMMAND_LAST As Integer = RESPONSE_ERROR_START - 1

    Public Const RESPONSE_ERROR_START As Integer = 100000

    Public Enum PipeCommand
        ' Commands that do not operate on a control, but on the GUI itself
        Quit = COMMAND_NOT_A_CONTROL_START
        GUI_Version
        Reset_Counter_Test
        Battery_Selection_Show
        Ipeak_Selection_Show
        Vout_Selection_Show
        En_Selection_Show
        Battery_Capacity_Ah
        Battery_Coulombs_Discharged_Ah
        Battery_Alarm_Ah

        ' Click Button Commands
        ClickButtonPopupOK = COMMAND_BUTTON_START
        ClickButtonBatteryCurrentEstimate
        ClickBatterySelectionOK
        ClickBatterySelectionCancel
        ClickIpeakSelectionOK
        ClickIpeakSelectionCancel
        ClickVoutSelectionOK
        ClickVoutSelectionCancel
        ClickEnSelectionOK
        ClickEnSelectionCancel

        ' Read Label Text commands
        GetLabelTextCurrentEstimate = COMMAND_READ_LABEL_START

        ' Set TextBox text
        SetTextBoxAlarmThreshold = COMMAND_SET_TEXTBOX_START
        SetTextBoxAccumulator
        SetTextBoxBatteryCapacity
        SetTextBoxBatteryPercentDischarged
        SetTextBoxBatteryPercentAlarm
        SetTextBoxAccumulatorPoll_Period
        SetTextBoxADCPoll_Period
        SetTextBoxSerialNumber
        SetTextBoxVBAT
        SetTextBoxVOUT

        ' Read TextBox text commands
        ReadTextBoxAlarmThreshold = COMMAND_READ_TEXTBOX_START
        ReadTextBoxAccumulator
        ReadTextBoxBatteryCapacity
        ReadTextBoxBatteryPercentDischarged
        ReadTextBoxBatteryPercentAlarm
        ReadTextBoxSoftwareCorrectionFactor
        ReadTextBoxnIRQ
        ReadTextBoxPGOOD
        ReadTextBoxCounterTestResult
        ReadTextBoxVBAT
        ReadTextBoxVOUT
        ReadTextBoxSerialNumber
        ReadTextBoxFirmwareRevision
        ReadTextBoxAccumulatorPoll_Period
        ReadTextBoxADCPoll_Period

        ' Select ComboBox items
        SelectComboBoxItemVoltage = COMMAND_SET_COMBOBOX_START
        SelectComboBoxItemPrescaler
        SelectComboBoxItemIpeakSelection
        SelectComboBoxItemVoutSelection
        SelectComboBoxItemEnSelection

        ' Read ComboBox items
        ReadComboBoxItemVoltage = COMMAND_READ_COMBOBOX_START
        ReadComboBoxItemPrescaler
        ReadComboBoxItemIpeakSelection
        ReadComboBoxItemVoutSelection
        ReadComboBoxItemEnSelection

        ' Set CheckBox state
        SetCheckBoxManufacturingMode = COMMAND_SET_CHECKBOX_START
        SetCheckBoxSoftwareCorrection

        ' Read Checkbox state
        ReadCheckBoxManufacturingMode = COMMAND_READ_CHECKBOX_START
        ReadCheckBoxSoftwareCorrection

        ' Error Responses
        Response_UnknownCommand = RESPONSE_ERROR_START
        Response_IncorrectNumberofArguments
        Response_IncorrectFormatofArguments
        Response_ArgumentOutOfRange
        Response_Popup

    End Enum

End Module
