﻿Option Strict On
Option Explicit On

Imports Microsoft.VisualBasic
Imports System.Windows.Forms

Public Class Popup

    Public Enum Conditions
        None
        NotWindowsXP
        DPI_Not_Supported
        Initialization_Failed
        Mfg_Test
        Num_Popups
    End Enum

    Public Condition As Conditions = Conditions.None

    ' Popup window should always be on top of GUI Control Panel, but not always on Top of all Windows programs.
    Private Declare Function BringWindowToTop Lib "user32" Alias "BringWindowToTop" (ByVal hwnd As Long) As Long
    Private Sub Popup_Activated(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        BringWindowToTop(CLng(Me.Handle))
    End Sub

    Friend Sub OK_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OK_Button.Click
        Condition = Conditions.None
        If OK_Button.Text = "Yes" Then
            Me.DialogResult = System.Windows.Forms.DialogResult.Yes
        Else
            Me.DialogResult = System.Windows.Forms.DialogResult.OK
        End If
        Me.Close()
    End Sub

    Private Sub Cancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cancel_Button.Click
        Condition = Conditions.None
        If OK_Button.Text = "No" Then
            Me.DialogResult = System.Windows.Forms.DialogResult.No
        Else
            Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
        End If
        Me.Close()
    End Sub

    Public Function NotWindowsXP(ByVal gui_name As String) As Boolean
        Dim label_text As String

        ' Store the condition that caused this popup
        Condition = Conditions.NotWindowsXP

        ' Build the label text
        label_text = "The operating system is not Windows XP or later."
        label_text += ChrW(13) + ChrW(10)
        label_text += ChrW(13) + ChrW(10)
        label_text += gui_name + " requires Windows XP or later."

        ' Display the label text
        PopUp_Label.Text = label_text

        'Resize the display to a bit bigger than the label
        Me.Width = PopUp_Label.Width + PopUp_Label.Location.X * 3
        Me.Height = PopUp_Label.Height + PopUp_Label.Location.Y * 3 + (Me.Height - OK_Button.Location.Y)

        ' Set up the buttons for this popup
        OK_Button.Text = "OK"
        OK_Button.Visible = True
        Cancel_Button.Visible = False

        ' Show popup
        Me.ShowDialog()

        Return True

    End Function

    Public Function DPI_Not_Supported() As Boolean
        Dim label_text As String

        ' Store the condition that caused this popup
        Condition = Conditions.DPI_Not_Supported

        ' Build the label text
        label_text = "This GUI may not display properly with your Windows Display settings."
        label_text += ChrW(13) + ChrW(10)
        label_text += ChrW(13) + ChrW(10)
        label_text += "Please use Small (100%) or Large (150%) fonts, or uncheck the"
        label_text += ChrW(13) + ChrW(10)
        label_text += """Use Windows XP Style DPI Scaling"" checkbox if available."

        ' Display the label text
        PopUp_Label.Text = label_text

        'Resize the display to a bit bigger than the label
        Me.Width = PopUp_Label.Width + PopUp_Label.Location.X * 3
        Me.Height = PopUp_Label.Height + PopUp_Label.Location.Y * 3 + (Me.Height - OK_Button.Location.Y)

        ' Set up the buttons for this popup
        OK_Button.Text = "OK"
        OK_Button.Visible = True
        Cancel_Button.Visible = False

        ' Show popup
        Me.ShowDialog()

        Return True

    End Function

    Public Function Initialization_Failed() As Boolean
        Dim label_text As String

        ' Store the condition that caused this popup
        Condition = Conditions.Initialization_Failed

        ' Build the label text
        label_text = "PIC16F1459 failed initial communication with LTC3335."
        label_text += ChrW(13) + ChrW(10)
        label_text += ChrW(13) + ChrW(10)
        label_text += "Is Jumper PCB on rows B and C of JP5, and the EN Jumper in the ""ON"" Position?"

        ' Display the label text
        PopUp_Label.Text = label_text

        'Resize the display to a bit bigger than the label
        Me.Width = PopUp_Label.Width + PopUp_Label.Location.X * 3
        Me.Height = PopUp_Label.Height + PopUp_Label.Location.Y * 3 + (Me.Height - OK_Button.Location.Y)

        ' Set up the buttons for this popup
        OK_Button.Text = "OK"
        OK_Button.Visible = True
        Cancel_Button.Visible = False

        ' Show popup
        Me.ShowDialog()

        Return True

    End Function

    Public Function Mfg_Test(ByVal test_msg As String, Optional ByVal pass As Boolean = False) As Boolean
        Dim label_text As String

        ' Store the condition that caused this popup
        Condition = Conditions.Mfg_Test

        ' Build the label text
        label_text = "Manufacturing Test "
        If pass = True Then
            label_text += "Passed:"
        Else
            label_text += "Failed:"
        End If
        label_text += ChrW(13) + ChrW(10)
        label_text += ChrW(13) + ChrW(10)
        label_text += test_msg

        ' Display the label text
        PopUp_Label.Text = label_text

        'Resize the display to a bit bigger than the label
        Me.Width = PopUp_Label.Width + PopUp_Label.Location.X * 3
        Me.Height = PopUp_Label.Height + PopUp_Label.Location.Y * 3 + (Me.Height - OK_Button.Location.Y)

        ' Set up the buttons for this popup
        OK_Button.Text = "OK"
        OK_Button.Visible = True
        Cancel_Button.Visible = False

        ' Show popup
        Me.ShowDialog()

        Return True

    End Function

    Private Sub Popup_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' todo - I can't seem to loop through all forms, so copy this code to each form for now.
        ' First set the Use Compatible Text Rendering property.
        For Each control As Control In MyBase.Controls
            D3335.Set_UseCompatibleTextRendering(control)
        Next control

        ' Next set all of the fonts to Linear Helv Cond.
        For Each control As Control In MyBase.Controls
            D3335.Set_Font(control)
        Next control
    End Sub
End Class
