﻿Option Strict On
Option Explicit On

Imports System.Globalization

<System.ComponentModel.DesignerCategory("code")> Partial Class D3335_USB

    Public Const RESPONSE_STREAM_FIRST As Byte = &H80
    Public Const NUM_RESPONSES As Integer = (USB_DATA_ID.NUM_DATA_IDS + (USB_DATA_ID.STREAM_LAST - RESPONSE_STREAM_FIRST))

    Public Enum USB_DATA_ID
        FW_REVISION
        SERIAL_NUMBER
        ERROR_DATA
        VOLTAGE_SELECTION
        PRESCALER_SELECTION
        IPEAK_CONFIGURATION
        VOUT_CONFIGURATION
        ALARM_THRESHOLD
        ACCUMULATOR
        ALARMS
        CLEAR_INT
        COUNTER_TEST
        IRQ
        PGOOD
        ADC
        CAPACITY
        NONSTANDARD_INDUCTOR
        EN_CONFIGURATION
        NUM_DATA_IDS
        TODO = RESPONSE_STREAM_FIRST
        STREAM_LAST
        NONE = &HFF
    End Enum

    Public Class DC2343_USB_COMMAND
        Public data_id As USB_DATA_ID
        Public data As Object

        Public Sub New(ByVal data_id As USB_DATA_ID, ByVal data As Object)
            Me.data_id = data_id
            Me.data = data
        End Sub
    End Class

    Public Structure DC2343_USB_RESPONSE
        Public data_id As USB_DATA_ID
        Public time As Single
        Public data As Object
    End Structure

    Public Class Timestamp
        Public Const PIC16F_TIMER_TICKS_PER_SEC As Integer = 1500000            ' in ticks per second
        Public Const SYSTEM_TIMER_TICKS_OVERFLOW As ULong = (1UL << 32)         ' the value at which the timestamps overflow
        Public timestamp_last As UInteger
        Public time As Single
        Public time_difference As Single

        Public Sub New()
            Me.Reset()
        End Sub

        Public Sub Reset()
            Me.timestamp_last = 0
            Me.time = 0
        End Sub

        Public Sub Update(ByVal timestamp_new As UInteger)
            Dim timestamp_difference As ULong

            ' Calculate the amount of time passed since last update, and add to counter
            If (timestamp_new >= Me.timestamp_last) Then
                timestamp_difference = timestamp_new - Me.timestamp_last
            Else
                timestamp_difference = SYSTEM_TIMER_TICKS_OVERFLOW + timestamp_new - Me.timestamp_last
            End If

            Me.time_difference = CSng(timestamp_difference) / PIC16F_TIMER_TICKS_PER_SEC
            Me.time += CSng(timestamp_difference) / PIC16F_TIMER_TICKS_PER_SEC

            Me.timestamp_last = timestamp_new
        End Sub

    End Class

    Public Class READ_COMMAND
    End Class

    Public Class RESET_COMMAND
    End Class

    Public Class REFRESH_COMMAND
    End Class

    Public Function ReceiveResponse(ByRef response As DC2343_USB_RESPONSE) As Boolean
        If USB_Comm_List_In.Count <= 0 Then
            Return False
        Else
            response = USB_Comm_List_In(0)
            USB_Comm_List_In.RemoveAt(0)
            Return True
        End If
    End Function

    Public Function SendCommand(ByVal data_id As USB_DATA_ID, ByVal data As Object) As Boolean
        ' Add new command to list to send.
        If USB_Comm_List_Out.Count < USB_MAX_COMMANDS Then
            USB_Comm_List_Out.Add(New D3335_USB.DC2343_USB_COMMAND(data_id, data))

            ' Start up background worker if not currently sending a command.
            If writepipe_backgroundworker.IsBusy <> True Then
                writepipe_backgroundworker.RunWorkerAsync(USB_Comm_List_Out(0))
            End If

            Return True
        Else
            Return False
        End If
    End Function

    Public Function SendCommandAndReceiveResponse(ByRef response As DC2343_USB_RESPONSE, ByVal command As DC2343_USB_COMMAND, ByVal timeout_ms As Integer) As Boolean

        ' Calculate time after which we should time out.

        ' Tell readpipe_backgroundworker that we're pending on a response

        ' Send the command

        ' Wait for the response or for a time out

    End Function

    Public Enum Callback_Events
        EXCEPTION_HANDLED
        DEVICE_ERROR
        DEVICE_CONNECTED
        DEVICE_DISCONNECTED
        RESPONSE_RECEIVED
        COMMAND_SENT
        NUM_CALLBACKS
    End Enum
    Private Event_CallBacks(Callback_Events.NUM_CALLBACKS - 1) As Action(Of Object)

    Public Enum Device_Event_Codes
        FAILED_TO_START
        NUM_DEVICE_EVENT_CODES
    End Enum

    Public Structure Device_Error_Structure
        Public Event_Code As Device_Event_Codes
        Public error_msg As String
    End Structure

    Public Sub Set_CallBack(ByVal callback_event As Callback_Events, ByVal callback_function As Action(Of Object))
        If callback_event < Callback_Events.NUM_CALLBACKS Then
            Event_CallBacks(callback_event) = callback_function
        End If
    End Sub

#Region "FW_REVISION"
    Public Class FW_REVISION_RESPONSE
        Friend fw_revision As Firmware_Revision

        Public Class Firmware_Revision
            Public Const APP_FW_STRING_DEFAULT As String = "N/A        "
            Public Const delimiter As String = "."
            Public Const beta_delimiter As String = "b"
            Public Valid As Boolean
            Public Rev_Major As Integer
            Public Rev_Minor As Integer
            Public Rev_SubMinor As Integer
            Public Rev_Beta As Integer

            Public Sub New()
                Me.Valid = False
            End Sub

            Public Sub New(ByVal rev_major As Integer, ByVal rev_minor As Integer, ByVal rev_subminor As Integer, ByVal rev_beta As Integer)
                Me.Rev_Major = rev_major
                Me.Rev_Minor = rev_minor
                Me.Rev_SubMinor = rev_subminor
                Me.Rev_Beta = rev_beta
                Me.Valid = True
            End Sub

            Public Sub Set_String(ByVal fw_string As String)
                Dim temp_rev_major As Integer
                Dim temp_rev_minor As Integer
                Dim temp_rev_subminor As Integer
                Dim temp_rev_beta As Integer

                Try
                    ' Attempt to unpack the FW revision.
                    temp_rev_major = Integer.Parse(fw_string.Substring(0, fw_string.IndexOf(delimiter)), NumberStyles.AllowHexSpecifier)
                    fw_string = fw_string.Remove(0, fw_string.IndexOf(delimiter) + 1)
                    temp_rev_minor = Integer.Parse(fw_string.Substring(0, fw_string.IndexOf(delimiter)), NumberStyles.AllowHexSpecifier)
                    fw_string = fw_string.Remove(0, fw_string.IndexOf(delimiter) + 1)
                    If (fw_string.IndexOf(beta_delimiter) >= 0) Then
                        ' This is a beta revision
                        temp_rev_subminor = Integer.Parse(fw_string.Substring(0, fw_string.IndexOf(beta_delimiter)), NumberStyles.AllowHexSpecifier)
                        fw_string = fw_string.Remove(0, fw_string.IndexOf(beta_delimiter) + 1)
                        temp_rev_beta = Integer.Parse(fw_string, NumberStyles.AllowHexSpecifier)
                    Else
                        ' This is not a beta revision
                        temp_rev_subminor = Integer.Parse(fw_string.Trim(), NumberStyles.AllowHexSpecifier)
                        temp_rev_beta = 0
                    End If

                    ' If unpacking was successful, we have a valid FW revision to set
                    Me.Rev_Major = temp_rev_major
                    Me.Rev_Minor = temp_rev_minor
                    Me.Rev_SubMinor = temp_rev_subminor
                    Me.Rev_Beta = temp_rev_beta
                    Me.Valid = True
                Catch ex As Exception
                    Me.Valid = False
                End Try

            End Sub

            Public Function Get_String() As String
                If (Me.Valid = False) Then
                    ' There is no FW revision detected
                    Return APP_FW_STRING_DEFAULT
                ElseIf (Me.Rev_Beta = 0) Then
                    ' This is not a beta FW revision
                    Return Me.Rev_Major.ToString() + delimiter + Me.Rev_Minor.ToString() + delimiter + Me.Rev_SubMinor.ToString()
                Else
                    ' There is a beta FW revision
                    Return Me.Rev_Major.ToString() + delimiter + Me.Rev_Minor.ToString() + delimiter + Me.Rev_SubMinor.ToString() + beta_delimiter + Me.Rev_Beta.ToString()
                End If
            End Function

            Public Function Is_Older_Than(ByVal fw_rev As Firmware_Revision) As Boolean
                If (Me.Valid = False) Or (fw_rev.Valid = False) Then
                    Return True
                ElseIf (Me.Rev_Major <> fw_rev.Rev_Major) Then
                    If (Me.Rev_Major < fw_rev.Rev_Major) Then
                        Return True
                    Else
                        Return False
                    End If
                ElseIf (Me.Rev_Minor <> fw_rev.Rev_Minor) Then
                    If (Me.Rev_Minor < fw_rev.Rev_Minor) Then
                        Return True
                    Else
                        Return False
                    End If
                ElseIf (Me.Rev_SubMinor <> fw_rev.Rev_SubMinor) Then
                    If (Me.Rev_SubMinor < fw_rev.Rev_SubMinor) Then
                        Return True
                    Else
                        Return False
                    End If
                ElseIf (Me.Rev_Beta <> fw_rev.Rev_Beta) Then
                    If (Me.Rev_Beta < fw_rev.Rev_Beta) Then
                        Return True
                    Else
                        Return False
                    End If
                Else
                    Return False
                End If
            End Function

            Public Shared Operator =(ByVal ver1 As Firmware_Revision, ByVal ver2 As Firmware_Revision) As Boolean
                Dim equals As Boolean = True
                equals = equals And (ver1.Rev_Major = ver2.Rev_Major)
                equals = equals And (ver1.Rev_Minor = ver2.Rev_Minor)
                equals = equals And (ver1.Rev_SubMinor = ver2.Rev_SubMinor)
                equals = equals And (ver1.Rev_Beta = ver2.Rev_Beta)
                Return equals
            End Operator

            Public Shared Operator <>(ByVal ver1 As Firmware_Revision, ByVal ver2 As Firmware_Revision) As Boolean
                Return Not (ver1 = ver2)
            End Operator
        End Class

        Public Sub New(Optional ByVal board_num As Integer = 0)
            Me.fw_revision = New Firmware_Revision
        End Sub

    End Class
#End Region

#Region "SERIAL_NUMBER"

    Public Const SERIAL_NUM_DEFAULT As String = "None             "

    Public Class SERIAL_NUMBER_COMMAND_WRITE
        Public serial_number As String

        Public Sub New()
            serial_number = SERIAL_NUM_DEFAULT
        End Sub
    End Class

    Public Class SERIAL_NUMBER_RESPONSE
        Public serial_number As String
        Public Sub New()
            serial_number = SERIAL_NUM_DEFAULT
        End Sub
    End Class

#End Region

#Region "ERROR_DATA"
    ' From "Error Codes" Sheet in DC2343_Design.xlsm
    ' Definition of error codes passed from the App FW to the GUI
    Public Enum Error_Codes
        NONE                        ' No Error is present in DC2343 System.
        TEST                        ' Test Code for sending ERROR_DATA_SIZE raw bytes to the DC2343 GUI.
        LTC3335_I2C_FAIL            ' An I2C read or write failed to a register in the LTC3335.
        USB_ERROR                   ' A USB Error was detected.
        NUM_ERROR_CODES
    End Enum
    Public Const ERROR_DATA_SIZE As Integer = 12

    Public Class ERROR_DATA_RESPONSE
        Public error_code As Error_Codes
        Public error_data(ERROR_DATA_SIZE - 1) As Byte
        Public error_count(Error_Codes.NUM_ERROR_CODES - 1) As Integer

        Public Sub New()
            error_code = Error_Codes.NONE
        End Sub
    End Class

#End Region

#Region "VOLTAGE_SELECTION"

    Public Class VOLTAGE_SELECTION_COMMAND_WRITE
        Public voltage_selection As LTC3335.LTC3335.VOLTAGE_SELECTION_OPTIONS

        Public Sub New()
            voltage_selection = LTC3335.LTC3335.VOLTAGE_SELECTION_OPTIONS.HARDWARE_CONTROL
        End Sub
    End Class

    Public Class VOLTAGE_SELECTION_RESPONSE
        Public voltage_selection As LTC3335.LTC3335.VOLTAGE_SELECTION_OPTIONS

        Public Sub New()
            voltage_selection = LTC3335.LTC3335.VOLTAGE_SELECTION_OPTIONS.HARDWARE_CONTROL
        End Sub
    End Class
#End Region

#Region "PRESCALER_SELECTION"

    Public Class PRESCALER_SELECTION_COMMAND_WRITE
        Public prescaler_selection As Integer

        Public Sub New()
            prescaler_selection = 0
        End Sub
    End Class

    Public Class PRESCALER_SELECTION_RESPONSE
        Public prescaler_selection As Integer
        Public Sub New()
            prescaler_selection = 0
        End Sub
    End Class
#End Region

#Region "IPEAK_CONFIGURATION"

    Public Class Ipeak_Selection_COMMAND_WRITE
        Public Ipeak_Selection As LTC3335.IPEAK_SELECTION_OPTIONS

        Public Sub New()
            Ipeak_Selection = LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_100MA
        End Sub
    End Class

    Public Class Ipeak_Selection_RESPONSE
        Public Ipeak_Selection As LTC3335.IPEAK_SELECTION_OPTIONS
        Public Sub New()
            Ipeak_Selection = LTC3335.IPEAK_SELECTION_OPTIONS.IPEAK_100MA
        End Sub
    End Class
#End Region

#Region "VOUT_CONFIGURATION"

    Public Class VOUT_CONFIGURATION_COMMAND_WRITE
        Public vout_configuration As LTC3335.LTC3335.VOLTAGE_SELECTION_OPTIONS

        Public Sub New()
            vout_configuration = LTC3335.LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_3_3V
        End Sub
    End Class

    Public Class VOUT_CONFIGURATION_RESPONSE
        Public vout_configuration As LTC3335.LTC3335.VOLTAGE_SELECTION_OPTIONS
        Public Sub New()
            vout_configuration = LTC3335.LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_3_3V
        End Sub
    End Class
#End Region

#Region "ALARM_THRESHOLD"
    Public Const ALARM_THRESHOLD_MIN As Integer = 0
    Public Const ALARM_THRESHOLD_MAX As Integer = 255

    Public Class ALARM_THRESHOLD_COMMAND_WRITE
        Public threshold As Integer

        Public Sub New()
            threshold = 0
        End Sub
    End Class

    Public Class ALARM_THRESHOLD_RESPONSE
        Public threshold As Integer

        Public Sub New()
            threshold = 0
        End Sub
    End Class
#End Region

#Region "ACCUMULATOR"
    Public Const ACCUMULATOR_MIN As Integer = 0
    Public Const ACCUMULATOR_MAX As Integer = 255

    Public Class ACCUMULATOR_COMMAND_WRITE
        Public accumulator As Integer

        Public Sub New()
            accumulator = 0
        End Sub
    End Class

    Public Class ACCUMULATOR_RESPONSE
        Public accumulator As Integer

        Public Sub New()
            accumulator = 0
        End Sub
    End Class
#End Region

#Region "ALARMS"
    Public Class ALARMS_RESPONSE
        Public ac_on_time_overflow As Boolean
        Public coulomb_counter_overflow As Boolean
        Public alarm_threshold As Boolean

        Public Sub New()
            ac_on_time_overflow = False
            coulomb_counter_overflow = False
            alarm_threshold = False
        End Sub
    End Class
#End Region

#Region "CLEAR_INT"
    Public Class CLEAR_INT_COMMAND_WRITE
        Public activate As Boolean

        Public Sub New()
            activate = False
        End Sub
    End Class

    Public Class CLEAR_INT_RESPONSE
        Public activated As Boolean

        Public Sub New()
            activated = False
        End Sub
    End Class
#End Region

#Region "COUNTER_TEST"
    Public Class COUNTER_TEST_COMMAND_WRITE
        Public activate As Boolean

        Public Sub New()
            activate = False
        End Sub
    End Class

    Public Class COUNTER_TEST_RESPONSE
        Public activated As Boolean
        Public edge_count As UInteger
        Public time As Single

        Public Sub New()
            activated = False
            edge_count = 0
            time = 0
        End Sub
    End Class
#End Region

#Region "IRQ"
    Public Class IRQ_RESPONSE
        Public pin_state As Boolean
        Public Sub New()
            pin_state = False
        End Sub
    End Class
#End Region

#Region "PGOOD"

    Public Class PGOOD_RESPONSE
        Public pin_state As Boolean
        Public Sub New()
            pin_state = False
        End Sub
    End Class
#End Region

#Region "ADC"
    Public Class ADC_COMMAND_WRITE
        Public vbat As Boolean
        Public voltage As Single

        Public Sub New()
            vbat = False
            voltage = 0
        End Sub
    End Class

    Public Class ADC_RESPONSE
        Public vbat As Single
        Public vout As Single
        Public Sub New()
            vbat = 0
            vout = 0
        End Sub
    End Class
#End Region

#Region "CAPACITY"
    Public Class CAPACITY_COMMAND_WRITE
        Public capacity As Single
        Public Sub New(ByVal capacity As Single)
            Me.capacity = capacity
        End Sub
    End Class

    Public Class CAPACITY_RESPONSE
        Public capacity As Single
    End Class
#End Region

#Region "NONSTANDARD_INDUCTOR"
    Public Const NONSTANDARD_INDUCTOR_SIZE As Integer = 16

    Public Class NONSTANDARD_INDUCTOR_COMMAND_WRITE
        Public nonstandard_inductor As String

        Public Sub New()
            Dim temp_bytes(NONSTANDARD_INDUCTOR_SIZE - 1) As Byte

            For Each item As Byte In temp_bytes
                item = 0
            Next item

            nonstandard_inductor = System.Text.Encoding.ASCII.GetString(temp_bytes)
        End Sub

    End Class

    Public Class NONSTANDARD_INDUCTOR_RESPONSE
        Public nonstandard_inductor As String

        Public Sub New()
            Dim temp_bytes(NONSTANDARD_INDUCTOR_SIZE - 1) As Byte

            For Each item As Byte In temp_bytes
                item = 0
            Next item

            nonstandard_inductor = System.Text.Encoding.ASCII.GetString(temp_bytes)
        End Sub


    End Class

#End Region

#Region "EN_CONFIGURATION"

    Public Class EN_CONFIGURATION_COMMAND_WRITE
        Public en_configuration As LTC3335.EN_CONFIGURATION_OPTIONS

        Public Sub New()
            en_configuration = LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_ON
        End Sub
    End Class

    Public Class EN_CONFIGURATION_RESPONSE
        Public en_configuration As LTC3335.EN_CONFIGURATION_OPTIONS

        Public Sub New()
            en_configuration = LTC3335.EN_CONFIGURATION_OPTIONS.EN_CONFIGURATION_ON
        End Sub
    End Class
#End Region

End Class
