﻿Option Strict On
Option Explicit On

Imports Microsoft.VisualBasic
Imports System.Net
Imports System.Text
Imports System.Runtime.InteropServices
Imports System.Runtime.InteropServices.Marshal

Partial Class D3335_USB

#Region "FW_REVISION"

    ' Wicked hack to allow support of the FW Rev 1.0.0b2 boards that were sent early to a few customers.
    Private fw_revision As New FW_REVISION_RESPONSE.Firmware_Revision

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure FW_REVISION_STRUCT
        <FieldOffset(0)> Friend rev_major As Byte
        <FieldOffset(1)> Friend rev_minor As Byte
        <FieldOffset(2)> Friend rev_subminor As Byte
        <FieldOffset(3)> Friend rev_beta As Byte
    End Structure

    Friend Function Send_Fw_Revision_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Fw_Revision_Response(ByVal input_bytes() As Byte, ByRef output_obj As FW_REVISION_RESPONSE) As Boolean
        Dim data As New FW_REVISION_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.rev_major)
        success = success And Get_Network_Byte_Order(input_bytes, index, data.rev_minor)
        success = success And Get_Network_Byte_Order(input_bytes, index, data.rev_subminor)
        success = success And Get_Network_Byte_Order(input_bytes, index, data.rev_beta)

        ' Put data in response object
        output_obj.fw_revision = New FW_REVISION_RESPONSE.Firmware_Revision(data.rev_major, data.rev_minor, data.rev_subminor, data.rev_beta)

        ' Wicked hack to allow support of the FW Rev 1.0.0b2 boards that were sent early to a few customers.
        Me.fw_revision = output_obj.fw_revision

        Return success
    End Function

#End Region

#Region "SERIAL_NUMBER"
    <StructLayout(LayoutKind.Sequential, Pack:=1)> _
    Private Structure SERIAL_NUMBER_STRUCT
        <MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst:=17)> Friend serial_num() As Byte
    End Structure

    Friend Function Send_Serial_Number_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(SERIAL_NUMBER_COMMAND_WRITE)) Then
            Dim usb_data As SERIAL_NUMBER_STRUCT
            Dim api_data As SERIAL_NUMBER_COMMAND_WRITE = DirectCast(input_obj, SERIAL_NUMBER_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim usb_data.serial_num(SERIAL_NUM_DEFAULT.Length - 1)
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            usb_data.serial_num = Encoding.ASCII.GetBytes(api_data.serial_number)

            ' Put packed struct in output buffer
            Array.Copy(usb_data.serial_num, output_bytes, usb_data.serial_num.Length)

        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Serial_Number_Response(ByVal input_bytes() As Byte, ByRef output_obj As SERIAL_NUMBER_RESPONSE) As Boolean
        Dim data As New SERIAL_NUMBER_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ReDim data.serial_num(SERIAL_NUM_DEFAULT.Length - 1)
        Array.Copy(input_bytes, index, data.serial_num, 0, data.serial_num.Length)

        ' Put data in response object
        output_obj.serial_number = Encoding.ASCII.GetString(data.serial_num)

        Return success
    End Function
#End Region

#Region "ERROR_DATA"
    <StructLayout(LayoutKind.Sequential, Pack:=1)> _
    Private Structure ERROR_DATA_STRUCT
        <MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst:=1)> Friend error_code As Byte
        <MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst:=D3335_USB.ERROR_DATA_SIZE)> Friend error_data() As Byte
        <MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst:=D3335_USB.Error_Codes.NUM_ERROR_CODES)> Friend error_count() As UInt16
    End Structure

    Friend Function Send_Error_Data_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Error_Data_Response(ByVal input_bytes() As Byte, ByRef output_obj As ERROR_DATA_RESPONSE) As Boolean
        Dim data As New ERROR_DATA_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        ReDim data.error_data(D3335_USB.ERROR_DATA_SIZE - 1)
        ReDim data.error_count(D3335_USB.Error_Codes.NUM_ERROR_CODES - 1)

        success = success And Get_Network_Byte_Order(input_bytes, index, data.error_code)
        For byte_num As Integer = 0 To D3335_USB.ERROR_DATA_SIZE - 1
            success = success And Get_Network_Byte_Order(input_bytes, index, data.error_data(byte_num))
        Next byte_num

        ' Wicked hack to allow support of the FW Rev 1.0.0b2 boards that were sent early to a few customers.
        For error_num As Integer = 0 To If(Me.fw_revision.Get_String = "1.0.0b2", 2, D3335_USB.Error_Codes.NUM_ERROR_CODES - 1)
            success = success And Get_Network_Byte_Order(input_bytes, index, data.error_count(error_num))
        Next error_num

        ' Put data in response object
        output_obj.error_code = CType(data.error_code, Error_Codes)
        For byte_num As Integer = 0 To D3335_USB.ERROR_DATA_SIZE - 1
            output_obj.error_data(byte_num) = data.error_data(byte_num)
        Next byte_num
        For error_num As Integer = 0 To D3335_USB.Error_Codes.NUM_ERROR_CODES - 1
            output_obj.error_count(error_num) = data.error_count(error_num)
        Next error_num

        Return success
    End Function

#End Region

#Region "VOLTAGE_SELECTION"

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure VOLTAGE_SELECTION_COMMAND_STRUCT
        <FieldOffset(0)> Friend enabled As Byte
        <FieldOffset(1)> Friend voltage_selection As Byte
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure VOLTAGE_SELECTION_RESPONSE_STRUCT
        <FieldOffset(0)> Friend enabled As Byte
        <FieldOffset(1)> Friend voltage_selection As Byte
    End Structure

    Friend Function Send_Voltage_Selection_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(VOLTAGE_SELECTION_COMMAND_WRITE)) Then
            Dim usb_data As VOLTAGE_SELECTION_COMMAND_STRUCT
            Dim api_data As VOLTAGE_SELECTION_COMMAND_WRITE = DirectCast(input_obj, VOLTAGE_SELECTION_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            If (api_data.voltage_selection = LTC3335.VOLTAGE_SELECTION_OPTIONS.HARDWARE_CONTROL) Then
                usb_data.enabled = 0
                usb_data.voltage_selection = 0
            Else
                usb_data.enabled = 1
                usb_data.voltage_selection = CByte(api_data.voltage_selection - LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_1_8V)
            End If

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.enabled))
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.voltage_selection))
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Voltage_Selection_Response(ByVal input_bytes() As Byte, ByRef output_obj As VOLTAGE_SELECTION_RESPONSE) As Boolean
        Dim data As New VOLTAGE_SELECTION_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.enabled)
        success = success And Get_Network_Byte_Order(input_bytes, index, data.voltage_selection)

        ' Put data in response object
        If (data.enabled = 0) Then
            output_obj.voltage_selection = LTC3335.VOLTAGE_SELECTION_OPTIONS.HARDWARE_CONTROL
        Else
            output_obj.voltage_selection = CType(data.voltage_selection + LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_1_8V, LTC3335.VOLTAGE_SELECTION_OPTIONS)
        End If

        Return success
    End Function
#End Region

#Region "PRESCALER_SELECTION"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure PRESCALER_SELECTION_COMMAND_STRUCT
        <FieldOffset(0)> Friend prescaler_selection As Byte
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure PRESCALER_SELECTION_RESPONSE_STRUCT
        <FieldOffset(0)> Friend prescaler_selection As Byte
    End Structure

    Friend Function Send_Prescaler_Selection_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(PRESCALER_SELECTION_COMMAND_WRITE)) Then
            Dim usb_data As PRESCALER_SELECTION_COMMAND_STRUCT
            Dim api_data As PRESCALER_SELECTION_COMMAND_WRITE = DirectCast(input_obj, PRESCALER_SELECTION_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            usb_data.prescaler_selection = CByte(api_data.prescaler_selection)

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.prescaler_selection))
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Prescaler_Selection_Response(ByVal input_bytes() As Byte, ByRef output_obj As PRESCALER_SELECTION_RESPONSE) As Boolean
        Dim data As New PRESCALER_SELECTION_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.prescaler_selection)

        ' Put data in response object
        output_obj.prescaler_selection = data.prescaler_selection

        Return success
    End Function
#End Region

#Region "IPEAK_CONFIGURATION"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure Ipeak_Selection_COMMAND_STRUCT
        <FieldOffset(0)> Friend Ipeak_Selection As Byte
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure Ipeak_Selection_RESPONSE_STRUCT
        <FieldOffset(0)> Friend Ipeak_Selection As Byte
    End Structure

    Friend Function Send_Ipeak_Selection_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(Ipeak_Selection_COMMAND_WRITE)) Then
            Dim usb_data As Ipeak_Selection_COMMAND_STRUCT
            Dim api_data As Ipeak_Selection_COMMAND_WRITE = DirectCast(input_obj, Ipeak_Selection_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            usb_data.Ipeak_Selection = CByte(api_data.Ipeak_Selection)

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.Ipeak_Selection))
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Ipeak_Selection_Response(ByVal input_bytes() As Byte, ByRef output_obj As Ipeak_Selection_RESPONSE) As Boolean
        Dim data As New Ipeak_Selection_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.Ipeak_Selection)

        ' Put data in response object
        output_obj.Ipeak_Selection = CType(data.Ipeak_Selection, LTC3335.IPEAK_SELECTION_OPTIONS)

        Return success
    End Function
#End Region

#Region "VOUT_CONFIGURATION"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure VOUT_CONFIGURATION_COMMAND_STRUCT
        <FieldOffset(0)> Friend vout_setting As Byte
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure VOUT_CONFIGURATION_RESPONSE_STRUCT
        <FieldOffset(0)> Friend vout_setting As Byte
    End Structure

    Friend Function Send_Vout_Configuration_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(VOUT_CONFIGURATION_COMMAND_WRITE)) Then
            Dim usb_data As VOUT_CONFIGURATION_COMMAND_STRUCT
            Dim api_data As VOUT_CONFIGURATION_COMMAND_WRITE = DirectCast(input_obj, VOUT_CONFIGURATION_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            usb_data.vout_setting = CByte(api_data.vout_configuration)

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.vout_setting))
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Vout_Configuration_Response(ByVal input_bytes() As Byte, ByRef output_obj As VOUT_CONFIGURATION_RESPONSE) As Boolean
        Dim data As New VOUT_CONFIGURATION_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.vout_setting)

        ' Put data in response object
        output_obj.vout_configuration = CType(data.vout_setting, LTC3335.VOLTAGE_SELECTION_OPTIONS)

        Return success
    End Function
#End Region

#Region "ALARM_THRESHOLD"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure ALARM_THRESHOLD_COMMAND_STRUCT
        <FieldOffset(0)> Friend threshold As Byte
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure ALARM_THRESHOLD_RESPONSE_STRUCT
        <FieldOffset(0)> Friend threshold As Byte
    End Structure

    Friend Function Send_Alarm_Threshold_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(ALARM_THRESHOLD_COMMAND_WRITE)) Then
            Dim usb_data As ALARM_THRESHOLD_COMMAND_STRUCT
            Dim api_data As ALARM_THRESHOLD_COMMAND_WRITE = DirectCast(input_obj, ALARM_THRESHOLD_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            usb_data.threshold = CByte(api_data.threshold)

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.threshold))
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Alarm_Threshold_Response(ByVal input_bytes() As Byte, ByRef output_obj As ALARM_THRESHOLD_RESPONSE) As Boolean
        Dim data As New ALARM_THRESHOLD_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.threshold)

        ' Put data in response object
        output_obj.threshold = data.threshold

        Return success
    End Function
#End Region

#Region "ACCUMULATOR"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure ACCUMULATOR_COMMAND_STRUCT
        <FieldOffset(0)> Friend accumulator As Byte
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure ACCUMULATOR_RESPONSE_STRUCT
        <FieldOffset(0)> Friend accumulator As Byte
    End Structure

    Friend Function Send_Accumulator_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(ACCUMULATOR_COMMAND_WRITE)) Then
            Dim usb_data As ACCUMULATOR_COMMAND_STRUCT
            Dim api_data As ACCUMULATOR_COMMAND_WRITE = DirectCast(input_obj, ACCUMULATOR_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            usb_data.accumulator = CByte(api_data.accumulator)

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.accumulator))
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Accumulator_Response(ByVal input_bytes() As Byte, ByRef output_obj As ACCUMULATOR_RESPONSE) As Boolean
        Dim data As New ACCUMULATOR_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.accumulator)

        ' Put data in response object
        output_obj.accumulator = data.accumulator

        Return success
    End Function
#End Region

#Region "ALARMS"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure ALARMS_RESPONSE_STRUCT
        <FieldOffset(0)> Friend alarms As Byte
    End Structure

    Friend Function Send_Alarms_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Alarms_Response(ByVal input_bytes() As Byte, ByRef output_obj As ALARMS_RESPONSE) As Boolean
        Dim data As New ALARMS_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.alarms)

        ' Put data in response object
        output_obj.ac_on_time_overflow = ((data.alarms And &H1) = &H1)
        output_obj.coulomb_counter_overflow = ((data.alarms And &H2) = &H2)
        output_obj.alarm_threshold = ((data.alarms And &H4) = &H4)

        Return success
    End Function
#End Region

#Region "CLEAR_INT"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure CLEAR_INT_COMMAND_STRUCT
        <FieldOffset(0)> Friend clear_int As Byte
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure CLEAR_INT_RESPONSE_STRUCT
        <FieldOffset(0)> Friend clear_int As Byte
    End Structure

    Friend Function Send_Clear_Int_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(CLEAR_INT_COMMAND_WRITE)) Then
            Dim usb_data As CLEAR_INT_COMMAND_STRUCT
            Dim api_data As CLEAR_INT_COMMAND_WRITE = DirectCast(input_obj, CLEAR_INT_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            If api_data.activate = True Then
                usb_data.clear_int = 1
            Else
                usb_data.clear_int = 0
            End If

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.clear_int))
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Clear_Int_Response(ByVal input_bytes() As Byte, ByRef output_obj As CLEAR_INT_RESPONSE) As Boolean
        Dim data As New CLEAR_INT_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.clear_int)

        ' Put data in response object
        output_obj.activated = (data.clear_int <> 0)

        Return success
    End Function
#End Region

#Region "COUNTER_TEST"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure COUNTER_TEST_COMMAND_STRUCT
        <FieldOffset(0)> Friend enabled As Byte
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure COUNTER_TEST_RESPONSE_STRUCT
        <FieldOffset(0)> Friend enabled As Byte
        <FieldOffset(1)> Friend edge_count As UInt32
        <FieldOffset(5)> Friend time As UInt32
    End Structure

    Friend Function Send_Counter_Test_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(RESET_COMMAND)) Or (input_obj.GetType() Is GetType(REFRESH_COMMAND)) Then

            Dim usb_data As COUNTER_TEST_COMMAND_STRUCT
            usb_data.enabled = 1
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.enabled))

        ElseIf (input_obj.GetType() Is GetType(COUNTER_TEST_COMMAND_WRITE)) Then
            Dim usb_data As COUNTER_TEST_COMMAND_STRUCT
            Dim api_data As COUNTER_TEST_COMMAND_WRITE = DirectCast(input_obj, COUNTER_TEST_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            If api_data.activate = True Then
                usb_data.enabled = 1
            Else
                usb_data.enabled = 0
            End If

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.enabled))
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Counter_Test_Response(ByVal input_bytes() As Byte, ByRef output_obj As COUNTER_TEST_RESPONSE) As Boolean
        Dim data As New COUNTER_TEST_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.enabled)
        success = success And Get_Network_Byte_Order(input_bytes, index, data.edge_count)
        success = success And Get_Network_Byte_Order(input_bytes, index, data.time)

        ' Put data in response object
        output_obj.activated = (data.enabled <> 0)
        output_obj.edge_count = data.edge_count
        output_obj.time = CSng(data.time) / Timestamp.PIC16F_TIMER_TICKS_PER_SEC

        Return success
    End Function
#End Region

#Region "IRQ"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure IRQ_RESPONSE_STRUCT
        <FieldOffset(0)> Friend high As Byte
    End Structure

    Friend Function Send_Irq_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Irq_Response(ByVal input_bytes() As Byte, ByRef output_obj As IRQ_RESPONSE) As Boolean
        Dim data As New IRQ_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.high)

        ' Put data in response object
        output_obj.pin_state = (data.high <> 0)

        Return success
    End Function
#End Region

#Region "PGOOD"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure PGOOD_RESPONSE_STRUCT
        <FieldOffset(0)> Friend high As Byte
    End Structure

    Friend Function Send_Pgood_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Pgood_Response(ByVal input_bytes() As Byte, ByRef output_obj As PGOOD_RESPONSE) As Boolean
        Dim data As New PGOOD_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.high)

        ' Put data in response object
        output_obj.pin_state = (data.high <> 0)

        Return success
    End Function
#End Region

#Region "ADC"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure ADC_COMMAND_STRUCT
        <FieldOffset(0)> Friend adc_select As Byte
        <FieldOffset(1)> Friend voltage As UInt16
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure ADC_RESPONSE_STRUCT
        <FieldOffset(0)> Friend vbat As UInt16
        <FieldOffset(2)> Friend vout As UInt16
    End Structure

    Friend Function Send_Adc_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(ADC_COMMAND_WRITE)) Then
            Dim usb_data As ADC_COMMAND_STRUCT
            Dim api_data As ADC_COMMAND_WRITE = DirectCast(input_obj, ADC_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            If api_data.vbat = True Then
                usb_data.adc_select = 0
            Else
                usb_data.adc_select = 1
            End If
            usb_data.voltage = Convert.ToUInt16(api_data.voltage * 1000)

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, usb_data.adc_select)
            success = success And Put_Network_Byte_Order(output_bytes, index, usb_data.voltage)
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Adc_Response(ByVal input_bytes() As Byte, ByRef output_obj As ADC_RESPONSE) As Boolean
        Dim data As New ADC_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.vbat)
        success = success And Get_Network_Byte_Order(input_bytes, index, data.vout)

        ' Put data in response object

        ' First scale the ADC values into decimal voltages.
        output_obj.vbat = CSng(data.vbat / 1000.0)
        output_obj.vout = CSng(data.vout / 1000.0)

        Return success
    End Function
#End Region

#Region "CAPACITY"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure CAPACITY_COMMAND_STRUCT
        <FieldOffset(0)> Friend capacity As Single
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure CAPACITY_RESPONSE_STRUCT
        <FieldOffset(0)> Friend capacity As Single
    End Structure

    Friend Function Send_Capacity_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(CAPACITY_COMMAND_WRITE)) Then
            Dim usb_data As CAPACITY_COMMAND_STRUCT
            Dim api_data As CAPACITY_COMMAND_WRITE = DirectCast(input_obj, CAPACITY_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            usb_data.capacity = api_data.capacity

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, usb_data.capacity)
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Capacity_Response(ByVal input_bytes() As Byte, ByRef output_obj As CAPACITY_RESPONSE) As Boolean
        Dim data As New CAPACITY_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.capacity)

        ' Put data in response object
        output_obj.capacity = data.capacity

        Return success
    End Function
#End Region

#Region "NONSTANDARD_INDUCTOR"
    <StructLayout(LayoutKind.Sequential, Pack:=1)> _
    Private Structure NONSTANDARD_INDUCTOR_STRUCT
        <MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst:=16)> Friend nonstandard_inductor() As Byte
    End Structure

    Friend Function Send_Nonstandard_Inductor_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(NONSTANDARD_INDUCTOR_COMMAND_WRITE)) Then
            Dim usb_data As NONSTANDARD_INDUCTOR_STRUCT
            Dim api_data As NONSTANDARD_INDUCTOR_COMMAND_WRITE = DirectCast(input_obj, NONSTANDARD_INDUCTOR_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim usb_data.nonstandard_inductor(NONSTANDARD_INDUCTOR_SIZE - 1)
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            usb_data.nonstandard_inductor = Encoding.ASCII.GetBytes(api_data.nonstandard_inductor)

            ' Put packed struct in output buffer
            Array.Copy(usb_data.nonstandard_inductor, output_bytes, usb_data.nonstandard_inductor.Length)

        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_Nonstandard_Inductor_Response(ByVal input_bytes() As Byte, ByRef output_obj As NONSTANDARD_INDUCTOR_RESPONSE) As Boolean
        Dim data As New NONSTANDARD_INDUCTOR_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' VB doesn't drop the 0 elements at the end of the byte array when converting them to a string.  Move them manually.
        Dim length As Integer = 0
        Do While (length < NONSTANDARD_INDUCTOR_SIZE) AndAlso (input_bytes(length) <> 0)
            length += 1
        Loop

        ReDim data.nonstandard_inductor(length - 1)
        Array.Copy(input_bytes, index, data.nonstandard_inductor, 0, length)

        ' Put data in response object
        output_obj.nonstandard_inductor = Encoding.ASCII.GetString(data.nonstandard_inductor)

        Return success
    End Function
#End Region

#Region "EN_CONFIGURATION"
    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure EN_CONFIGURATION_COMMAND_STRUCT
        <FieldOffset(0)> Friend en_setting As Byte
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Private Structure EN_CONFIGURATION_RESPONSE_STRUCT
        <FieldOffset(0)> Friend en_setting As Byte
    End Structure

    Friend Function Send_En_Configuration_Command(ByVal input_obj As Object, ByRef output_bytes() As Byte) As Boolean
        Dim success As Boolean = True
        Dim index As Integer = 0

        If (input_obj.GetType() Is GetType(READ_COMMAND)) Then
            success = True
        ElseIf (input_obj.GetType() Is GetType(EN_CONFIGURATION_COMMAND_WRITE)) Then
            Dim usb_data As EN_CONFIGURATION_COMMAND_STRUCT
            Dim api_data As EN_CONFIGURATION_COMMAND_WRITE = DirectCast(input_obj, EN_CONFIGURATION_COMMAND_WRITE)

            ' Resize Output Buffer for response.
            ReDim output_bytes(SizeOf(usb_data) - 1)

            ' Place data in packed struct
            usb_data.en_setting = CByte(api_data.en_configuration)

            ' Put packed struct in output buffer
            success = success And Put_Network_Byte_Order(output_bytes, index, CByte(usb_data.en_setting))
        Else
            success = False
        End If

        Return success
    End Function

    Friend Function Receive_En_Configuration_Response(ByVal input_bytes() As Byte, ByRef output_obj As EN_CONFIGURATION_RESPONSE) As Boolean
        Dim data As New EN_CONFIGURATION_RESPONSE_STRUCT
        Dim success As Boolean = True
        Dim index As Integer = 0

        ' Get data out of packed struct
        success = success And Get_Network_Byte_Order(input_bytes, index, data.en_setting)

        ' Put data in response object
        output_obj.en_configuration = CType(data.en_setting, LTC3335.EN_CONFIGURATION_OPTIONS)

        Return success
    End Function
#End Region

#Region "Utility Functions"

    Friend Function Get_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As Char) As Boolean
        Try
            value = BitConverter.ToChar(byte_array, position)
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Get_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As SByte) As Boolean
        Try
            If byte_array(position) <= SByte.MaxValue Then
                value = CSByte(byte_array(position))
            Else
                value = CSByte(CInt(byte_array(position)) - (Byte.MaxValue + 1))
            End If
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Get_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As Byte) As Boolean
        Try
            value = byte_array(position)
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Get_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As Int16) As Boolean
        Try
            value = BitConverter.ToInt16(byte_array, position)
            value = IPAddress.NetworkToHostOrder(value)
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Get_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As UInt16) As Boolean
        Try
            ' Note that NetworkToHostOrder not having unsigned methods requires the double conversion of the number.
            Dim temp_value As Int16
            temp_value = BitConverter.ToInt16(byte_array, position)
            temp_value = IPAddress.NetworkToHostOrder(temp_value)
            Dim temp_bytes() As Byte = BitConverter.GetBytes(temp_value)
            value = BitConverter.ToUInt16(temp_bytes, 0)
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Get_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As Int32) As Boolean
        Try
            value = BitConverter.ToInt32(byte_array, position)
            value = IPAddress.NetworkToHostOrder(value)
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Get_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As UInt32) As Boolean
        Try
            ' Note that NetworkToHostOrder not having unsigned methods requires the double conversion of the number.
            Dim temp_value As Int32
            temp_value = BitConverter.ToInt32(byte_array, position)
            temp_value = IPAddress.NetworkToHostOrder(temp_value)
            Dim temp_bytes() As Byte = BitConverter.GetBytes(temp_value)
            value = BitConverter.ToUInt32(temp_bytes, 0)
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Get_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As Single) As Boolean
        Try
            ' Note that NetworkToHostOrder not having floating point methods requires the double conversion of the number.
            Dim temp_value As Int32
            temp_value = BitConverter.ToInt32(byte_array, position)
            temp_value = IPAddress.NetworkToHostOrder(temp_value)
            Dim temp_bytes() As Byte = BitConverter.GetBytes(temp_value)
            value = BitConverter.ToSingle(temp_bytes, 0)
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Put_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As Char) As Boolean
        Try
            byte_array(position) = Convert.ToByte(value)
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Put_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As SByte) As Boolean
        Try
            If value >= Byte.MinValue Then
                byte_array(position) = CByte(value)
            Else
                byte_array(position) = CByte(CInt(value) + (Byte.MaxValue + 1))
            End If
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Put_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As Byte) As Boolean
        Try
            byte_array(position) = value
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Put_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As Int16) As Boolean
        Try
            Dim temp_value As Int16
            temp_value = IPAddress.HostToNetworkOrder(value)
            Dim temp_bytes() As Byte = BitConverter.GetBytes(temp_value)
            Array.Copy(temp_bytes, 0, byte_array, position, SizeOf(value))
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Put_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As UInt16) As Boolean
        Try
            ' Note that HostToNetworkOrder not having unsigned methods requires the double conversion of the number.
            Dim temp_bytes() As Byte = BitConverter.GetBytes(value)
            Dim temp_value As Int16 = BitConverter.ToInt16(temp_bytes, 0)
            temp_value = IPAddress.HostToNetworkOrder(temp_value)
            temp_bytes = BitConverter.GetBytes(temp_value)
            Array.Copy(temp_bytes, 0, byte_array, position, SizeOf(value))
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Put_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As Int32) As Boolean
        Try
            Dim temp_value As Int32
            temp_value = IPAddress.HostToNetworkOrder(value)
            Dim temp_bytes() As Byte = BitConverter.GetBytes(temp_value)
            Array.Copy(temp_bytes, 0, byte_array, position, SizeOf(value))
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Put_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As UInt32) As Boolean
        Try
            ' Note that HostToNetworkOrder not having unsigned methods requires the double conversion of the number.
            Dim temp_bytes() As Byte = BitConverter.GetBytes(value)
            Dim temp_value As Int32 = BitConverter.ToInt32(temp_bytes, 0)
            temp_value = IPAddress.HostToNetworkOrder(temp_value)
            temp_bytes = BitConverter.GetBytes(temp_value)
            Array.Copy(temp_bytes, 0, byte_array, position, SizeOf(value))
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Friend Function Put_Network_Byte_Order(ByRef byte_array() As Byte, ByRef position As Integer, ByRef value As Single) As Boolean
        Try
            ' Note that HostToNetworkOrder not having floating point methods requires the double conversion of the number.
            Dim temp_bytes() As Byte = BitConverter.GetBytes(value)
            Dim temp_value As Int32 = BitConverter.ToInt32(temp_bytes, 0)
            temp_value = IPAddress.HostToNetworkOrder(temp_value)
            temp_bytes = BitConverter.GetBytes(temp_value)
            Array.Copy(temp_bytes, 0, byte_array, position, SizeOf(value))
            position += SizeOf(value)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

#End Region

End Class
