﻿Option Strict On
Option Explicit On

Imports Microsoft.VisualBasic
Imports System.Windows.Forms
Imports System.Drawing
Imports Microsoft.VisualBasic.PowerPacks

Public Class Vout_Selection

    Private usb_comm As D3335_USB
    Friend selection As LTC3335.VOLTAGE_SELECTION_OPTIONS

    Private Const NUM_OUT_BITS As Integer = 3

    Public Class vout_option
        Private Const SELECTION_WIDTH As Integer = 1

        Private out_labels(NUM_OUT_BITS) As Label
        Private selection_label As Label
        Private _selected As Boolean = False
        Private _originally_selected As Boolean
        Private option_list() As vout_option
        Private hardware_warning_label As Label

        Public Sub New(ByVal out2_label As Label, ByVal out1_label As Label, ByVal out0_label As Label, ByVal selection_label As Label, ByRef option_list() As vout_option, ByVal hardware_warning_label As Label)
            Me.out_labels(0) = out2_label
            Me.out_labels(1) = out1_label
            Me.out_labels(2) = out0_label
            Me.selection_label = selection_label
            Me.option_list = option_list
            Me.hardware_warning_label = hardware_warning_label
            Me.selected = False
            Me.originally_selected = False
        End Sub

        ' Create properties so that Forms can read values but not write them.
        Public Property selected() As Boolean
            Get
                Return _selected
            End Get

            Set(ByVal value As Boolean)
                _selected = value
                If _selected = True Then
                    For out_bit_num As Integer = 0 To NUM_OUT_BITS - 1
                        Me.out_labels(out_bit_num).Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Bold)
                        D3335.Set_Font(DirectCast(Me.out_labels(out_bit_num), Control))
                    Next out_bit_num

                    Me.selection_label.Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Bold)
                    D3335.Set_Font(DirectCast(Me.selection_label, Control))

                    ' Add handlers for painting the box around the selection.
                    AddHandler Me.out_labels(0).Paint, AddressOf OUT2_Label_Paint
                    AddHandler Me.out_labels(1).Paint, AddressOf OUT1_Label_Paint
                    AddHandler Me.out_labels(2).Paint, AddressOf OUT0_Label_Paint
                    AddHandler Me.selection_label.Paint, AddressOf Selection_Label_Paint
                    AddHandler Me.selection_label.Parent.Paint, AddressOf VOUT_TableLayoutPanel_Paint

                    ' Remove handlers for clicking now that this is already selected.
                    RemoveHandler Me.out_labels(0).Click, AddressOf Label_Click
                    RemoveHandler Me.out_labels(1).Click, AddressOf Label_Click
                    RemoveHandler Me.out_labels(2).Click, AddressOf Label_Click
                    RemoveHandler Me.selection_label.Click, AddressOf Label_Click

                Else
                    For out_bit_num As Integer = 0 To NUM_OUT_BITS - 1
                        Me.out_labels(out_bit_num).Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Regular)
                        D3335.Set_Font(DirectCast(Me.out_labels(out_bit_num), Control))
                    Next out_bit_num

                    Me.selection_label.Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Regular)
                    D3335.Set_Font(DirectCast(Me.selection_label, Control))

                    ' Remove handlers for painting the box around the selection.
                    RemoveHandler Me.out_labels(0).Paint, AddressOf OUT2_Label_Paint
                    RemoveHandler Me.out_labels(1).Paint, AddressOf OUT1_Label_Paint
                    RemoveHandler Me.out_labels(2).Paint, AddressOf OUT0_Label_Paint
                    RemoveHandler Me.selection_label.Paint, AddressOf Selection_Label_Paint
                    RemoveHandler Me.selection_label.Parent.Paint, AddressOf VOUT_TableLayoutPanel_Paint

                    ' Add handlers for clicking now so that this option can be selected.
                    AddHandler Me.out_labels(0).Click, AddressOf Label_Click
                    AddHandler Me.out_labels(1).Click, AddressOf Label_Click
                    AddHandler Me.out_labels(2).Click, AddressOf Label_Click
                    AddHandler Me.selection_label.Click, AddressOf Label_Click

                End If
            End Set

        End Property

        Public Property originally_selected() As Boolean
            Get
                Return _originally_selected
            End Get

            Set(ByVal value As Boolean)
                _originally_selected = value
            End Set

        End Property

        Private Sub OUT2_Label_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs)

            If Me.selected = True Then
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(SELECTION_WIDTH - 1, SELECTION_WIDTH - 1), _
                                    New Point(SELECTION_WIDTH - 1, out_labels(0).Height - SELECTION_WIDTH))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(SELECTION_WIDTH - 1, SELECTION_WIDTH - 1), _
                                    New Point(out_labels(0).Width, SELECTION_WIDTH - 1))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(SELECTION_WIDTH - 1, out_labels(0).Height - SELECTION_WIDTH), _
                                    New Point(out_labels(0).Width, out_labels(0).Height - SELECTION_WIDTH))
            End If

        End Sub

        Private Sub OUT1_Label_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs)
            If Me.selected = True Then
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(0, SELECTION_WIDTH - 1), _
                                    New Point(out_labels(1).Width, SELECTION_WIDTH - 1))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(0, out_labels(1).Height - SELECTION_WIDTH), _
                                    New Point(out_labels(1).Width, out_labels(1).Height - SELECTION_WIDTH))
            End If

        End Sub

        Private Sub OUT0_Label_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs)
            If Me.selected = True Then
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(0, SELECTION_WIDTH - 1), _
                                    New Point(out_labels(2).Width, SELECTION_WIDTH - 1))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(0, out_labels(2).Height - SELECTION_WIDTH), _
                                    New Point(out_labels(2).Width, out_labels(2).Height - SELECTION_WIDTH))
            End If
        End Sub

        Private Sub Selection_Label_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs)

            If Me.selected = True Then
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(0, SELECTION_WIDTH - 1), _
                                    New Point(selection_label.Width - SELECTION_WIDTH, SELECTION_WIDTH - 1))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(0, selection_label.Height - SELECTION_WIDTH), _
                                    New Point(selection_label.Width - SELECTION_WIDTH, selection_label.Height - SELECTION_WIDTH))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(selection_label.Width - SELECTION_WIDTH, SELECTION_WIDTH - 1), _
                                    New Point(selection_label.Width - SELECTION_WIDTH, selection_label.Height - SELECTION_WIDTH))
            End If
        End Sub

        Private Sub VOUT_TableLayoutPanel_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs)

            If Me.selected = True Then
                ' Draw line to screen.
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(out_labels(0).Location.X, out_labels(0).Location.Y + SELECTION_WIDTH - 1),
                                    New Point(selection_label.Location.X + selection_label.Width - SELECTION_WIDTH, selection_label.Location.Y + SELECTION_WIDTH - 1))
                e.Graphics.DrawLine(New Pen(Color.Black, SELECTION_WIDTH), _
                                    New Point(out_labels(0).Location.X, out_labels(0).Location.Y + out_labels(0).Height - SELECTION_WIDTH), _
                                    New Point(selection_label.Location.X + selection_label.Width - SELECTION_WIDTH, selection_label.Location.Y + selection_label.Height - SELECTION_WIDTH - 1))
            End If
        End Sub

        Private Sub Label_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)

            ' First unselected all option that's already selected
            For Each other_option As vout_option In option_list
                If other_option.selected = True Then
                    other_option.selected = False
                End If
            Next other_option

            ' Now selected this option
            Me.selected = True

            ' Show warning that jumpers must be set.
            Me.hardware_warning_label.Visible = Not Me.originally_selected
        End Sub

    End Class

    Public vout_options(LTC3335.VOLTAGE_SELECTION_OPTIONS.NUM_VOLTAGE_SELECTION_OPTIONS - 2) As vout_option

    ' Popup window should always be on top of GUI Control Panel, but not always on Top of all Windows programs.
    Private Declare Function BringWindowToTop Lib "user32" Alias "BringWindowToTop" (ByVal hwnd As Long) As Long
    Private Sub Vout_Selection_Activated(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        BringWindowToTop(CLng(Me.Handle))
    End Sub

    Friend Overloads Sub Show(ByVal selection As LTC3335.VOLTAGE_SELECTION_OPTIONS)
        For out_option As LTC3335.VOLTAGE_SELECTION_OPTIONS = LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_1_8V To LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_5_0V
            vout_options(out_option - 1).originally_selected = If(selection = out_option, True, False) ' subtract 1 as HARDWARE_CONTROL is not an option for the jumpers.
        Next out_option

        ' Start with warning that jumpers must be set hidden.
        Me.Hardware_Warning_Label.Visible = False

        Me.Show()
    End Sub

    Friend Sub New(ByVal usb_comm As D3335_USB)

        InitializeComponent()

        Me.usb_comm = usb_comm

        ' Set the Use Compatible Text Rendering property.
        For Each control As Control In MyBase.Controls
            D3335.Set_UseCompatibleTextRendering(control)
        Next control

        ' Next set all of the fonts to Linear Helv Cond.
        For Each control As Control In MyBase.Controls
            D3335.Set_Font(control)
        Next control

        ' Turn off all of the tabstops on this page.
        For Each control As Control In MyBase.Controls
            D3335.Set_TabStops(control, False)
        Next control

        ' Set up all of the vout option objects.
        vout_options(LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_1_8V - 1) = New vout_option(OUT2_1_8V_Label, OUT1_1_8V_Label, OUT0_1_8V_Label, Selection_1_8V_Label, vout_options, Hardware_Warning_Label)
        vout_options(LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_2_5V - 1) = New vout_option(OUT2_2_5V_Label, OUT1_2_5V_Label, OUT0_2_5V_Label, Selection_2_5V_Label, vout_options, Hardware_Warning_Label)
        vout_options(LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_2_8V - 1) = New vout_option(OUT2_2_8V_Label, OUT1_2_8V_Label, OUT0_2_8V_Label, Selection_2_8V_Label, vout_options, Hardware_Warning_Label)
        vout_options(LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_3_0V - 1) = New vout_option(OUT2_3_0V_Label, OUT1_3_0V_Label, OUT0_3_0V_Label, Selection_3_0V_Label, vout_options, Hardware_Warning_Label)
        vout_options(LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_3_3V - 1) = New vout_option(OUT2_3_3V_Label, OUT1_3_3V_Label, OUT0_3_3V_Label, Selection_3_3V_Label, vout_options, Hardware_Warning_Label)
        vout_options(LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_3_6V - 1) = New vout_option(OUT2_3_6V_Label, OUT1_3_6V_Label, OUT0_3_6V_Label, Selection_3_6V_Label, vout_options, Hardware_Warning_Label)
        vout_options(LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_4_5V - 1) = New vout_option(OUT2_4_5V_Label, OUT1_4_5V_Label, OUT0_4_5V_Label, Selection_4_5V_Label, vout_options, Hardware_Warning_Label)
        vout_options(LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_5_0V - 1) = New vout_option(OUT2_5_0V_Label, OUT1_5_0V_Label, OUT0_5_0V_Label, Selection_5_0V_Label, vout_options, Hardware_Warning_Label)

        ' The 3.3V setting with EN on is the default from the factory.
        Set_Selection(LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_3_3V)

    End Sub

    Friend Sub Set_Selection(ByVal selection As LTC3335.VOLTAGE_SELECTION_OPTIONS)
        Me.selection = selection
        Set_Selection_GUI(selection)
    End Sub

    Friend Sub Set_Selection_GUI(ByVal selection As LTC3335.VOLTAGE_SELECTION_OPTIONS)
        For out_option As LTC3335.VOLTAGE_SELECTION_OPTIONS = LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_1_8V To LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_5_0V
            vout_options(out_option - 1).selected = If(selection = out_option, True, False) ' subtract 1 as HARDWARE_CONTROL is not an option for the jumpers.
        Next out_option
    End Sub

    Friend Sub OK_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OK_Button.Click

        ' Set VOUT selection to that specified by this popup.
        Me.selection = LTC3335.VOLTAGE_SELECTION_OPTIONS.SOFTWARE_CONTROL_1_8V
        For Each this_option As vout_option In vout_options
            If this_option.selected = True Then
                Exit For
            Else
                Me.selection = CType(Me.selection + 1, LTC3335.VOLTAGE_SELECTION_OPTIONS)
            End If
        Next this_option

        ' Send command to FW to set the VOUT selection.
        Dim data As New D3335_USB.VOUT_CONFIGURATION_COMMAND_WRITE
        data.vout_configuration = Me.selection
        Me.usb_comm.SendCommand(D3335_USB.USB_DATA_ID.VOUT_CONFIGURATION, data)

        ' Close the popup
        Me.Hide()
    End Sub

    Friend Sub Cancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cancel_Button.Click
        ' Revert this popup to show the VOUT selection before the user started to change controls.
        Me.Set_Selection_GUI(Me.selection)

        ' Close the popup
        Me.Hide()
    End Sub

    Protected Overrides Sub OnFormClosing(ByVal e As System.Windows.Forms.FormClosingEventArgs)
        Cancel_Button_Click(Cancel_Button, New EventArgs)
        e.Cancel = True
        MyBase.OnFormClosing(e)
    End Sub

End Class
